@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2003 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,11 +16,10 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 4592 $
+ *  $Revision: 6186 $
  */
 package org.jajuk.base;
 
-import java.awt.Container;
 import java.awt.MediaTracker;
 import java.awt.image.BufferedImage;
 import java.io.File;
@@ -28,53 +28,60 @@ import java.util.ArrayList;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Locale;
 import java.util.Set;
-import java.util.regex.PatternSyntaxException;
 
 import javax.imageio.ImageIO;
 import javax.swing.ImageIcon;
 
+import org.apache.commons.lang.StringUtils;
 import org.jajuk.ui.thumbnails.ThumbnailManager;
 import org.jajuk.util.Const;
 import org.jajuk.util.IconLoader;
+import org.jajuk.util.JajukFileFilter;
 import org.jajuk.util.JajukIcons;
 import org.jajuk.util.Messages;
 import org.jajuk.util.UtilFeatures;
 import org.jajuk.util.UtilString;
-import org.jajuk.util.UtilSystem;
+import org.jajuk.util.filters.ImageFilter;
 import org.jajuk.util.log.Log;
 
 /**
  * An Album *
  * <p>
- * Logical item
+ * Logical item.
  */
 public class Album extends LogicalItem implements Comparable<Album> {
 
-  /**
-   * For perfs, we cache the associated tracks. This cache is filled by the
-   * TrackManager using the getTracksCache() method
-   */
-  private List<Track> cache = new ArrayList<Track>(15);
+  /** For perfs, we cache the associated tracks. This cache is filled by the TrackManager using the getTracksCache() method */
+  private final List<Track> cache = new ArrayList<Track>(15);
 
-  /**
-   * This array stores thumbnail presence for all the available size
-   * (performance) By default all booleans are false
-   */
+  /** This array stores thumbnail presence for all the available size (performance) By default all booleans are false. */
   private boolean[] availableTumbs;
 
   /**
-   * Album constructor
+   * Album constructor.
    * 
-   * @param id
-   * @param sName
+   * @param sName DOCUMENT_ME
+   * @param sId DOCUMENT_ME
+   * @param discID DOCUMENT_ME
    */
-  public Album(String sId, String sName) {
+  Album(String sId, String sName, long discID) {
     super(sId, sName);
+    setProperty(Const.XML_ALBUM_DISC_ID, discID);
   }
 
   /**
-   * Return album name, dealing with unknown for any language
+   * Gets the disc id.
+   * 
+   * @return the discID
+   */
+  public long getDiscID() {
+    return getLongValue(Const.XML_ALBUM_DISC_ID);
+  }
+
+  /**
+   * Return album name, dealing with unknown for any language.
    * 
    * @return album name
    */
@@ -87,21 +94,31 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
-   * toString method
+   * toString method.
+   * 
+   * @return the string
    */
   @Override
   public String toString() {
-    return "Album[ID=" + getID() + " Name={{" + getName() + "}}]";
+    return "Album[ID=" + getID() + " Name={{" + getName() + "}}" + " disk ID={{" + getDiscID()
+        + "}}]";
   }
 
   /**
-   * Alphabetical comparator used to display ordered lists
+   * Alphabetical comparator on the name
+   * <p>
+   * Used to display ordered lists.
+   * 
+   * @param otherAlbum DOCUMENT_ME
    * 
-   * @param other
-   *          item to be compared
    * @return comparison result
    */
+  @Override
   public int compareTo(Album otherAlbum) {
+    if (otherAlbum == null) {
+      return -1;
+    }
+
     // compare using name and id to differentiate unknown items
     StringBuilder current = new StringBuilder(getName2());
     current.append(getID());
@@ -111,6 +128,8 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Checks if is unknown.
+   * 
    * @return whether the album is Unknown or not
    */
   public boolean isUnknown() {
@@ -128,7 +147,9 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
-   * Get item description
+   * Get item description.
+   * 
+   * @return the desc
    */
   @Override
   public String getDesc() {
@@ -144,12 +165,12 @@ public class Album extends LogicalItem implements Comparable<Album> {
   public String getHumanValue(String sKey) {
     // We compute here all pseudo keys (non album real attributes) that can be
     // required on an album
-    if (Const.XML_AUTHOR.equals(sKey)) {
-      return handleAuthor();
+    if (Const.XML_ARTIST.equals(sKey)) {
+      return handleArtist();
     } else if (Const.XML_ALBUM.equals(sKey)) {
       return getName2();
-    } else if (Const.XML_STYLE.equals(sKey)) {
-      return handleStyle();
+    } else if (Const.XML_GENRE.equals(sKey)) {
+      return handleGenre();
     } else if (Const.XML_YEAR.equals(sKey)) {
       return handleYear();
     } else if (Const.XML_TRACK_RATE.equals(sKey)) {
@@ -170,33 +191,42 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
-   * @return
+   * Handle artist.
+   * DOCUMENT_ME
+   * 
+   * @return the string
    */
-  private String handleAuthor() {
-    Author author = getAuthor();
-    if (author != null) {
-      return author.getName2();
+  private String handleArtist() {
+    Artist artist = getArtist();
+    if (artist != null) {
+      return artist.getName2();
     } else {
-      // More than one author, display void string
+      // More than one artist, display void string
       return "";
     }
   }
 
   /**
-   * @return
+   * Handle genre.
+   * DOCUMENT_ME
+   * 
+   * @return the string
    */
-  private String handleStyle() {
-    Style style = getStyle();
-    if (style != null) {
-      return style.getName2();
+  private String handleGenre() {
+    Genre genre = getGenre();
+    if (genre != null) {
+      return genre.getName2();
     } else {
-      // More than one style, display void string
+      // More than one genre, display void string
       return "";
     }
   }
 
   /**
-   * @return
+   * Handle year.
+   * DOCUMENT_ME
+   * 
+   * @return the string
    */
   private String handleYear() {
     Year year = getYear();
@@ -208,6 +238,8 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the any.
+   * 
    * @return a human representation of all concatenated properties
    */
   @Override
@@ -216,13 +248,24 @@ public class Album extends LogicalItem implements Comparable<Album> {
     StringBuilder sb = new StringBuilder(100);
     sb.append(super.getAny()); // add all album-based properties
     // now add others properties
-    Author author = getAuthor();
-    if (author != null) {
-      sb.append(author.getName2());
+    Artist artist = getArtist();
+    if (artist != null) {
+      sb.append(artist.getName2());
     }
-    Style style = getStyle();
-    if (style != null) {
-      sb.append(style.getName2());
+    // Try to add album artist
+    Track first = null;
+    List<Track> cache = getTracksCache();
+    synchronized (cache) {
+      first = cache.get(0);
+    }
+    // (every track maps at minimum an "unknown artist" album artist
+    if (first.getAlbumArtist() != null) {
+      sb.append(first.getAlbumArtist().getName2());
+    }
+
+    Genre genre = getGenre();
+    if (genre != null) {
+      sb.append(genre.getName2());
     }
     Year year = getYear();
     if (year != null) {
@@ -237,114 +280,108 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the cover file.
+   * <p>Can be a long action</p>
+   * 
    * @return associated best cover file available or null if none. The returned
-   *         file can not be readable, so use a try/catch around a future access
+   * file can not be readable, so use a try/catch around a future access to it.
    */
-  public File getCoverFile() {
+  public File findCoverFile() {
     String cachedCoverPath = getStringValue(XML_ALBUM_COVER);
-    // If none cover is found, we save this information to save discovery time
-    // afterwards (performance factor x2 or x3 in catalog view)
-    if ("none".equals(cachedCoverPath)) {
+    if (COVER_NONE.equals(cachedCoverPath)) {
       return null;
-    } else if (!UtilString.isVoid(cachedCoverPath)) {
-      File coverCache = new File(cachedCoverPath);
-      // if (coverCache.exists()) {
-      return coverCache;
-      // }
+    } else if (!StringUtils.isBlank(cachedCoverPath)) {
+      // Check if cover still exist. There is an overhead
+      // drawback but otherwise, the album's cover
+      // property may be stuck to an old device's cover url.
+      Device device = DeviceManager.getInstance().getDeviceByPath(new File(cachedCoverPath));
+      // If the device is not mounted, do not perform this existence check up
+      if (device != null && device.isMounted()) {
+        if (new File(cachedCoverPath).exists()) {
+          return new File(cachedCoverPath);
+        }
+      } else {
+        return new File(cachedCoverPath);
+      }
     }
-    File fDir = null; // analyzed directory
     // search for local covers in all directories mapping the current track
     // to reach other devices covers and display them together
-    List<Track> lTracks = TrackManager.getInstance().getAssociatedTracks(this, false);
+    List<Track> lTracks = cache;
     if (lTracks.size() == 0) {
-      setProperty(XML_ALBUM_COVER, "none");
+      setProperty(XML_ALBUM_COVER, COVER_NONE);
       return null;
     }
-    // List if directories we have to look in
+    // List at directories we have to look in
     Set<Directory> dirs = new HashSet<Directory>(2);
     for (Track track : lTracks) {
-      for (org.jajuk.base.File file : track.getFiles()) {
-        if (file.isReady()) {
-          // note that hashset ensures directory unicity
-          dirs.add(file.getDirectory());
-        }
+      for (org.jajuk.base.File file : track.getReadyFiles()) {
+        // note that hashset ensures directory unicity
+        dirs.add(file.getDirectory());
       }
     }
-    // look for absolute cover in collection
-    for (Directory dir : dirs) {
-      String sAbsolut = dir.getStringValue(Const.XML_DIRECTORY_DEFAULT_COVER);
-      if (!UtilString.isVoid(sAbsolut.trim())) {
-        File fAbsoluteDefault = new File(dir.getAbsolutePath() + '/' + sAbsolut);
-        if (fAbsoluteDefault.exists()) {
-          setProperty(XML_ALBUM_COVER, fAbsoluteDefault.getAbsolutePath());
-          return fAbsoluteDefault;
-        }
-      }
+    // If none available dir, we can't search for cover for now (may be better
+    // next time when at least one device will be mounted)
+    if (dirs.size() == 0) {
+      return null;
     }
 
     // look for standard cover in collection
-    for (Directory dir : dirs) {
-      fDir = dir.getFio(); // store this dir
-      java.io.File[] files = fDir.listFiles();// null if none file
-      // found
-      for (int i = 0; files != null && i < files.length; i++) {
-        // test file exists, do not use the File.canRead() method: it can be
-        // very costly when using a NAS under Windows
-        if (files[i].exists() && files[i].length() < MAX_COVER_SIZE * 1024) {
-          // check size to avoid out of memory errors
-          String sExt = UtilSystem.getExtension(files[i]);
-          if (sExt.equalsIgnoreCase("jpg") || sExt.equalsIgnoreCase("png")
-              || sExt.equalsIgnoreCase("gif")) {
-            if (UtilFeatures.isStandardCover(files[i])) {
-              // Test the image is not corrupted
-              try {
-                MediaTracker mediaTracker = new MediaTracker(new Container());
-                ImageIcon ii = new ImageIcon(files[i].getAbsolutePath());
-                mediaTracker.addImage(ii.getImage(), 0);
-                mediaTracker.waitForID(0); // wait for image
-                if (!mediaTracker.isErrorAny()) {
-                  setProperty(XML_ALBUM_COVER, files[i].getAbsolutePath());
-                  return files[i];
-                }
-              } catch (Exception e) {
-                Log.error(e);
-              }
-            }
-          }
-        }
-      }
-    }
+    File cover = findCover(dirs, true);
+
     // none ? OK, return first cover file we find
+    if (cover == null) {
+      cover = findCover(dirs, false);
+    }
+
+    // Still nothing ? ok, set no cover
+    if (cover == null) {
+      setProperty(XML_ALBUM_COVER, COVER_NONE);
+    }
+    return cover;
+  }
+
+  /**
+   * Return a cover file matching criteria or null.
+   * 
+   * @param dirs : list of directories to search in
+   * @param onlyStandardCovers to we consider only standard covers ?
+   * 
+   * @return a cover file matching criteria or null
+   */
+  private File findCover(Set<Directory> dirs, boolean onlyStandardCovers) {
+    JajukFileFilter filter = new JajukFileFilter(ImageFilter.getInstance());
     for (Directory dir : dirs) {
-      fDir = dir.getFio(); // store this dir
+      File fDir = dir.getFio(); // store this dir
       java.io.File[] files = fDir.listFiles();// null if none file
       // found
       for (int i = 0; files != null && i < files.length; i++) {
-        // test file exists, do not use the File.canRead() method: it can be
-        // very costly when using a NAS under Windows
-        if (files[i].exists() && files[i].length() < MAX_COVER_SIZE * 1024) {
-          // check size to avoid out of memory errors
-          String sExt = UtilSystem.getExtension(files[i]);
-          if (sExt.equalsIgnoreCase("jpg") || sExt.equalsIgnoreCase("png")
-              || sExt.equalsIgnoreCase("gif")) {
-            // Test the image is not corrupted
-            try {
-              MediaTracker mediaTracker = new MediaTracker(new Container());
-              ImageIcon ii = new ImageIcon(files[i].getAbsolutePath());
-              mediaTracker.addImage(ii.getImage(), 0);
-              mediaTracker.waitForID(0); // wait for image
-              if (!mediaTracker.isErrorAny()) {
-                setProperty(XML_ALBUM_COVER, files[i].getAbsolutePath());
-                return files[i];
-              }
-            } catch (Exception e) {
-              Log.error(e);
+        if (files[i].exists()
+        // check size to avoid out of memory errors
+            && files[i].length() < MAX_COVER_SIZE * 1024
+            // Is it an image ?
+            && filter.accept(files[i])) {
+          // Filter standard view if required
+          if (onlyStandardCovers && !UtilFeatures.isStandardCover(files[i])) {
+            continue;
+          }
+
+          // Test the image is not corrupted
+          try {
+            ImageIcon ii = new ImageIcon(files[i].getAbsolutePath());
+            // Note that at this point, the image is fully loaded (done in the ImageIcon
+            // constructor)
+            if (ii.getImageLoadStatus() == MediaTracker.COMPLETE) {
+              setProperty(XML_ALBUM_COVER, files[i].getAbsolutePath());
+              return files[i];
+            } else {
+              Log.debug("Problem loading: " + files[i].getAbsolutePath());
             }
+          } catch (Exception e) {
+            Log.error(e);
           }
         }
       }
     }
-    setProperty(XML_ALBUM_COVER, "none");
     return null;
   }
 
@@ -359,6 +396,8 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the rate.
+   * 
    * @return album rating
    */
   @Override
@@ -371,9 +410,10 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the thumbnail.
+   * 
+   * @param size size using format width x height
    * 
-   * @param size
-   *          size using format width x height
    * @return album thumb for given size
    */
   public ImageIcon getThumbnail(int size) {
@@ -388,61 +428,101 @@ public class Album extends LogicalItem implements Comparable<Album> {
     } catch (IOException e) {
       Log.error(e);
     }
+    // can be null now if an error occurred, we reported a error to the log
+    // already...
+    if (img == null) {
+      return null;
+    }
+
     ImageIcon icon = new ImageIcon(img);
     // Free thumb memory (DO IT AFTER FULL ImageIcon loading)
     img.flush();
-    // accelerate GC cleanup
-    img = null;
+
     return icon;
   }
 
   /**
+   * Gets the genre.
    * 
-   * @return style for the album. Return null if the album contains tracks with
-   *         different styles
+   * @return genre for the album. Return null if the album contains tracks with
+   * different genres
    */
-  public Style getStyle() {
-    Set<Style> styles = new HashSet<Style>(1);
+  public Genre getGenre() {
+    Set<Genre> genres = new HashSet<Genre>(1);
     for (Track track : cache) {
-      styles.add(track.getStyle());
+      genres.add(track.getGenre());
     }
-    // If different styles, the album style is null
-    if (styles.size() == 1) {
-      return styles.iterator().next();
+    // If different genres, the album genre is null
+    if (genres.size() == 1) {
+      return genres.iterator().next();
     } else {
       return null;
     }
   }
 
   /**
+   * Gets the artist.
    * 
-   * @return author for the album. Return null if the album contains tracks with
-   *         different authors
+   * @return artist for the album. <br>
+   * Return null if the album contains tracks with different artists
    */
-  public Author getAuthor() {
-    Set<Author> authors = new HashSet<Author>(1);
-    for (Track track : cache) {
-      authors.add(track.getAuthor());
-    }
-    // If different Authors, the album Author is null
-    if (authors.size() == 1) {
-      return authors.iterator().next();
-    } else {
+  public Artist getArtist() {
+    if (cache.size() == 0) {
       return null;
     }
+    Artist first = cache.get(0).getArtist();
+    for (Track track : cache) {
+      if (!track.getArtist().equals(first)) {
+        return null;
+      }
+    }
+    return first;
+  }
+
+  /**
+   * Gets the artist or the album artist if not available
+   * 
+   * <u>Used algorithm is following :
+   * <li>If none available tags : return "unknown artist"</li>
+   * <li>If the album contains tracks with different artists, display the first album artist found if any</li>
+   * <li>In this case, if no album artist is available, display the first artist found</li>
+   * </u>.
+   * 
+   * @return artist for the album. <br>
+   * Return Always an artist, eventually a "Unknown Artist" one
+   */
+  public String getArtistOrALbumArtist() {
+    // no track => no artist
+    if (cache.size() == 0) {
+      return Const.UNKNOWN_ARTIST;
+    }
+
+    Artist artist = getArtist();
+    if (artist != null && !artist.isUnknown()) {
+      return artist.getName();
+    } else {
+      Track first = cache.get(0);
+      AlbumArtist albumArtist = first.getAlbumArtist();
+      if (!albumArtist.isUnknown()) {
+        return albumArtist.getName();
+      } else {
+        return first.getArtist().getName();
+      }
+    }
   }
 
   /**
+   * Gets the year.
    * 
    * @return year for the album. Return null if the album contains tracks with
-   *         different years
+   * different years
    */
   public Year getYear() {
     Set<Year> years = new HashSet<Year>(1);
     for (Track track : cache) {
       years.add(track.getYear());
     }
-    // If different Authors, the album Author is null
+    // If different Artists, the album Artist is null
     if (years.size() == 1) {
       return years.iterator().next();
     } else {
@@ -451,7 +531,9 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
-   * Return full album length in secs
+   * Return full album length in secs.
+   * 
+   * @return the duration
    */
   public long getDuration() {
     long length = 0;
@@ -462,6 +544,8 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the nb of tracks.
+   * 
    * @return album nb of tracks
    */
   public int getNbOfTracks() {
@@ -469,6 +553,8 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the hits.
+   * 
    * @return album total nb of hits
    */
   public long getHits() {
@@ -480,6 +566,8 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Contains ready files.
+   * 
    * @return whether the album contains a least one available track
    */
   public boolean containsReadyFiles() {
@@ -492,6 +580,8 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the discovery date.
+   * 
    * @return First found track discovery date
    */
   public Date getDiscoveryDate() {
@@ -502,45 +592,53 @@ public class Album extends LogicalItem implements Comparable<Album> {
     }
   }
 
+  /**
+   * Returns true, if the pattern matches the specified property.
+   * 
+   * Currently only Const.XML_ALBUM and Const.XML_GENRE are supported
+   * properties. The pattern is used for a case-insensitive sub-string match,
+   * no regular expression is used!
+   * 
+   * @param property The property to use for the match, currently either Cosnt.XML_ALBUM
+   * or Const.XML_GENRE
+   * @param pattern The string to search for as case-insensitive sub-string
+   * 
+   * @return true if either parameter is null or if the pattern matches, false otherwise.
+   */
   public boolean matches(String property, String pattern) {
-    if (UtilString.isVoid(property) || UtilString.isVoid(pattern)) {
+    if (StringUtils.isBlank(property) || StringUtils.isBlank(pattern)) {
       return true;
     }
+
     String sValue = null;
     if (Const.XML_ALBUM.equals(property)) {
       sValue = getName2();
-    } else if (Const.XML_STYLE.equals(property)) {
-      Style style = getStyle();
-      if (style == null) {
+    } else if (Const.XML_GENRE.equals(property)) {
+      Genre genre = getGenre();
+      if (genre == null) {
         return false;
       }
-      sValue = style.getName2();
+      sValue = genre.getName2();
     }
     if (sValue == null) {
       return false;
     }
-    boolean match = false;
-    try {
-      // do not use regexp matches(<string>) because the string may contain
-      // characters to be escaped
-      match = (sValue.toLowerCase().indexOf(pattern.toLowerCase()) != -1);
-      // test if the item property contains this
-      // property value (ignore case)
-    } catch (PatternSyntaxException pse) {
-      // wrong pattern syntax
-      Log.error(pse);
-    }
-    return match;
+
+    // do not use regexp matches(<string>) because the string may contain
+    // characters to be escaped
+    return (sValue.toLowerCase(Locale.getDefault()).indexOf(
+        pattern.toLowerCase(Locale.getDefault())) != -1);
   }
 
   /**
-   * Reset tracks cache
+   * Reset tracks cache.
    */
   protected void resetTracks() {
     cache.clear();
   }
 
   /**
+   * Gets the tracks cache.
    * 
    * @return ordered tracks cache for this album (perf)
    */
@@ -549,6 +647,7 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
+   * Gets the any track.
    * 
    * @return a track from this album
    */
@@ -561,11 +660,10 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
-   * Set that the thumb for given size is available
+   * Set that the thumb for given size is available.
    * 
-   * @param size
-   *          (thumb size like 50)
-   * @param available
+   * @param size (thumb size like 50)
+   * @param available DOCUMENT_ME
    */
   public void setAvailableThumb(int size, boolean available) {
     if (availableTumbs == null) {
@@ -575,10 +673,10 @@ public class Album extends LogicalItem implements Comparable<Album> {
   }
 
   /**
-   * Return whether a thumb is available for given size
+   * Return whether a thumb is available for given size.
+   * 
+   * @param size (thumb size like 50)
    * 
-   * @param size
-   *          (thumb size like 50)
    * @return whether a thumb is available for given size
    */
   public boolean isThumbAvailable(int size) {
