@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2003 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 4517 $
+ *  $Revision: 6186 $
  */
 
 package org.jajuk.base;
@@ -27,6 +28,7 @@ import java.util.Date;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
 
@@ -34,6 +36,7 @@ import org.apache.commons.collections.CollectionUtils;
 import org.jajuk.events.JajukEvent;
 import org.jajuk.events.JajukEvents;
 import org.jajuk.events.ObservationManager;
+import org.jajuk.services.players.QueueModel;
 import org.jajuk.util.Conf;
 import org.jajuk.util.Const;
 import org.jajuk.util.MD5Processor;
@@ -46,17 +49,18 @@ import org.jajuk.util.filters.JajukPredicates;
 import org.jajuk.util.log.Log;
 
 /**
- * Convenient class to manage files
+ * Convenient class to manage files.
  */
 public final class FileManager extends ItemManager {
-  /** Best of files */
-  private List<File> alBestofFiles = new ArrayList<File>(20);
 
-  /** Self instance */
-  private static FileManager singleton;
+  /** Best of files. */
+  private final List<File> alBestofFiles = new ArrayList<File>(20);
 
-  /** File comparator based on rate */
-  private Comparator<File> rateComparator = new Comparator<File>() {
+  /** Self instance. */
+  private static FileManager singleton = new FileManager();
+
+  /** File comparator based on rate. */
+  private final Comparator<File> rateComparator = new Comparator<File>() {
     public int compare(File file1, File file2) {
       long lRate1 = file1.getTrack().getRate();
       long lRate2 = file2.getTrack().getRate();
@@ -71,7 +75,7 @@ public final class FileManager extends ItemManager {
   };
 
   /**
-   * No constructor available, only static access
+   * No constructor available, only static access.
    */
   private FileManager() {
     super();
@@ -100,48 +104,75 @@ public final class FileManager extends ItemManager {
   }
 
   /**
+   * Gets the instance.
+   * 
    * @return singleton
    */
   public static FileManager getInstance() {
-    if (singleton == null) {
-      singleton = new FileManager();
-    }
     return singleton;
   }
 
   /**
-   * Register an File with a known id
+   * Register an File with a known id.
    * 
-   * @param sName
+   * @param sId DOCUMENT_ME
+   * @param sName DOCUMENT_ME
+   * @param directory DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * @param lSize DOCUMENT_ME
+   * @param lQuality DOCUMENT_ME
+   * 
+   * @return the file
    */
-  public synchronized File registerFile(String sId, String sName, Directory directory, Track track,
-      long lSize, long lQuality) {
-    File file = getFileByID(sId);
-    if (file == null) {
-      file = new File(sId, sName, directory, track, lSize, lQuality);
-      registerItem(file);
-      if (directory.getDevice().isRefreshing() && Log.isDebugEnabled()) {
-        Log.debug("registrated new file: " + file);
+  public File registerFile(String sId, String sName, Directory directory, Track track, long lSize,
+      long lQuality) {
+    lock.writeLock().lock();
+    try {
+      File file = getFileByID(sId);
+      if (file == null) {
+        file = new File(sId, sName, directory, track, lSize, lQuality);
+        registerItem(file);
+        if (directory.getDevice().isRefreshing() && Log.isDebugEnabled()) {
+          Log.debug("registrated new file: " + file);
+        }
+      } else {
+        // If file already exist and the track has changed, make changes
+        // Set name again because under Windows, the file name case
+        // could have changed but we keep the same file object
+        file.setName(sName);
       }
-    } else {
-      // If file already exist and the track has changed, make changes
-      // Set name again because under Windows, the file name case
-      // could have changed but we keep the same file object
-      file.setName(sName);
+      // add this file to track
+      file.setTrack(track);
+      // Add file to track
+      track.addFile(file);
+      return file;
+    } finally {
+      lock.writeLock().unlock();
     }
-    // add this file to track
-    file.setTrack(track);
-    // Add file to track
-    track.addFile(file);
-    return file;
   }
 
   /**
-   * Get file hashcode (ID)
+   * Register an File without known id.
+   * 
+   * @param sName DOCUMENT_ME
+   * @param directory DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * @param lSize DOCUMENT_ME
+   * @param lQuality DOCUMENT_ME
+   * 
+   * @return the file
+   */
+  public File registerFile(String sName, Directory directory, Track track, long lSize, long lQuality) {
+    String sId = createID(sName);
+    return registerFile(sId, sName, directory, track, lSize, lQuality);
+  }
+
+  /**
+   * Get file hashcode (ID).
+   * 
+   * @param sName DOCUMENT_ME
+   * @param dir DOCUMENT_ME
    * 
-   * @param sName
-   * @param device
-   * @param dir
    * @return file ID
    */
   protected static String createID(String sName, Directory dir) {
@@ -150,7 +181,8 @@ public final class FileManager extends ItemManager {
     // the same ID
     if (UtilSystem.isUnderWindows()) {
       id = MD5Processor.hash(new StringBuilder(dir.getDevice().getName()).append(
-          dir.getRelativePath().toLowerCase()).append(sName.toLowerCase()).toString());
+          dir.getRelativePath().toLowerCase(Locale.getDefault())).append(
+          sName.toLowerCase(Locale.getDefault())).toString());
     } else {
       id = MD5Processor.hash(new StringBuilder(dir.getDevice().getName()).append(
           dir.getRelativePath()).append(sName).toString());
@@ -159,158 +191,210 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Change a file name
+   * Change a file name.
+   * 
+   * @param fileOld DOCUMENT_ME
+   * @param sNewName DOCUMENT_ME
    * 
-   * @param fileOld
-   * @param sNewName
    * @return new file
+   * 
+   * @throws JajukException the jajuk exception
    */
-  public synchronized File changeFileName(org.jajuk.base.File fileOld, String sNewName)
-      throws JajukException {
-    // check given name is different
-
-    if (fileOld.getName().equals(sNewName)) {
-      return fileOld;
-    }
-    // check if this file still exists
-    if (!fileOld.getFIO().exists()) {
-      throw new CannotRenameException(135);
-    }
-    java.io.File fileNew = new java.io.File(fileOld.getFIO().getParentFile().getAbsolutePath()
-        + java.io.File.separator + sNewName);
-    // recalculate file ID
-    Directory dir = fileOld.getDirectory();
-    String sNewId = createID(sNewName, dir);
-    // create a new file (with own fio and sAbs)
-    Track track = fileOld.getTrack();
-    org.jajuk.base.File fNew = new File(sNewId, sNewName, fileOld.getDirectory(), track, fileOld
-        .getSize(), fileOld.getQuality());
-    // transfer all properties and reset id and name
-    fNew.setProperties(fileOld.getProperties());
-    fNew.setProperty(Const.XML_ID, sNewId); // reset new id and name
-    fNew.setProperty(Const.XML_NAME, sNewName); // reset new id and name
-    // check file name and extension
-    if (!(UtilSystem.getExtension(fileNew).equals(UtilSystem.getExtension(fileOld.getFIO())))) {
-      // no extension change
-      throw new CannotRenameException(134);
-    }
-    // check if future file exists (under windows, file.exists
-    // return true even with
-    // different case so we test file name is different)
-    if (!fileNew.getName().equalsIgnoreCase(fileOld.getName()) && fileNew.exists()) {
-      throw new CannotRenameException(134);
-    }
-    // try to rename file on disk
+  public File changeFileName(org.jajuk.base.File fileOld, String sNewName) throws JajukException {
+    lock.writeLock().lock();
     try {
-      if (!fileOld.getFIO().renameTo(fileNew)) {
+      // check given name is different
+      if (fileOld.getName().equals(sNewName)) {
+        return fileOld;
+      }
+      // check if this file still exists
+      if (!fileOld.getFIO().exists()) {
+        throw new CannotRenameException(135);
+      }
+      // check that the file is not currently played
+      if (QueueModel.getCurrentItem() != null
+          && QueueModel.getCurrentItem().getFile().equals(fileOld) && QueueModel.isPlayingTrack()) {
+        throw new CannotRenameException(172);
+      }
+
+      java.io.File fileNew = new java.io.File(fileOld.getFIO().getParentFile().getAbsolutePath()
+          + java.io.File.separator + sNewName);
+
+      // check file name and extension
+      if (!(UtilSystem.getExtension(fileNew).equals(UtilSystem.getExtension(fileOld.getFIO())))) {
+        // no extension change
         throw new CannotRenameException(134);
       }
-    } catch (Exception e) {
-      throw new CannotRenameException(134, e);
-    }
+      // check if destination file already exists (under windows, file.exists
+      // return true even with different case so we test file name is different)
+      if (!fileNew.getName().equalsIgnoreCase(fileOld.getName()) && fileNew.exists()) {
+        throw new CannotRenameException(134);
+      }
+      // try to rename file on disk
+      try {
+        if (!fileOld.getFIO().renameTo(fileNew)) {
+          throw new CannotRenameException(134);
+        }
+      } catch (Exception e) {
+        throw new CannotRenameException(134, e);
+      }
 
-    // OK, remove old file and register this new file
-    removeFile(fileOld);
-    registerItem(fNew);
-    track.addFile(fNew);
-    // notify everybody for the file change
-    Properties properties = new Properties();
-    properties.put(Const.DETAIL_OLD, fileOld);
-    properties.put(Const.DETAIL_NEW, fNew);
-    // Notify interested items (like history manager)
-    ObservationManager.notifySync(new JajukEvent(JajukEvents.FILE_NAME_CHANGED, properties));
-    return fNew;
+      // OK, remove old file and register this new file
+      // Compute file ID
+      Directory dir = fileOld.getDirectory();
+      String sNewId = createID(sNewName, dir);
+      // create a new file (with own fio and sAbs)
+      Track track = fileOld.getTrack();
+      // Remove old file from associated track
+      track.removeFile(fileOld);
+      org.jajuk.base.File fNew = new File(sNewId, sNewName, fileOld.getDirectory(), track, fileOld
+          .getSize(), fileOld.getQuality());
+      // transfer all properties and reset id and name
+      // We use a shallow copy of properties to avoid any properties share between
+      // two items
+      fNew.setProperties(fileOld.getShallowProperties());
+      fNew.setProperty(Const.XML_ID, sNewId); // reset new id and name
+      fNew.setProperty(Const.XML_NAME, sNewName); // reset new id and name
+
+      removeFile(fileOld);
+      registerItem(fNew);
+      track.addFile(fNew);
+      // notify everybody for the file change
+      Properties properties = new Properties();
+      properties.put(Const.DETAIL_OLD, fileOld);
+      properties.put(Const.DETAIL_NEW, fNew);
+      // Notify interested items (like history manager)
+      ObservationManager.notifySync(new JajukEvent(JajukEvents.FILE_NAME_CHANGED, properties));
+      return fNew;
+    } finally {
+      lock.writeLock().unlock();
+    }
   }
 
   /**
-   * Change a file directory
+   * Change a file directory.
+   * 
+   * @param old old file
+   * @param newDir new dir
    * 
-   * @param old
-   *          old file
-   * @param newDir
-   *          new dir
    * @return new file or null if an error occurs
    */
-  public synchronized File changeFileDirectory(File old, Directory newDir) {
-    // recalculate file ID
-    String sNewId = FileManager.createID(old.getName(), newDir);
-    Track track = old.getTrack();
-    // create a new file (with own fio and sAbs)
-    File fNew = new File(sNewId, old.getName(), newDir, track, old.getSize(), old.getQuality());
-    // transfert all properties (inc id), then set right id and directory
-    fNew.setProperties(old.getProperties());
-    fNew.setProperty(Const.XML_ID, sNewId);
-    fNew.setProperty(Const.XML_DIRECTORY, newDir.getID());
+  public File changeFileDirectory(File old, Directory newDir) {
+    lock.writeLock().lock();
+    try {
+      // recalculate file ID
+      String sNewId = FileManager.createID(old.getName(), newDir);
+      Track track = old.getTrack();
+      // create a new file (with own fio and sAbs)
+      File fNew = new File(sNewId, old.getName(), newDir, track, old.getSize(), old.getQuality());
+      // Transfer all properties (including id), then set right id and directory
+      // We use a shallow copy of properties to avoid any properties share between
+      // two items
+      fNew.setProperties(old.getShallowProperties());
+      fNew.setProperty(Const.XML_ID, sNewId);
+      fNew.setProperty(Const.XML_DIRECTORY, newDir.getID());
 
-    // OK, remove old file and register this new file
-    removeFile(old);
-    registerItem(fNew);
-    track.addFile(fNew);
-    return fNew;
-  }
-
-  /**
-   * Clear all references for the given device
-   * 
-   * @param sId :
-   *          Device id
-   */
-  public synchronized void clearDevice(String sId) {
-    for (File file : getFiles()) {
-      if (file.getDirectory() == null || file.getDirectory().getDevice().getID().equals(sId)) {
-        removeItem(file);
-      }
+      // OK, remove old file and register this new file
+      removeFile(old);
+      registerItem(fNew);
+      track.addFile(fNew);
+      return fNew;
+    } finally {
+      lock.writeLock().unlock();
     }
   }
 
   /**
-   * Remove a file reference
+   * Clean all references for the given device.
    * 
-   * @param file
+   * @param sId :
+   * Device id
    */
-  public synchronized void removeFile(File file) {
-    // We need to remove the file from the track !
-    file.getTrack().removeFile(file);
-    removeItem(file);
+  public void cleanDevice(String sId) {
+    lock.writeLock().lock();
+    try {
+      for (File file : getFiles()) {
+        if (file.getDirectory() == null || file.getDirectory().getDevice().getID().equals(sId)) {
+          removeItem(file);
+        }
+      }
+    } finally {
+      lock.writeLock().unlock();
+    }
   }
 
   /**
-   * Return file by full path
+   * Remove a file reference.
+   * 
+   * @param file DOCUMENT_ME
+   */
+  public void removeFile(File file) {
+    lock.writeLock().lock();
+    try {
+      // We need to remove the file from the track !
+      TrackManager.getInstance().removefile(file.getTrack(), file);
+      removeItem(file);
+    } finally {
+      lock.writeLock().unlock();
+    }
+  }
+
+  /**
+   * Return file by full path.
    * 
    * @param sPath :
-   *          full path
+   * full path
+   * 
    * @return file or null if given path is not known
    */
 
-  public synchronized File getFileByPath(String sPath) {
-    File fOut = null;
-    java.io.File fToCompare = new java.io.File(sPath);
-    ReadOnlyIterator<File> it = getFilesIterator();
-    while (it.hasNext()) {
-      File file = it.next();
-      // we compare io files and not paths
-      // to avoid dealing with path name issues
-      if (file.getFIO().equals(fToCompare)) {
-        fOut = file;
-        break;
+  public File getFileByPath(String sPath) {
+    lock.readLock().lock();
+    try {
+      File fOut = null;
+      java.io.File fToCompare = new java.io.File(sPath);
+      ReadOnlyIterator<File> it = getFilesIterator();
+      while (it.hasNext()) {
+        File file = it.next();
+        // we compare io files and not paths
+        // to avoid dealing with path name issues
+        if (file.getFIO().equals(fToCompare)) {
+          fOut = file;
+          break;
+        }
       }
+      // Fix  #1717 (Cannot load some playlists) : if the file is not found, second chance ignoring the case
+      // This can happen under Unix when using an SMB drive
+      if (fOut == null) {
+        it = getFilesIterator();
+        while (it.hasNext()) {
+          File file = it.next();
+          if (file.getFIO().getAbsolutePath().equalsIgnoreCase(fToCompare.getAbsolutePath())) {
+            fOut = file;
+            break;
+          }
+        }
+      }
+      return fOut;
+    } finally {
+      lock.readLock().unlock();
     }
-    return fOut;
   }
 
   /**
+   * Gets the ready files.
+   * 
    * @return All accessible files of the collection
    */
   public List<File> getReadyFiles() {
-    List<File> files = null;
-    files = FileManager.getInstance().getFiles();
+    List<File> files = FileManager.getInstance().getFiles();
     CollectionUtils.filter(files, new JajukPredicates.ReadyFilePredicate());
     return files;
   }
 
   /**
    * Return a shuffle mounted and unbaned file from the entire collection or
-   * null if none available using these criterias
+   * null if none available using these criterias.
    * 
    * @return the file
    */
@@ -318,8 +402,8 @@ public final class FileManager extends ItemManager {
     List<File> alEligibleFiles = getReadyFiles();
     // filter banned files
     CollectionUtils.filter(alEligibleFiles, new JajukPredicates.BannedFilePredicate());
-    int index = UtilSystem.getRandom().nextInt(alEligibleFiles.size() - 1);
     if (alEligibleFiles.size() > 0) {
+      int index = UtilSystem.getRandom().nextInt(alEligibleFiles.size() - 1);
       return alEligibleFiles.get(index);
     } else {
       return null;
@@ -327,10 +411,10 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return an ordered playlist with the entire accessible shuffle collection
+   * Return an ordered playlist with the entire accessible shuffle collection.
    * 
    * @return The entire accessible shuffle collection (can return a void
-   *         collection)
+   * collection)
    */
   public List<File> getGlobalShufflePlaylist() {
     List<File> alEligibleFiles = getReadyFiles();
@@ -375,9 +459,9 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return a shuffle mounted file from the novelties
+   * Return a shuffle mounted file from the novelties.
    * 
-   * @return
+   * @return the novelty file
    */
   public File getNoveltyFile() {
     List<File> alEligibleFiles = getGlobalNoveltiesPlaylist();
@@ -385,10 +469,10 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return a shuffled playlist with the entire accessible novelties collection
+   * Return a shuffled playlist with the entire accessible novelties collection.
    * 
    * @return The entire accessible novelties collection (can return a void
-   *         collection)
+   * collection)
    */
   public List<org.jajuk.base.File> getGlobalNoveltiesPlaylist() {
     return getGlobalNoveltiesPlaylist(true);
@@ -397,9 +481,10 @@ public final class FileManager extends ItemManager {
   /**
    * Return an ordered playlist with the accessible novelties collection The
    * number of returned items is limited to NB_TRACKS_ON_ACTION for performance
-   * reasons
+   * reasons.
+   * 
+   * @param bHideUnmounted DOCUMENT_ME
    * 
-   * @param bHideUnmounted
    * @return The entire accessible novelties collection
    */
   public List<File> getGlobalNoveltiesPlaylist(boolean bHideUnmounted) {
@@ -414,7 +499,7 @@ public final class FileManager extends ItemManager {
       if (alEligibleFiles.size() > Const.NB_TRACKS_ON_ACTION) {
         break;
       }
-      File file = track.getPlayeableFile(bHideUnmounted);
+      File file = track.getBestFile(bHideUnmounted);
       // try to get a mounted file
       // (can return null)
       if (file == null) {// none mounted file, take first file we find
@@ -434,7 +519,7 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return a shuffled playlist with the entire accessible novelties collection
+   * Return a shuffled playlist with the entire accessible novelties collection.
    * 
    * @return The entire accessible novelties collection
    */
@@ -452,9 +537,10 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Convenient method used to return shuffled files by album
+   * Convenient method used to return shuffled files by album.
+   * 
+   * @param alEligibleFiles DOCUMENT_ME
    * 
-   * @param alEligibleFiles
    * @return Shuffled tracks by album
    */
   private List<File> getShuffledFilesByAlbum(List<File> alEligibleFiles) {
@@ -486,6 +572,8 @@ public final class FileManager extends ItemManager {
   }
 
   /**
+   * Gets the sorted by rate.
+   * 
    * @return a sorted set of the collection by rate, highest first
    */
   private List<File> getSortedByRate() {
@@ -498,7 +586,7 @@ public final class FileManager extends ItemManager {
 
   /**
    * Return a shuffled playlist with the entire accessible bestof collection,
-   * best first
+   * best first.
    * 
    * @return Shuffled best tracks (n% of favorite)
    */
@@ -510,7 +598,7 @@ public final class FileManager extends ItemManager {
     if (al.size() > 0) {
       // find superior interval value
       int sup = (int) ((Const.BESTOF_PROPORTION) * al.size());
-      if (sup < 0) {
+      if (sup < 2) {
         sup = al.size();
       }
       alBest = al.subList(0, sup - 1);
@@ -520,33 +608,43 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return ordered (by rate) bestof files
+   * Return ordered (by rate) bestof files.
    * 
-   * @param bHideUnmounted
-   *          if true, unmounted files are not chosen
    * @return top files
    */
   public List<File> getBestOfFiles() {
-    refreshBestOfFiles();
+    // Don't refresh best of files at each call because it makes  the playlist view
+    // unusable for bestof files : each time a file is played, the view is changed
+    if (alBestofFiles.size() == 0) {
+      refreshBestOfFiles();
+    }
     return alBestofFiles;
   }
 
+  /**
+   * Refresh best of files.
+   */
   public void refreshBestOfFiles() {
-    int iNbBestofFiles = Integer.parseInt(Conf.getString(Const.CONF_BESTOF_TRACKS_SIZE));
+    Log.debug("Invoking Refresh of BestOf-Files");
+
     // clear data
     alBestofFiles.clear();
+
     // create a temporary table to remove unmounted files
+    int iNbBestofFiles = Integer.parseInt(Conf.getString(Const.CONF_BESTOF_TRACKS_SIZE));
     List<File> alEligibleFiles = new ArrayList<File>(iNbBestofFiles);
     List<Track> tracks = TrackManager.getInstance().getTracks();
+
     // filter banned tracks
     CollectionUtils.filter(tracks, new JajukPredicates.BannedTrackPredicate());
     for (Track track : tracks) {
-      File file = track.getPlayeableFile(Conf.getBoolean(Const.CONF_OPTIONS_HIDE_UNMOUNTED));
+      File file = track.getBestFile(Conf.getBoolean(Const.CONF_OPTIONS_HIDE_UNMOUNTED));
       if (file != null) {
         alEligibleFiles.add(file);
       }
     }
     Collections.sort(alEligibleFiles, rateComparator);
+
     // Keep as much items as we can
     int i = 0;
     while (i < alEligibleFiles.size() && i < iNbBestofFiles) {
@@ -557,13 +655,14 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return next mounted file ( used in continue mode )
+   * Return next mounted file ( used in continue mode ).
    * 
    * @param file :
-   *          a file
+   * a file
+   * 
    * @return next file from entire collection
    */
-  public synchronized File getNextFile(File file) {
+  public File getNextFile(File file) {
     List<File> files = getFiles();
     if (file == null) {
       return null;
@@ -595,20 +694,44 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return previous mounted file
+   * Return next mounted file from a different album than the provided file.
    * 
    * @param file :
-   *          a file
+   * a file
+   * 
+   * @return next file from entire collection
+   */
+  public File getNextAlbumFile(File file) {
+    File testedFile = file;
+    if (DirectoryManager.getInstance().getDirectories().size() > 1) {
+      while (testedFile.getDirectory().equals(file.getDirectory())) {
+        testedFile = getNextFile(testedFile);
+      }
+    }
+    if (!testedFile.getDirectory().equals(file.getDirectory())) {
+      return testedFile;
+    }
+    // Should not happen
+    else {
+      return null;
+    }
+  }
+
+  /**
+   * Return previous mounted file.
+   * 
+   * @param file :
+   * a file
+   * 
    * @return previous file from entire collection
    */
-  public synchronized File getPreviousFile(File file) {
+  public File getPreviousFile(File file) {
     List<File> files = getFiles();
     if (file == null) {
       return null;
     }
     File filePrevious = null;
-    List<Item> alSortedFiles = new ArrayList<Item>(files);
-    int i = alSortedFiles.indexOf(file);
+    int i = files.indexOf(file);
     // test if this file is the very first one
     if (i == 0) {
       Messages.showErrorMessage(128);
@@ -617,7 +740,7 @@ public final class FileManager extends ItemManager {
     // look for a correct file from index to collection begin
     boolean bOk = false;
     for (int index = i - 1; index >= 0; index--) {
-      filePrevious = (File) alSortedFiles.get(index);
+      filePrevious = files.get(index);
       if (filePrevious.isReady()) { // file must be on a mounted
         // device not refreshing
         bOk = true;
@@ -631,17 +754,18 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * Return whether the given file is the very first file from collection
+   * Return whether the given file is the very first file from collection.
    * 
-   * @param file
-   * @return
+   * @param file DOCUMENT_ME
+   * 
+   * @return true, if checks if is very first file
    */
   public boolean isVeryfirstFile(File file) {
     List<File> files = getFiles();
     if (file == null || files.size() == 0) {
       return false;
     }
-    return (file.equals(files.get(0)));
+    return file.equals(files.get(0));
   }
 
   /*
@@ -655,8 +779,10 @@ public final class FileManager extends ItemManager {
   }
 
   /**
-   * @param sID
-   *          Item ID
+   * Gets the file by id.
+   * 
+   * @param sID Item ID
+   * 
    * @return File matching the id
    */
   public File getFileByID(String sID) {
@@ -664,20 +790,22 @@ public final class FileManager extends ItemManager {
   }
 
   /**
+   * Gets the files.
    * 
    * @return ordered files list
    */
   @SuppressWarnings("unchecked")
-  public synchronized List<File> getFiles() {
+  public List<File> getFiles() {
     return (List<File>) getItems();
   }
 
   /**
+   * Gets the files iterator.
    * 
    * @return files iterator
    */
   @SuppressWarnings("unchecked")
-  public synchronized ReadOnlyIterator<File> getFilesIterator() {
+  public ReadOnlyIterator<File> getFilesIterator() {
     return new ReadOnlyIterator<File>((Iterator<File>) getItemsIterator());
   }
 
