@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2005 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 4591 $
+ *  $Revision: 6186 $
  */
 
 package org.jajuk.base;
@@ -29,50 +30,44 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.collections.Predicate;
+import org.jajuk.services.tags.Tag;
 import org.jajuk.util.Const;
-import org.jajuk.util.Messages;
-import org.jajuk.util.ReadOnlyIterator;
+import org.jajuk.util.MD5Processor;
 import org.jajuk.util.error.JajukException;
 import org.jajuk.util.log.Log;
 
 /**
- * Managers parent class
+ * Managers parent class.
  */
 public abstract class ItemManager {
 
-  /**
-   * Maps item classes -> instance, must be a linked map for ordering (mandatory
-   * in commited collection)
-   */
-  private static LinkedHashMap<Class<?>, ItemManager> hmItemManagers = new LinkedHashMap<Class<?>, ItemManager>(
+  /** Maps item classes -> instance, must be a linked map for ordering (mandatory in commited collection). */
+  private static Map<Class<?>, ItemManager> hmItemManagers = new LinkedHashMap<Class<?>, ItemManager>(
       10);
 
-  /** Maps properties meta information name and object */
-  private HashMap<String, PropertyMetaInformation> hmPropertiesMetaInformation = new LinkedHashMap<String, PropertyMetaInformation>(
+  /** Maps properties meta information name and object. */
+  private final Map<String, PropertyMetaInformation> hmPropertiesMetaInformation = new LinkedHashMap<String, PropertyMetaInformation>(
       10);
 
-  /*****************************************************************************
-   * Items collection
-   * <p>
-   * We use a concrete type here and not an upper bounded wildcard type (?
-   * extends Item) because this prevent from calling methods in it like
-   * put(String,Album)
-   * </p>
-   ****************************************************************************/
-
-  private Set<Item> finalItems = new TreeSet<Item>();
+  /** The Lock. */
+  ReadWriteLock lock = new ReentrantReadWriteLock();
 
+  /** Use an array list during startup which is faster during loading the collection. */
   private List<Item> startupItems = new ArrayList<Item>(100);
 
-  private Map<String, Item> internalMap = new HashMap<String, Item>(100);
+  /** Stores the items by ID to have quick access if necessary. */
+  private final Map<String, Item> internalMap = new HashMap<String, Item>(100);
 
+  /** Collection pointer : at the beginning point to the ArrayList, later this is replaced by a TreeSet to have correct ordering. */
   private Collection<Item> items = startupItems;
 
   /**
-   * Item manager default constructor
+   * Item manager default constructor.
    */
   ItemManager() {
   }
@@ -81,7 +76,7 @@ public abstract class ItemManager {
    * Switch all item managers to ordered mode See
    * ItemManager.switchToOrderState() for more details
    */
-  public static synchronized void switchAllManagersToOrderState() {
+  public static void switchAllManagersToOrderState() {
     Log.debug("Switching to sorted mode");
     for (ItemManager manager : hmItemManagers.values()) {
       manager.switchToOrderState();
@@ -91,38 +86,47 @@ public abstract class ItemManager {
   /**
    * Switch this item manager to order mode This feature allows faster
    * collection loading As collection.xml contains ordered elements, we simply a
-   * use an arraylist to store items first, then few seconds after startup and
-   * before user could make changes to the collection, we populate a treeset
-   * from the arraylist and begin ti use it.
-   * 
+   * use an ArrayList to store items first, then few seconds after startup and
+   * before user could make changes to the collection, we populate a TreeSet
+   * from the ArrayList and begin to use it.
    */
-  public synchronized void switchToOrderState() {
-    for (Item item : startupItems) {
-      finalItems.add(item);
+  public void switchToOrderState() {
+    lock.writeLock().lock();
+    try {
+      // populate a new TreeSet with the startup-items
+      if (startupItems != null) {
+        items = new TreeSet<Item>(startupItems);
+
+        // Free startup memory
+        startupItems = null;
+      }
+    } finally {
+      lock.writeLock().unlock();
     }
-    items = finalItems;
-    // Free startup memory
-    startupItems = null;
   }
 
   /**
-   * Registrates a new item manager
+   * Registers a new item manager.
    * 
-   * @param c
-   *          Managed item class
-   * @param itemManager
+   * @param c Managed item class
+   * @param itemManager DOCUMENT_ME
    */
   public static void registerItemManager(Class<?> c, ItemManager itemManager) {
     hmItemManagers.put(c, itemManager);
   }
 
   /**
+   * Gets the label.
+   * 
    * @return identifier used for XML generation
    */
   public abstract String getLabel();
 
   /**
-   * @param sPropertyName
+   * Gets the meta information.
+   * 
+   * @param sPropertyName DOCUMENT_ME
+   * 
    * @return meta data for given property
    */
   public PropertyMetaInformation getMetaInformation(String sPropertyName) {
@@ -130,52 +134,59 @@ public abstract class ItemManager {
   }
 
   /**
-   * Return a human representation for a given property name when we don't now
-   * item type we work on. Otherwise, use PropertyMetaInformation.getHumanType
+   * Remove a property *.
    * 
-   * @param s
-   * @return
+   * @param sProperty DOCUMENT_ME
    */
-  public static String getHumanType(String sKey) {
-    String sOut = sKey;
-    if (Messages.contains(Const.PROPERTY_SEPARATOR + sKey)) {
-      return Messages.getString(Const.PROPERTY_SEPARATOR + sKey);
-    }
-    return sOut;
-  }
-
-  /** Remove a property * */
   public void removeProperty(String sProperty) {
     PropertyMetaInformation meta = getMetaInformation(sProperty);
     hmPropertiesMetaInformation.remove(sProperty);
-    applyRemoveProperty(meta); // remove this property to all items
+    applyRemoveProperty(meta); // remove this property from all items
   }
 
-  /** Remove a custom property to all items for the given manager */
-  public synchronized void applyRemoveProperty(PropertyMetaInformation meta) {
-    for (Item item : items) {
-      item.removeProperty(meta.getName());
+  /**
+   * Remove a custom property from all items for the given manager.
+   * 
+   * @param meta DOCUMENT_ME
+   */
+  public void applyRemoveProperty(PropertyMetaInformation meta) {
+    lock.readLock().lock();
+    try {
+      for (Item item : items) {
+        item.removeProperty(meta.getName());
+      }
+    } finally {
+      lock.readLock().unlock();
     }
   }
 
   /**
-   * Generic method to access to a parametrized list of items
+   * Generic method to access to a parameterized list of items.
    * 
-   * @return the item-parametrized list
+   * @param meta DOCUMENT_ME
+   * 
+   * @return the item-parameterized list
    * 
    * protected abstract HashMap<String, Item> getItemsMap();
    */
 
   /** Add a custom property to all items for the given manager */
-  public synchronized void applyNewProperty(PropertyMetaInformation meta) {
-    for (Item item : items) {
-      item.setProperty(meta.getName(), meta.getDefaultValue());
+  public void applyNewProperty(PropertyMetaInformation meta) {
+    lock.readLock().lock();
+    try {
+      for (Item item : items) {
+        item.setProperty(meta.getName(), meta.getDefaultValue());
+      }
+    } finally {
+      lock.readLock().unlock();
     }
   }
 
   /**
+   * Attention, this method does not return a full XML, but rather an excerpt
+   * that is then completed in Collection.commit()!
    * 
-   * @return XML representation of this manager
+   * @return (partial) XML representation of this manager
    */
   public String toXML() {
     StringBuilder sb = new StringBuilder("<").append(getLabel() + ">");
@@ -189,6 +200,45 @@ public abstract class ItemManager {
   }
 
   /**
+   * Return the associated read write lock.
+   * 
+   * @return the associated read write lock
+   */
+  public ReadWriteLock getLock() {
+    return lock;
+  }
+
+  /**
+   * Format the item name to be normalized :
+   * <p>
+   * -no underscores or other non-ascii characters
+   * <p>
+   * -no spaces at the begin and the end
+   * <p>
+   * -All in lower case expect first letter of first word
+   * <p>
+   * example: "My artist".
+   * 
+   * @param sName The name to format.
+   * 
+   * @return the string
+   * 
+   * TODO: the "all lowercase" part is not done currently, should this be changed??
+   */
+  public static String format(String sName) {
+    String sOut;
+    sOut = sName.trim(); // suppress spaces at the begin and the end
+    sOut = sOut.replace('-', ' '); // move - to space
+    sOut = sOut.replace('_', ' '); // move _ to space
+    char c = sOut.charAt(0);
+    StringBuilder sb = new StringBuilder(sOut);
+    sb.setCharAt(0, Character.toUpperCase(c));
+    return sb.toString();
+  }
+
+  /**
+   * Gets the properties.
+   * 
    * @return properties Meta informations
    */
   public Collection<PropertyMetaInformation> getProperties() {
@@ -196,6 +246,8 @@ public abstract class ItemManager {
   }
 
   /**
+   * Gets the custom properties including activated extra tags.
+   * 
    * @return custom properties Meta informations
    */
   public Collection<PropertyMetaInformation> getCustomProperties() {
@@ -211,6 +263,25 @@ public abstract class ItemManager {
   }
 
   /**
+   * Gets the custom properties without the activated extra tags.
+   * 
+   * @return custom properties Meta informations
+   */
+  public Collection<PropertyMetaInformation> getUserCustomProperties() {
+    List<PropertyMetaInformation> col = new ArrayList<PropertyMetaInformation>();
+    Iterator<PropertyMetaInformation> it = hmPropertiesMetaInformation.values().iterator();
+    while (it.hasNext()) {
+      PropertyMetaInformation meta = it.next();
+      if (meta.isCustom() && !Tag.getActivatedExtraTags().contains(meta.getName())) {
+        col.add(meta);
+      }
+    }
+    return col;
+  }
+
+  /**
+   * Gets the visible properties.
+   * 
    * @return visible properties Meta informations
    */
   public Collection<PropertyMetaInformation> getVisibleProperties() {
@@ -226,10 +297,12 @@ public abstract class ItemManager {
   }
 
   /**
-   * Get ItemManager manager with a given attribute name
+   * Get the manager from a given attribute name.
    * 
-   * @param sItem
-   * @return
+   * @param sProperty The property to compare.
+   * 
+   * @return an ItemManager if one is found for the property or null if none
+   * found.
    */
   public static ItemManager getItemManager(String sProperty) {
     if (Const.XML_DEVICE.equals(sProperty)) {
@@ -238,10 +311,12 @@ public abstract class ItemManager {
       return TrackManager.getInstance();
     } else if (Const.XML_ALBUM.equals(sProperty)) {
       return AlbumManager.getInstance();
-    } else if (Const.XML_AUTHOR.equals(sProperty)) {
-      return AuthorManager.getInstance();
-    } else if (Const.XML_STYLE.equals(sProperty)) {
-      return StyleManager.getInstance();
+    } else if (Const.XML_ARTIST.equals(sProperty)) {
+      return ArtistManager.getInstance();
+    } else if (Const.XML_ALBUM_ARTIST.equals(sProperty)) {
+      return AlbumArtistManager.getInstance();
+    } else if (Const.XML_GENRE.equals(sProperty)) {
+      return GenreManager.getInstance();
     } else if (Const.XML_DIRECTORY.equals(sProperty)) {
       return DirectoryManager.getInstance();
     } else if (Const.XML_FILE.equals(sProperty)) {
@@ -256,9 +331,10 @@ public abstract class ItemManager {
   }
 
   /**
-   * Get ItemManager manager for given item class
+   * Get ItemManager manager for given item class.
+   * 
+   * @param c DOCUMENT_ME
    * 
-   * @param class
    * @return associated item manager or null if none was found
    */
   public static ItemManager getItemManager(Class<?> c) {
@@ -266,106 +342,133 @@ public abstract class ItemManager {
   }
 
   /**
-   * Return an iteration over item managers
+   * Return an iteration over item managers.
+   * 
+   * @return the item managers
    */
   public static Iterator<ItemManager> getItemManagers() {
     return hmItemManagers.values().iterator();
   }
 
   /**
-   * Perform an cleanup : delete useless items
+   * Perform cleanup : delete useless items.
    */
   @SuppressWarnings("unchecked")
-  public synchronized void cleanup() {
-    // Prefetch item manager type for performances
-    short managerType = 0; // Album
-    if (this instanceof AuthorManager) {
-      managerType = 1;
-    } else if (this instanceof StyleManager) {
-      Log.debug("Style cleanup not allowed");
-      return;
-    } else if (this instanceof YearManager) {
-      managerType = 2;
-    }
-    // build used items set
-    List<Item> items = new ArrayList<Item>(100);
-    ReadOnlyIterator<Track> tracks = TrackManager.getInstance().getTracksIterator();
-    while (tracks.hasNext()) {
-      Track track = tracks.next();
-      switch (managerType) {
-      case 0:
-        items.add(track.getAlbum());
-        break;
-      case 1:
-        items.add(track.getAuthor());
-        break;
-      case 2:
-        items.add(track.getYear());
-        break;
+  public void cleanup() {
+    lock.writeLock().lock();
+    try {
+
+      // Prefetch item manager type for performances
+      short managerType = 0; // Album
+      if (this instanceof ArtistManager) {
+        managerType = 1;
+      } else if (this instanceof GenreManager) {
+        Log.debug("Genre cleanup not allowed");
+        return;
+      } else if (this instanceof YearManager) {
+        managerType = 2;
+      } else if (this instanceof AlbumArtistManager) {
+        managerType = 3;
       }
-    }
-    // Now iterate over this manager items to check if it is present in the
-    // items list
-    Iterator<Item> it = (Iterator<Item>) getItemsIterator();
-    while (it.hasNext()) {
-      Item item = it.next();
-      // check if this item still maps some tracks
-      if (!items.contains(item)) {
-        it.remove();
+      // build used items set
+      List<Item> lItems = new ArrayList<Item>(100);
+      List<Track> tracks = TrackManager.getInstance().getTracks();
+      for (Track track : tracks) {
+        switch (managerType) {
+        case 0:
+          lItems.add(track.getAlbum());
+          break;
+        case 1:
+          lItems.add(track.getArtist());
+          break;
+        case 2:
+          lItems.add(track.getYear());
+          break;
+        case 3:
+          lItems.add(track.getAlbumArtist());
+          break;
+        }
       }
+      // Now iterate over this manager items to check if it is present in the
+      // items list
+      Iterator<Item> it = (Iterator<Item>) getItemsIterator();
+      while (it.hasNext()) {
+        Item item = it.next();
+        // check if this item still maps some tracks
+        if (!lItems.contains(item)) {
+          it.remove();
+          internalMap.remove(item.getID());
+        }
+      }
+    } finally {
+      lock.writeLock().unlock();
     }
+
   }
 
   /**
-   * Perform a cleanup of all orphan tracks associated with given item
-   * 
-   * @param item
-   *          item whose associated tracks should be checked for cleanup
+   * Perform a cleanup of all orphan tracks associated with given item.
    * 
+   * @param item item whose associated tracks should be checked for cleanup
    */
-  protected synchronized void cleanOrphanTracks(Item item) {
-    if (TrackManager.getInstance().getAssociatedTracks(item, false).size() == 0) {
+  protected void cleanOrphanTracks(Item item) {
+    if (TrackManager.getInstance().getAssociatedTracks(item, false).isEmpty()) {
       removeItem(item);
     }
   }
 
-  /** Remove a given item */
-  protected synchronized void removeItem(Item item) {
-    if (item != null) {
-      items.remove(item);
-      internalMap.remove(item.getID());
+  /**
+   * Remove a given item.
+   * 
+   * @param item DOCUMENT_ME
+   */
+  protected void removeItem(Item item) {
+    lock.writeLock().lock();
+    try {
+      if (item != null) {
+        items.remove(item);
+        internalMap.remove(item.getID());
+      }
+    } finally {
+      lock.writeLock().unlock();
     }
   }
 
   /**
-   * Register a given item
+   * Register a given item.
    * 
-   * @param item :
-   *          the item to add
+   * @param item : the item to add
    */
-  protected synchronized void registerItem(Item item) {
-    items.add(item);
-    internalMap.put(item.getID(), item);
+  protected void registerItem(Item item) {
+    lock.writeLock().lock();
+    try {
+      items.add(item);
+      internalMap.put(item.getID(), item);
+    } finally {
+      lock.writeLock().unlock();
+    }
   }
 
   /**
-   * Register a new property
+   * Register a new property.
    * 
-   * @param meta
+   * @param meta DOCUMENT_ME
    */
   public void registerProperty(PropertyMetaInformation meta) {
     hmPropertiesMetaInformation.put(meta.getName(), meta);
   }
 
   /**
-   * Change any item
+   * Change any item.
+   * 
+   * @param itemToChange DOCUMENT_ME
+   * @param sKey DOCUMENT_ME
+   * @param oValue DOCUMENT_ME
+   * @param filter : files we want to deal with
    * 
-   * @param itemToChange
-   * @param sKey
-   * @param oValue
-   * @param filter:
-   *          files we want to deal with
    * @return the changed item
+   * 
+   * @throws JajukException the jajuk exception
    */
   public static Item changeItem(Item itemToChange, String sKey, Object oValue, Set<File> filter)
       throws JajukException {
@@ -380,14 +483,14 @@ public abstract class ItemManager {
       } else if (Const.XML_TRACK.equals(sKey)) { // track name
         newItem = TrackManager.getInstance().changeTrackName(file.getTrack(), (String) oValue,
             filter);
-      } else if (Const.XML_STYLE.equals(sKey)) {
-        newItem = TrackManager.getInstance().changeTrackStyle(file.getTrack(), (String) oValue,
+      } else if (Const.XML_GENRE.equals(sKey)) {
+        newItem = TrackManager.getInstance().changeTrackGenre(file.getTrack(), (String) oValue,
             filter);
       } else if (Const.XML_ALBUM.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackAlbum(file.getTrack(), (String) oValue,
             filter);
-      } else if (Const.XML_AUTHOR.equals(sKey)) {
-        newItem = TrackManager.getInstance().changeTrackAuthor(file.getTrack(), (String) oValue,
+      } else if (Const.XML_ARTIST.equals(sKey)) {
+        newItem = TrackManager.getInstance().changeTrackArtist(file.getTrack(), (String) oValue,
             filter);
       } else if (Const.XML_TRACK_COMMENT.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackComment(file.getTrack(), (String) oValue,
@@ -395,13 +498,23 @@ public abstract class ItemManager {
       } else if (Const.XML_TRACK_ORDER.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackOrder(file.getTrack(), (Long) oValue,
             filter);
+      } else if (Const.XML_ALBUM_ARTIST.equals(sKey)) {
+        newItem = TrackManager.getInstance().changeTrackAlbumArtist(file.getTrack(),
+            (String) oValue, filter);
       } else if (Const.XML_YEAR.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackYear(file.getTrack(),
             String.valueOf(oValue), filter);
       } else if (Const.XML_TRACK_RATE.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackRate(file.getTrack(), (Long) oValue);
       } else { // others properties
-        itemToChange.setProperty(sKey, oValue);
+        // check if this key is known for files
+        if (file.getMeta(sKey) != null) {
+          itemToChange.setProperty(sKey, oValue);
+        }
+        // Unknown ? check if it is a track custom property
+        else if (file.getTrack().getMeta(sKey) != null) {
+          file.getTrack().setProperty(sKey, oValue);
+        }
       }
       // Get associated track file
       if (newItem instanceof Track) {
@@ -426,14 +539,17 @@ public abstract class ItemManager {
       if (Const.XML_NAME.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackName((Track) itemToChange, (String) oValue,
             filter);
-      } else if (Const.XML_STYLE.equals(sKey)) {
-        newItem = TrackManager.getInstance().changeTrackStyle((Track) itemToChange,
+      } else if (Const.XML_GENRE.equals(sKey)) {
+        newItem = TrackManager.getInstance().changeTrackGenre((Track) itemToChange,
             (String) oValue, filter);
       } else if (Const.XML_ALBUM.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackAlbum((Track) itemToChange,
             (String) oValue, filter);
-      } else if (Const.XML_AUTHOR.equals(sKey)) {
-        newItem = TrackManager.getInstance().changeTrackAuthor((Track) itemToChange,
+      } else if (Const.XML_ARTIST.equals(sKey)) {
+        newItem = TrackManager.getInstance().changeTrackArtist((Track) itemToChange,
+            (String) oValue, filter);
+      } else if (Const.XML_ALBUM_ARTIST.equals(sKey)) {
+        newItem = TrackManager.getInstance().changeTrackAlbumArtist((Track) itemToChange,
             (String) oValue, filter);
       } else if (Const.XML_TRACK_COMMENT.equals(sKey)) {
         newItem = TrackManager.getInstance().changeTrackComment((Track) itemToChange,
@@ -455,16 +571,16 @@ public abstract class ItemManager {
       } else { // others properties
         itemToChange.setProperty(sKey, oValue);
       }
-    } else if (itemToChange instanceof Author) {
+    } else if (itemToChange instanceof Artist) {
       if (Const.XML_NAME.equals(sKey)) {
-        newItem = AuthorManager.getInstance().changeAuthorName((Author) itemToChange,
+        newItem = ArtistManager.getInstance().changeArtistName((Artist) itemToChange,
             (String) oValue);
       } else { // others properties
         itemToChange.setProperty(sKey, oValue);
       }
-    } else if (itemToChange instanceof Style) {
+    } else if (itemToChange instanceof Genre) {
       if (Const.XML_NAME.equals(sKey)) {
-        newItem = StyleManager.getInstance().changeStyleName((Style) itemToChange, (String) oValue);
+        newItem = GenreManager.getInstance().changeGenreName((Genre) itemToChange, (String) oValue);
       } else { // others properties
         itemToChange.setProperty(sKey, oValue);
       }
@@ -475,6 +591,7 @@ public abstract class ItemManager {
   }
 
   /**
+   * Gets the element count.
    * 
    * @return number of item
    */
@@ -483,66 +600,98 @@ public abstract class ItemManager {
   }
 
   /**
-   * @param sID
-   *          Item ID
+   * Gets the item by id.
+   * 
+   * @param sID Item ID
+   * 
    * @return Item
    */
   public Item getItemByID(String sID) {
-    Item item = internalMap.get(sID);
-    return item;
+    return internalMap.get(sID);
   }
 
   /**
-   * Return a shallow copy of all registrated items
+   * Return a copy of all registered items.
    * 
-   * @return a shallow copy of all registrated items
+   * @return a copy of all registered items
    */
-  public synchronized List<? extends Item> getItems() {
+  public List<? extends Item> getItems() {
     return new ArrayList<Item>(items);
   }
 
   /**
-   * Return a shallow copy of all registrated items filtered using the provided
-   * predicate*
+   * Return a shallow copy of all registered items filtered using the provided
+   * predicate*.
+   * 
+   * @param predicate DOCUMENT_ME
+   * 
+   * @return a shallow copy of all registered items filtered using the provided
    * 
    * @arg predicate : the predicate
-   * @return a shallow copy of all registrated items filtered using the provided
    */
-  public synchronized List<? extends Item> getFilteredItems(Predicate predicate) {
+  public List<? extends Item> getFilteredItems(Predicate predicate) {
     ArrayList<Item> itemsCopy = new ArrayList<Item>(items);
     CollectionUtils.filter(itemsCopy, predicate);
     return itemsCopy;
   }
 
-  /*****************************************************************************
-   * Return all registrated enumeration CAUTION : do not call remove() on this
-   * iterator, you should remove effective items
-   ****************************************************************************/
-  protected synchronized Iterator<? extends Item> getItemsIterator() {
+  /**
+   * ***************************************************************************
+   * Return all registered enumeration CAUTION : do not call remove() on this
+   * iterator, you would effectively remove items instead of using regular
+   * removeItem() primitive
+   * **************************************************************************.
+   * 
+   * @return the items iterator
+   */
+  protected Iterator<? extends Item> getItemsIterator() {
     return items.iterator();
   }
 
   /**
-   * Clear any entries from this manager
+   * Clear any entries from this manager.
    */
   public void clear() {
-    items.clear();
-    internalMap.clear();
+    lock.writeLock().lock();
+    try {
+      items.clear();
+      internalMap.clear();
+    } finally {
+      lock.writeLock().unlock();
+    }
   }
 
   /**
-   * Force files sorting after an order change for ie Called to ensure Set
+   * Force files sorting after an order change, i.e. Called to ensure Set
    * sorting contract <br>
    * We remove all items and add them all again to force sorting
    */
-  protected synchronized void forceSorting() {
-    ArrayList<Item> itemsCopy = new ArrayList<Item>(items);
-    for (Item item : itemsCopy) {
-      removeItem(item);
-    }
-    for (Item item : itemsCopy) {
-      registerItem(item);
+  public void forceSorting() {
+    lock.writeLock().lock();
+    try {
+      // first create a copy
+      ArrayList<Item> itemsCopy = new ArrayList<Item>(items);
+
+      // then remove all elements
+      clear();
+
+      // and then re-add all items again to make them correctly sorted again
+      for (Item item : itemsCopy) {
+        registerItem(item);
+      }
+    } finally {
+      lock.writeLock().unlock();
     }
   }
 
+  /**
+   * Basic implementation for item hashcode computation.
+   * 
+   * @param sName item name
+   * 
+   * @return ItemManager ID
+   */
+  protected static String createID(String sName) {
+    return MD5Processor.hash(sName);
+  }
 }
