@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2007 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -21,16 +22,29 @@
 package org.jajuk.base;
 
 /**
- * A logical Item (style, author, year, album...)
+ * A logical Item (genre, artist, year, album...)
  */
 public abstract class LogicalItem extends Item {
 
   /**
-   * @param sId
-   * @param sName
+   * The Constructor.
+   * 
+   * @param sId DOCUMENT_ME
+   * @param sName DOCUMENT_ME
    */
   LogicalItem(String sId, String sName) {
     super(sId, sName);
   }
 
+  /**
+   * toString method.
+   * 
+   * @return the string
+   */
+  @Override
+  public String toString() {
+    // Use class metadata because this method is used by several logical items
+    return getClass().getCanonicalName() + "[ID=" + getID() + " Name={{" + getName() + "}}]";
+  }
+
 }
