@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2003 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 4517 $
+ *  $Revision: 6186 $
  */
 package org.jajuk.base;
 
@@ -28,6 +29,7 @@ import java.util.Set;
 
 import javax.swing.ImageIcon;
 
+import org.apache.commons.lang.StringUtils;
 import org.jajuk.services.core.RatingManager;
 import org.jajuk.util.Conf;
 import org.jajuk.util.Const;
@@ -40,56 +42,60 @@ import org.jajuk.util.log.Log;
 /**
  * A track
  * <p>
- * Logical item
+ * Logical item.
  */
 public class Track extends LogicalItem implements Comparable<Track> {
 
-  /** Track album* */
+  /** Track album*. */
   private final Album album;
 
-  /** Track style */
-  private final Style style;
+  /** Track genre. */
+  private final Genre genre;
 
-  /** Track author in sec */
-  private final Author author;
+  /** Track artist. */
+  private final Artist artist;
 
-  /** Track length */
+  /** Track length. */
   private final long length;
 
-  /** Track year */
+  /** Track year. */
   private final Year year;
 
-  /** Track type */
+  /** Track type. */
   private final Type type;
 
-  /** Track associated files */
-  private List<File> alFiles = new ArrayList<File>(1);
+  /** Album Artist. */
+  private AlbumArtist albumArtist;
+
+  /** Track associated files. */
+  private final List<File> alFiles = new ArrayList<File>(1);
 
   /**
-   * Track constructor
+   * Track constructor.
    * 
-   * @param sId
-   * @param sName
-   * @param album
-   * @param style
-   * @param author
-   * @param length
-   * @param sYear
-   * @param type
-   * @param sAdditionDate
+   * @param sId DOCUMENT_ME
+   * @param sName DOCUMENT_ME
+   * @param album DOCUMENT_ME
+   * @param genre DOCUMENT_ME
+   * @param artist DOCUMENT_ME
+   * @param length DOCUMENT_ME
+   * @param type DOCUMENT_ME
+   * @param year DOCUMENT_ME
+   * @param lOrder DOCUMENT_ME
+   * @param lDiscNumber DOCUMENT_ME
    */
-  public Track(String sId, String sName, Album album, Style style, Author author, long length,
-      Year year, long lOrder, Type type) {
+  Track(String sId, String sName, Album album, Genre genre, Artist artist, long length, Year year,
+      long lOrder, Type type, long lDiscNumber) {
     super(sId, sName);
     // album
     this.album = album;
     setProperty(Const.XML_ALBUM, album.getID());
-    // style
-    this.style = style;
-    setProperty(Const.XML_STYLE, style.getID());
-    // author
-    this.author = author;
-    setProperty(Const.XML_AUTHOR, author.getID());
+    // genre
+    this.genre = genre;
+    setProperty(Const.XML_GENRE, genre.getID());
+    // artist
+    this.artist = artist;
+    setProperty(Const.XML_ARTIST, artist.getID());
     // Length
     this.length = length;
     setProperty(Const.XML_TRACK_LENGTH, length);
@@ -101,6 +107,8 @@ public class Track extends LogicalItem implements Comparable<Track> {
     setProperty(Const.XML_YEAR, year.getID());
     // Order
     setProperty(Const.XML_TRACK_ORDER, lOrder);
+    // Order
+    setProperty(Const.XML_TRACK_DISC_NUMBER, lDiscNumber);
     // Rate
     setProperty(Const.XML_TRACK_RATE, 0l);
     // Hits
@@ -108,21 +116,30 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * toString method
+   * toString method.
+   * 
+   * @return the string
    */
   @Override
   public String toString() {
-    String sOut = "Track[ID=" + getID() + " Name={{" + getName() + "}} " + album + " " + style
-        + " " + author + " Length=" + length + " Year=" + year.getValue() + " Rate=" + getRate()
-        + " " + type + " Hits=" + getHits() + " Addition date=" + getDiscoveryDate() + " Comment="
-        + getComment() + " order=" + getOrder() + " Nb of files=" + alFiles.size() + "]";
+    StringBuilder sOut = new StringBuilder();
+
+    sOut.append("Track[ID=").append(getID()).append(" Name={{").append(getName()).append("}} ")
+        .append(album).append(" ").append(genre).append(" ").append(artist).append(" Length=")
+        .append(length).append(" Year=").append(year.getValue()).append(" Rate=").append(getRate())
+        .append(" ").append(type).append(" Hits=").append(getHits()).append(" Addition date=")
+        .append(getDiscoveryDate()).append(" Comment=").append(getComment()).append(" order=")
+        .append(getOrder()).append(" Nb of files=").append(alFiles.size()).append(" Album artist=")
+        .append(getAlbumArtist()).append(" Disc=").append(getDiscNumber()).append("]");
     for (int i = 0; i < alFiles.size(); i++) {
-      sOut += '\n' + alFiles.get(i).toString();
+      sOut.append('\n').append(alFiles.get(i).toString());
     }
-    return sOut;
+    return sOut.toString();
   }
 
   /**
+   * Gets the any.
+   * 
    * @return a human representation of all concatenated properties
    */
   @Override
@@ -141,29 +158,62 @@ public class Track extends LogicalItem implements Comparable<Track> {
    * Default comparator for tracks, not used for sorting (use TrackComparator
    * for that) But only for storage.
    * 
-   * @param other
-   *          track to be compared
+   * @param otherTrack DOCUMENT_ME
+   * 
    * @return comparison result
    */
+  @Override
   public int compareTo(Track otherTrack) {
     return getID().compareTo(otherTrack.getID());
   }
 
   /**
-   * @return
+   * Gets the album.
+   * 
+   * @return the album
    */
   public Album getAlbum() {
     return album;
   }
 
   /**
-   * @return all associated files
+   * Gets a copy of associated files.
+   * 
+   * @return a copy of associated files
    */
   public List<org.jajuk.base.File> getFiles() {
-    return alFiles;
+    return new ArrayList<File>(alFiles);
   }
 
   /**
+   * Remove specified file from associated files.
+   * 
+   * @param file : the file to remove
+   */
+  public void removeFile(File file) {
+    alFiles.remove(file);
+  }
+
+  /**
+   * Gets the files.
+   * 
+   * @param filter files we want to deal with, null means no filter
+   * 
+   * @return all associated files
+   */
+  public List<org.jajuk.base.File> getFiles(Set<File> filter) {
+    List<File> out = new ArrayList<File>(alFiles.size());
+    for (File file : alFiles) {
+      if (filter == null || filter.contains(file)) {
+        out.add(file);
+      }
+    }
+    return out;
+  }
+
+  /**
+   * Gets the ready files.
+   * 
    * @return ready files
    */
   public List<File> getReadyFiles() {
@@ -177,9 +227,11 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
+   * Gets the ready files.
+   * 
+   * @param filter files we want to deal with, null means no filter
+   * 
    * @return ready files with given filter
-   * @param filter
-   *          files we want to deal with, null means no filter
    */
   public List<File> getReadyFiles(Set<File> filter) {
     List<File> alReadyFiles = new ArrayList<File>(alFiles.size());
@@ -192,7 +244,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * Get additionned size of all files this track map to
+   * Get sum size of all files this track map to.
    * 
    * @return the total size
    */
@@ -206,17 +258,19 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
+   * Gets the playable file.
+   * 
+   * @param bIgnoreUnmounted Do we return unmounted files
+   * 
    * @return best file to play for this track or null if none available
-   * @param bHideUnmounted
-   *          Do we return unmounted files
    */
-  public File getPlayeableFile(boolean bHideUnmounted) {
+  public File getBestFile(boolean bIgnoreUnmounted) {
     File fileOut = null;
     final List<File> alMountedFiles = new ArrayList<File>(2);
 
     // firstly, filter mounted files if needed
     for (final File file : alFiles) {
-      if (!bHideUnmounted || file.isReady()) {
+      if (!bIgnoreUnmounted || file.isReady()) {
         alMountedFiles.add(file);
       }
     }
@@ -225,6 +279,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
     } else if (alMountedFiles.size() > 0) {
       // then keep best quality and mounted first
       Collections.sort(alMountedFiles, new Comparator<File>() {
+        @Override
         public int compare(File file1, File file2) {
           long lQuality1 = file1.getQuality();
           boolean bMounted1 = file1.isReady();
@@ -249,44 +304,96 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * @return
+   * Gets the hits.
+   * 
+   * @return the hits
    */
   public long getHits() {
     return getLongValue(Const.XML_TRACK_HITS);
   }
 
   /**
-   * @return
+   * Gets the comment.
+   * 
+   * @return the comment
    */
   public String getComment() {
     return getStringValue(Const.XML_TRACK_COMMENT);
   }
 
   /**
-   * Get track number
+   * Get track number.
    * 
-   * @return
+   * @return the order
    */
   public long getOrder() {
     return getLongValue(Const.XML_TRACK_ORDER);
   }
 
   /**
-   * @return
+   * Get disc number.
+   * 
+   * @return the disc number
+   */
+  public long getDiscNumber() {
+    return getLongValue(Const.XML_TRACK_DISC_NUMBER);
+  }
+
+  /**
+   * Get album artist.
+   * 
+   * @return the album artist
+   */
+  public AlbumArtist getAlbumArtist() {
+    return albumArtist;
+  }
+
+  /**
+   * Gets the album artist or artist if album-artist is not available.
+   * 
+   * @return the albumArtist or artist if album artist not available
+   * <p>
+   * If this is various, the album artist is tried to be defined by the
+   * track artists of this album
+   * </p>
+   */
+  public String getAlbumArtistOrArtist() {
+    // If the album artist tag is provided, perfect, let's use it !
+    String albumArtist = getAlbumArtist().getName();
+    if (StringUtils.isNotBlank(albumArtist) && !(Const.UNKNOWN_ARTIST.equals(albumArtist))) {
+      return albumArtist;
+    }
+    // various artist? check if all artists are the same
+    Artist artist = getArtist();
+    if (artist == null) {
+      // Several different artist, return translated "various"
+      return Messages.getString(Const.VARIOUS_ARTIST);
+    } else {
+      // single artist, return it
+      return artist.getName2();
+    }
+  }
+
+  /**
+   * Gets the year.
+   * 
+   * @return the year
    */
   public Year getYear() {
     return year;
   }
 
   /**
+   * Gets the duration.
+   * 
    * @return length in sec
    */
   public long getDuration() {
     return length;
   }
 
-  /**
-   * @return
+  /* (non-Javadoc)
+   * @see org.jajuk.base.Item#getRate()
    */
   @Override
   public long getRate() {
@@ -294,38 +401,46 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
+   * Gets the discovery date.
+   * 
    * @return the date where the track has been discovered (added into the
-   *         collection)
+   * collection)
    */
   public Date getDiscoveryDate() {
     return getDateValue(Const.XML_TRACK_DISCOVERY_DATE);
   }
 
   /**
-   * @return
+   * Gets the type.
+   * 
+   * @return the type
    */
   public Type getType() {
     return type;
   }
 
   /**
-   * @return
-   */
-  public Author getAuthor() {
-    return author;
-  }
-
-  /**
-   * @return
-   */
-  public Style getStyle() {
-    return style;
-  }
-
-  /**
-   * Add an associated file
+   * Gets the artist.
    * 
-   * @param file
+   * @return the artist
+   */
+  public Artist getArtist() {
+    return artist;
+  }
+
+  /**
+   * Gets the genre.
+   * 
+   * @return the genre
+   */
+  public Genre getGenre() {
+    return genre;
+  }
+
+  /**
+   * Add an associated file.
+   * 
+   * @param file DOCUMENT_ME
    */
   public void addFile(File file) {
     // make sure a file will be referenced by only one track (first found)
@@ -335,17 +450,9 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * Remove an associated file
+   * Sets the hits.
    * 
-   * @param file
-   */
-  public void removeFile(File file) {
-    alFiles.remove(file);
-  }
-
-  /**
-   * @param hits
-   *          The iHits to set.
+   * @param hits The iHits to set.
    */
   public void setHits(long hits) {
     setProperty(Const.XML_TRACK_HITS, hits);
@@ -356,7 +463,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * Increase playcount number
+   * Increase playcount number.
    */
   public void incHits() {
     long value = getHits() + 1;
@@ -368,8 +475,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
    * +2=love +3=crazy). The preference is a factor given by the user to increase
    * or decrease a track rate.
    * 
-   * @param preference
-   *          from -3 to 3
+   * @param preference from -3 to 3
    */
   public void setPreference(long preference) {
     Log.debug("Changed preference of " + getID() + "=" + preference);
@@ -383,7 +489,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * Compute final track rate
+   * Compute final track rate.
    * 
    * @see http://trac.jajuk.info/ticket/1179
    */
@@ -401,7 +507,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
       float playtimeRate = 0.5f;
       if (duration == 0) {
         // If duration = 0, always set playtimeRate to 0.5
-        Log.info("Duration = 0 for: " + getName() + ". Playtime forced to 0.5");
+        Log.info("Duration = 0 for: {{" + getName() + "}}. Playtime forced to 0.5");
       } else {
         // Compute playtime rate = total play time / (play count * track length)
         playtimeRate = (float) getLongValue(Const.XML_TRACK_TOTAL_PLAYTIME)
@@ -411,7 +517,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
       // precision issue or if user seeks back into the track
       // set =1.
       if (playtimeRate > 1) {
-        Log.warn("Playtime rate > 1 for: " + getName() + " value=" + playtimeRate);
+        Log.warn("Playtime rate > 1 for: {{" + getName() + "}} value=" + playtimeRate);
         // We reset tpt and hits to
         // make things clear and to avoid increasing the error with time
         setProperty(Const.XML_TRACK_TOTAL_PLAYTIME, duration * playcount);
@@ -446,8 +552,9 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * @param rate
-   *          The lRate to set.
+   * Sets the rate.
+   * 
+   * @param rate The lRate to set.
    */
   protected void setRate(long rate) {
     setProperty(Const.XML_TRACK_RATE, rate);
@@ -455,28 +562,41 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * @param rate
-   *          The lRate to set.
+   * Sets the comment.
+   * 
+   * @param sComment DOCUMENT_ME
    */
   public void setComment(String sComment) {
     setProperty(Const.XML_TRACK_COMMENT, sComment);
   }
 
   /**
-   * @param additionDate
-   *          The sAdditionDate to set.
+   * Sets the album artist.
+   * 
+   * @param albumArtist : the album artist
+   */
+  public void setAlbumArtist(AlbumArtist albumArtist) {
+    this.albumArtist = albumArtist;
+    // We store the album-artist ID string, not the album-artist itself
+    setProperty(Const.XML_ALBUM_ARTIST, albumArtist.getID());
+  }
+
+  /**
+   * Sets the discovery date.
+   * 
+   * @param additionDate The sAdditionDate to set.
    */
   public void setDiscoveryDate(Date additionDate) {
     setProperty(Const.XML_TRACK_DISCOVERY_DATE, additionDate);
   }
 
   /**
-   * Return whether this item should be hidden with hide option
+   * Return whether this item should be hidden with hide option.
    * 
    * @return whether this item should be hidden with hide option
    */
   public boolean shouldBeHidden() {
-    if (getPlayeableFile(true) != null || !Conf.getBoolean(Const.CONF_OPTIONS_HIDE_UNMOUNTED)) {
+    if (getBestFile(true) != null || !Conf.getBoolean(Const.CONF_OPTIONS_HIDE_UNMOUNTED)) {
       return false;
     }
     return true;
@@ -493,7 +613,9 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
-   * Get item description
+   * Get item description.
+   * 
+   * @return the desc
    */
   @Override
   public String getDesc() {
@@ -513,22 +635,29 @@ public class Track extends LogicalItem implements Comparable<Track> {
         return lAlbum.getName2();
       }
       return null;
-    } else if (Const.XML_AUTHOR.equals(sKey)) {
-      Author lAuthor = AuthorManager.getInstance().getAuthorByID(getStringValue(sKey));
-      if (lAuthor != null) { // can be null after a fresh change
-        return lAuthor.getName2();
+    } else if (Const.XML_ARTIST.equals(sKey)) {
+      Artist artist = ArtistManager.getInstance().getArtistByID(getStringValue(sKey));
+      if (artist != null) { // can be null after a fresh change
+        return artist.getName2();
       }
       return null;
-    } else if (Const.XML_STYLE.equals(sKey)) {
-      Style lStyle = StyleManager.getInstance().getStyleByID(getStringValue(sKey));
-      if (lStyle != null) { // can be null after a fresh change
-        return lStyle.getName2();
+    } else if (Const.XML_ALBUM_ARTIST.equals(sKey)) {
+      AlbumArtist albumArtist = AlbumArtistManager.getInstance().getAlbumArtistByID(
+          getStringValue(sKey));
+      if (albumArtist != null) { // can be null after a fresh change
+        return albumArtist.getName2();
+      }
+      return null;
+    } else if (Const.XML_GENRE.equals(sKey)) {
+      Genre genre = GenreManager.getInstance().getGenreByID(getStringValue(sKey));
+      if (genre != null) { // can be null after a fresh change
+        return genre.getName2();
       }
       return null;
     } else if (Const.XML_TRACK_LENGTH.equals(sKey)) {
       return UtilString.formatTimeBySec(length);
     } else if (Const.XML_TYPE.equals(sKey)) {
-      return (TypeManager.getInstance().getTypeByID(getStringValue(sKey))).getName();
+      return TypeManager.getInstance().getTypeByID(getStringValue(sKey)).getName();
     } else if (Const.XML_YEAR.equals(sKey)) {
       return getStringValue(sKey);
     } else if (Const.XML_FILES.equals(sKey)) {
@@ -538,8 +667,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
         sbOut.append(file.getAbsolutePath());
         sbOut.append(',');
       }
-      return sbOut.substring(0, sbOut.length() - 1); // remove last
-      // ','
+      return sbOut.substring(0, sbOut.length() - 1); // remove trailing coma
     } else if (Const.XML_TRACK_DISCOVERY_DATE.equals(sKey)) {
       return UtilString.getLocaleDateFormatter().format(getDiscoveryDate());
     } else if (Const.XML_ANY.equals(sKey)) {
@@ -560,6 +688,7 @@ public class Track extends LogicalItem implements Comparable<Track> {
   }
 
   /**
+   * Gets the files string.
    * 
    * @return a list of associated files in format : file1,file2...
    */
@@ -573,5 +702,4 @@ public class Track extends LogicalItem implements Comparable<Track> {
     sb.deleteCharAt(sb.length() - 1);
     return sb.toString();
   }
-
 }
