@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2003 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 4517 $
+ *  $Revision: 6186 $
  */
 package org.jajuk.base;
 
@@ -23,27 +24,60 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 
+import org.jajuk.services.players.IPlayerImpl;
+import org.jajuk.services.tags.ITagImpl;
 import org.jajuk.util.Const;
 import org.jajuk.util.Messages;
 import org.jajuk.util.ReadOnlyIterator;
 import org.jajuk.util.UtilSystem;
-import org.jajuk.util.log.Log;
 
 /**
  * Manages types ( mp3, ogg...) supported by jajuk
  * <p>
  */
 public final class TypeManager extends ItemManager {
-  /** extensions->types */
-  private Map<String, Type> hmSupportedTypes = new HashMap<String, Type>(10);
 
-  /** Self instance */
-  private static TypeManager singleton;
+  /** The Constant ICON_16X16_VIDEO for the supported media types. */
+  private static final String ICON_16X16_VIDEO = "icons/16x16/type_video_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_APE for the supported media types. */
+  private static final String ICONS_16X16_TYPE_APE = "icons/16x16/type_ape_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_MP2 for the supported media types. */
+  private static final String ICONS_16X16_TYPE_MP2 = "icons/16x16/type_mp2_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_AAC for the supported media types. */
+  private static final String ICONS_16X16_TYPE_AAC = "icons/16x16/type_aac_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_WMA for the supported media types. */
+  private static final String ICONS_16X16_TYPE_WMA = "icons/16x16/type_wma_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_FLAC for the supported media types. */
+  private static final String ICONS_16X16_TYPE_FLAC = "icons/16x16/type_flac_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_RAM for the supported media types. */
+  private static final String ICONS_16X16_TYPE_RAM = "icons/16x16/type_ram_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_OGG for the supported media types. */
+  private static final String ICONS_16X16_TYPE_OGG = "icons/16x16/type_ogg_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_WAV for the supported media types. */
+  private static final String ICONS_16X16_TYPE_WAV = "icons/16x16/type_wav_16x16.png";
+
+  /** The Constant ICONS_16X16_TYPE_MP3 for the supported media types. */
+  private static final String ICONS_16X16_TYPE_MP3 = "icons/16x16/type_mp3_16x16.png";
+
+  /** extensions->types. */
+  private final Map<String, Type> hmSupportedTypes = new HashMap<String, Type>(10);
+
+  /** Self instance. */
+  private static TypeManager singleton = new TypeManager();
 
   /**
-   * No constructor available, only static access
+   * No constructor available, only static access.
    */
   private TypeManager() {
     super();
@@ -75,74 +109,73 @@ public final class TypeManager extends ItemManager {
   }
 
   /**
+   * Gets the instance.
+   * 
    * @return singleton
    */
   public static TypeManager getInstance() {
-    if (singleton == null) {
-      singleton = new TypeManager();
-    }
     return singleton;
   }
 
   /**
-   * Register a type jajuk can read
+   * Register a type jajuk can read.
    * 
-   * @param type
-   */
-  public Type registerType(String sName, String sExtension, Class<?> cPlayerImpl, Class<?> cTagImpl) {
-    return registerType(sExtension.toLowerCase(), sName, sExtension, cPlayerImpl, cTagImpl);
-  }
-
-  /**
-   * Register a type jajuk can read with a known id
+   * @param sName The human readable name of the type.
+   * @param sExtension The extension for the type, without a dot.
+   * @param cPlayerImpl The class used to play files of this type. Needs to implement the
+   * interface IPlayerImpl, @see IPlayerImpl
+   * @param cTagImpl The class used to retrieve and write tags for this file type.
+   * Needs to implement the interface ITagImpl, @see ITagImpl
    * 
-   * @param type
+   * @return the type
    */
   @SuppressWarnings("unchecked")
-  private Type registerType(String sId, String sName, String sExtension, Class cPlayerImpl,
-      Class cTagImpl) {
+  public Type registerType(String sName, String sExtension, Class<?> cPlayerImpl, Class<?> cTagImpl) {
+    String sId = sExtension.toLowerCase(Locale.getDefault());
     Type type = getTypeByID(sId);
     if (type != null) {
       return type;
     }
+    // TODO: can this happen at all?? I think the previous "if" already covers
+    // this case...
     if (hmSupportedTypes.containsKey(sExtension)) {
       // if the type is already in memory, use it
       return hmSupportedTypes.get(sExtension);
     }
-    try {
-      type = new Type(sId, sName, sExtension, cPlayerImpl, cTagImpl);
-      registerItem(type);
-      hmSupportedTypes.put(type.getExtension(), type);
-    } catch (Exception e) {
-      Log.error(109, "sPlayerImpl=" + cPlayerImpl + " sTagImpl=" + cTagImpl, e);
-    }
+
+    type = new Type(sId, sName, sExtension, (Class<IPlayerImpl>) cPlayerImpl,
+        (Class<ITagImpl>) cTagImpl);
+    registerItem(type);
+    hmSupportedTypes.put(type.getExtension(), type);
     return type;
   }
 
   /**
-   * Tells if the type is supported
+   * Tells if the type is supported.
    * 
-   * @param type
-   * @return
+   * @param sExt The extension to check (without leading dot)
+   * 
+   * @return true, if checks if is extension supported
    */
   public boolean isExtensionSupported(String sExt) {
-    return hmSupportedTypes.containsKey(sExt.toLowerCase());
+    return hmSupportedTypes.containsKey(sExt.toLowerCase(Locale.getDefault()));
   }
 
   /**
-   * Return type for a given extension
+   * Return type for a given extension.
    * 
-   * @param sExtension
-   * @return
+   * @param sExtension The extension to retrieve (without leading dot)
+   * 
+   * @return the type by extension
    */
   public Type getTypeByExtension(String sExtension) {
-    return hmSupportedTypes.get(sExtension.toLowerCase());
+    return hmSupportedTypes.get(sExtension.toLowerCase(Locale.getDefault()));
   }
 
   /**
-   * Return all music types
+   * Return all music types.
    * 
-   * @return
+   * @return the all music types
    */
   public List<Type> getAllMusicTypes() {
     List<Type> alResu = new ArrayList<Type>(5);
@@ -157,11 +190,15 @@ public final class TypeManager extends ItemManager {
   }
 
   /**
-   * Return a list "a,b,c" of registered extensions, used by FileChooser
+   * Return a list "a,b,c" of registered extensions, used by FileChooser.
    * 
-   * @return
+   * @return the types as one string.
    */
   public String getTypeListString() {
+    if (hmSupportedTypes.isEmpty()) {
+      return "";
+    }
+
     StringBuilder sb = new StringBuilder();
     Iterator<String> it = hmSupportedTypes.keySet().iterator();
     while (it.hasNext()) {
@@ -174,7 +211,7 @@ public final class TypeManager extends ItemManager {
 
   /*
    * (non-Javadoc)
-   * 
+   *
    * @see org.jajuk.base.ItemManager#getIdentifier()
    */
   @Override
@@ -183,8 +220,10 @@ public final class TypeManager extends ItemManager {
   }
 
   /**
-   * @param sID
-   *          Item ID
+   * Gets the type by id.
+   * 
+   * @param sID Item ID
+   * 
    * @return item
    */
   public Type getTypeByID(String sID) {
@@ -192,20 +231,22 @@ public final class TypeManager extends ItemManager {
   }
 
   /**
+   * Gets the types.
    * 
    * @return types list
    */
   @SuppressWarnings("unchecked")
-  public synchronized List<Type> getTypes() {
+  public List<Type> getTypes() {
     return (List<Type>) getItems();
   }
 
   /**
+   * Gets the types iterator.
    * 
    * @return types iterator
    */
   @SuppressWarnings("unchecked")
-  public synchronized ReadOnlyIterator<Type> getTypesIterator() {
+  public ReadOnlyIterator<Type> getTypesIterator() {
     return new ReadOnlyIterator<Type>((Iterator<Type>) getItemsIterator());
   }
 
@@ -216,9 +257,9 @@ public final class TypeManager extends ItemManager {
    * all icons at startup, we do it asynchronously to accelerate startup
    * </p>
    * 
-   * @throws Exception
+   * @throws ClassNotFoundException the class not found exception
    */
-  public static void registerTypesNoMplayer() throws Exception {
+  public static void registerTypesNoMplayer() throws ClassNotFoundException {
     // mp3
     Type type = TypeManager.getInstance().registerType(Messages.getString("Type.mp3"),
         Const.EXT_MP3, Class.forName(Const.PLAYER_IMPL_JAVALAYER),
@@ -227,7 +268,7 @@ public final class TypeManager extends ItemManager {
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
     // Do not use IconLoader to get icon, it takes too much time to
     // load all icons
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_mp3_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_MP3)
         .toExternalForm());
     // playlists
     type = TypeManager.getInstance().registerType(Messages.getString("Type.playlist"),
@@ -239,21 +280,21 @@ public final class TypeManager extends ItemManager {
         Class.forName(Const.PLAYER_IMPL_JAVALAYER), Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, false);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_ogg_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_OGG)
         .toExternalForm());
     // Wave
     type = TypeManager.getInstance().registerType(Messages.getString("Type.wav"), Const.EXT_WAV,
         Class.forName(Const.PLAYER_IMPL_JAVALAYER), Class.forName(Const.TAG_IMPL_NO_TAGS));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_wav_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WAV)
         .toExternalForm());
     // au
     type = TypeManager.getInstance().registerType(Messages.getString("Type.au"), Const.EXT_AU,
         Class.forName(Const.PLAYER_IMPL_JAVALAYER), Class.forName(Const.TAG_IMPL_NO_TAGS));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, false);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_wav_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WAV)
         .toExternalForm());
   }
 
@@ -264,16 +305,16 @@ public final class TypeManager extends ItemManager {
    * all icons at startup, we do it asynchronously to accelerate startup
    * </p>
    * 
-   * @throws Exception
-   */
-  public static void registerTypesMplayerAvailable() throws Exception {
+   * @throws ClassNotFoundException the class not found exception
+   * @throws Exception    */
+  public static void registerTypesMplayerAvailable() throws ClassNotFoundException {
     // mp3
     Type type = TypeManager.getInstance().registerType(Messages.getString("Type.mp3"),
         Const.EXT_MP3, Class.forName(Const.PLAYER_IMPL_MPLAYER),
         Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_mp3_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_MP3)
         .toExternalForm());
     // playlists
     type = TypeManager.getInstance().registerType(Messages.getString("Type.playlist"),
@@ -285,42 +326,56 @@ public final class TypeManager extends ItemManager {
         Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_ogg_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_OGG)
         .toExternalForm());
     // Wav
     type = TypeManager.getInstance().registerType(Messages.getString("Type.wav"), Const.EXT_WAV,
         Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_NO_TAGS));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_wav_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WAV)
         .toExternalForm());
     // au
     type = TypeManager.getInstance().registerType(Messages.getString("Type.au"), Const.EXT_AU,
         Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_NO_TAGS));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_wav_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WAV)
+        .toExternalForm());
+    // aiff
+    type = TypeManager.getInstance().registerType(Messages.getString("Type.aiff"), Const.EXT_AIFF,
+        Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_NO_TAGS));
+    type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
+    type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WAV)
+        .toExternalForm());
+    // aiff
+    type = TypeManager.getInstance().registerType(Messages.getString("Type.aiff"), Const.EXT_AIF,
+        Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_NO_TAGS));
+    type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
+    type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WAV)
         .toExternalForm());
     // flac
     type = TypeManager.getInstance().registerType(Messages.getString("Type.flac"), Const.EXT_FLAC,
         Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_flac_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_FLAC)
         .toExternalForm());
     // WMA
     type = TypeManager.getInstance().registerType(Messages.getString("Type.wma"), Const.EXT_WMA,
         Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_wma_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WMA)
         .toExternalForm());
     // M4A
     type = TypeManager.getInstance().registerType(Messages.getString("Type.aac"), Const.EXT_M4A,
         Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_aac_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_AAC)
         .toExternalForm());
     // Real audio (.rm)
     type = TypeManager.getInstance().registerType(Messages.getString("Type.real"),
@@ -328,7 +383,7 @@ public final class TypeManager extends ItemManager {
         Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_ram_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_RAM)
         .toExternalForm());
     // Real audio (.ra)
     type = TypeManager.getInstance().registerType(Messages.getString("Type.real"),
@@ -336,14 +391,14 @@ public final class TypeManager extends ItemManager {
         Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_ram_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_RAM)
         .toExternalForm());
     // mp2
     type = TypeManager.getInstance().registerType(Messages.getString("Type.mp2"), Const.EXT_MP2,
         Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, true);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_mp2_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_MP2)
         .toExternalForm());
     // web radios
     type = TypeManager.getInstance().registerType(Messages.getString("Type.radio"),
@@ -355,14 +410,14 @@ public final class TypeManager extends ItemManager {
         Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_ape_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_APE)
         .toExternalForm());
     // MAC = APE
     type = TypeManager.getInstance().registerType(Messages.getString("Type.mac"), Const.EXT_MAC,
         Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_ape_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_APE)
         .toExternalForm());
     // MPC
     type = TypeManager.getInstance().registerType(Messages.getString("Type.mpc"), Const.EXT_MPC,
@@ -370,7 +425,7 @@ public final class TypeManager extends ItemManager {
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
     // Change the MPC icon here if you find one
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_mp3_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_MP3)
         .toExternalForm());
     // MP+
     type = TypeManager.getInstance().registerType(Messages.getString("Type.mpc"), Const.EXT_MPPLUS,
@@ -378,7 +433,7 @@ public final class TypeManager extends ItemManager {
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
     // Change the MPC icon here if you find one
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_mp3_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_MP3)
         .toExternalForm());
     // MPP
     type = TypeManager.getInstance().registerType(Messages.getString("Type.mpc"), Const.EXT_MPP,
@@ -386,7 +441,7 @@ public final class TypeManager extends ItemManager {
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
     // Change the MPC icon here if you find one
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_mp3_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_MP3)
         .toExternalForm());
     // WavPack
     type = TypeManager.getInstance().registerType(Messages.getString("Type.wavpack"), Const.EXT_WV,
@@ -395,73 +450,97 @@ public final class TypeManager extends ItemManager {
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
     // Official logo contains text and doesn't display well in 16x16, take wav
     // logo
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource("icons/16x16/type_wav_16x16.png")
+    type.setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICONS_16X16_TYPE_WAV)
         .toExternalForm());
 
     // -- VIDEO --
     // AVI
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_AVI,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_AVI, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // mpg
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_MPG,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_MPG, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // MP4
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_MP4,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_MP4, Class.forName(Const.PLAYER_IMPL_MPLAYER),
+        Class.forName(Const.TAG_IMPL_JAUDIOTAGGER));
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // mpeg
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_MPEG,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_MPEG, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // mkv
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_MKV,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_MKV, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // asf
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_ASF,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_ASF, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // wmv
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_WMV,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_WMV, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // mov
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_MOV,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_MOV, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
     // ogm
-    type = TypeManager.getInstance().registerType(Messages.getString("Type.video"), Const.EXT_OGM,
-        Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_OGM, Class.forName(Const.PLAYER_IMPL_MPLAYER), null);
     type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
     type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
-    type.setProperty(Const.XML_TYPE_ICON, UtilSystem
-        .getResource("icons/16x16/type_video_16x16.png").toExternalForm());
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
+    // FLV - Flash Movie
+    type = TypeManager.getInstance().registerType(Messages.getString(Const.TYPE_VIDEO),
+        Const.EXT_FLV, Class.forName(Const.PLAYER_IMPL_MPLAYER),
+        Class.forName(Const.TAG_IMPL_NO_TAGS));
+    type.setProperty(Const.XML_TYPE_IS_MUSIC, true);
+    type.setProperty(Const.XML_TYPE_SEEK_SUPPORTED, Const.TRUE);
+    type
+        .setProperty(Const.XML_TYPE_ICON, UtilSystem.getResource(ICON_16X16_VIDEO).toExternalForm());
   }
 
+  /*
+   * (non-Javadoc)
+   *
+   * @see org.jajuk.base.ItemManager#clear()
+   */
+  @Override
+  public void clear() {
+    lock.writeLock().lock();
+    try {
+      hmSupportedTypes.clear();
+      super.clear();
+    } finally {
+      lock.writeLock().unlock();
+    }
+  }
 }
