@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2006 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,7 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 2164 $
+ *  $Revision$
  */
 
 package org.jajuk.services.reporting;
@@ -30,62 +31,70 @@ import java.util.TreeSet;
 
 import org.jajuk.base.Album;
 import org.jajuk.base.AlbumManager;
-import org.jajuk.base.Author;
-import org.jajuk.base.AuthorManager;
+import org.jajuk.base.Artist;
+import org.jajuk.base.ArtistManager;
 import org.jajuk.base.Device;
 import org.jajuk.base.DeviceManager;
 import org.jajuk.base.Directory;
 import org.jajuk.base.DirectoryManager;
+import org.jajuk.base.Genre;
+import org.jajuk.base.GenreManager;
 import org.jajuk.base.Item;
-import org.jajuk.base.Style;
-import org.jajuk.base.StyleManager;
 import org.jajuk.base.Track;
 import org.jajuk.base.TrackManager;
 import org.jajuk.base.Year;
+import org.jajuk.services.core.SessionService;
 import org.jajuk.util.Const;
 import org.jajuk.util.Messages;
 import org.jajuk.util.UtilString;
-import org.jajuk.util.UtilSystem;
 
 /**
  * This class exports music contents to XML.
  */
 public class XMLExporter extends Exporter {
 
-  /** Private Constants */
+  /** Private Constants. */
   private static final String NEWLINE = "\n";
 
+  /** The Constant XML_HEADER.  DOCUMENT_ME */
   private static final String XML_HEADER = "<?xml version='1.0' encoding='UTF-8'?>";
 
-  private BufferedWriter writer;
+  /** DOCUMENT_ME. */
+  private final BufferedWriter writer;
 
-  /** Do we want to export tracks ?* */
+  /** Do we want to export tracks ?*. */
   private boolean showTracks = true;
 
-  /** PUBLIC METHODS */
+  /**
+   * PUBLIC METHODS.
+   * 
+   * @throws IOException Signals that an I/O exception has occurred.
+   */
 
   public XMLExporter() throws IOException {
-    cache = UtilSystem.getConfFileByPath(Const.FILE_REPORTING_CACHE_FILE + "_XML_"
+    cache = SessionService.getConfFileByPath(Const.FILE_REPORTING_CACHE_FILE + "_XML_"
         + System.currentTimeMillis());
     writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(cache, false), "UTF-8"));
   }
 
   /**
-   * This method will create a tagging of the specified item
+   * This method will create a tagging of the specified item.
+   * 
+   * @param item The item to report (can be an album, a year, an artist, a genre, a
+   * directory or a device)
    * 
-   * @param item
-   *          The item to report (can be an album, a year, an author ,a style, a
-   *          directory or a device)
    * @return Returns a string containing the report, or null if an error
-   *         occurred.
+   * occurred.
+   * 
+   * @throws Exception the exception
    */
   public void process(Item item) throws Exception {
     if (item instanceof Album) {
       process((Album) item);
-    } else if (item instanceof Author) {
-      process((Author) item);
-    } else if (item instanceof Style) {
-      process((Style) item);
+    } else if (item instanceof Artist) {
+      process((Artist) item);
+    } else if (item instanceof Genre) {
+      process((Genre) item);
     } else if (item instanceof Year) {
       process((Year) item);
     } else if (item instanceof Directory) {
@@ -98,10 +107,12 @@ public class XMLExporter extends Exporter {
   /**
    * This method will create a tagging of the specified album and its tracks.
    * 
-   * @param album
-   *          The album to tag.
+   * @param album The album to tag.
+   * 
    * @return Returns a string containing the tagging, or null if an error
-   *         occurred.
+   * occurred.
+   * 
+   * @throws Exception the exception
    */
   public void process(Album album) throws Exception {
     // Make sure we have an album.
@@ -114,10 +125,12 @@ public class XMLExporter extends Exporter {
    * This method will create a reporting of the specified year and its albums
    * and associated tracks.
    * 
-   * @param year
-   *          The year to report.
+   * @param year The year to report.
+   * 
    * @return Returns a string containing the report, or null if an error
-   *         occurred.
+   * occurred.
+   * 
+   * @throws Exception the exception
    */
   public void process(Year year) throws Exception {
     if (year != null) {
@@ -126,31 +139,35 @@ public class XMLExporter extends Exporter {
   }
 
   /**
-   * This method will create a tagging of the specified author and its albums
+   * This method will create a tagging of the specified artist and its albums
    * and associated tracks.
    * 
-   * @param author
-   *          The author to tag.
+   * @param artist The artist to tag.
+   * 
    * @return Returns a string containing the tagging, or null if an error
-   *         occurred.
+   * occurred.
+   * 
+   * @throws Exception the exception
    */
-  public void process(Author author) throws Exception {
-    if (author != null) {
-      tagAuthor(author, 0);
+  public void process(Artist artist) throws Exception {
+    if (artist != null) {
+      tagArtist(artist, 0);
     }
   }
 
   /**
-   * This method will create a tagging of the specified style.
+   * This method will create a tagging of the specified genre.
+   * 
+   * @param genre The genre to tag.
    * 
-   * @param style
-   *          The style to tag.
    * @return Returns a string containing the tagging, or null is an error
-   *         occurred.
+   * occurred.
+   * 
+   * @throws Exception the exception
    */
-  public void process(Style style) throws Exception {
-    if (style != null) {
-      tagStyle(style, 0);
+  public void process(Genre genre) throws Exception {
+    if (genre != null) {
+      tagGenre(genre, 0);
     }
   }
 
@@ -158,10 +175,12 @@ public class XMLExporter extends Exporter {
    * This method will create a tagging of a directory and all its children files
    * and directories.
    * 
-   * @param directory
-   *          The directory to start from.
+   * @param directory The directory to start from.
+   * 
    * @return Returns a string containing the tagging, or null if an error
-   *         occurred.
+   * occurred.
+   * 
+   * @throws Exception the exception
    */
   public void process(Directory directory) throws Exception {
     if (directory != null) {
@@ -173,10 +192,12 @@ public class XMLExporter extends Exporter {
    * This method will create a tagging of a device and all its children files
    * and directories.
    * 
-   * @param device
-   *          The device to start from.
+   * @param device The device to start from.
+   * 
    * @return Returns a string containing the tagging, or null if an error
-   *         occurred.
+   * occurred.
+   * 
+   * @throws Exception the exception
    */
   public void process(Device device) throws Exception {
     if (device != null) {
@@ -185,22 +206,35 @@ public class XMLExporter extends Exporter {
   }
 
   /**
+   * Process collection.
+   * 
+   * @param type DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   * 
    * @see Exporter.processColllection
    */
   @Override
   @SuppressWarnings("unchecked")
   public void processCollection(int type) throws Exception {
     // If we are tagging the physical collection...
-    if (type == XMLExporter.PHYSICAL_COLLECTION) {
+    if (type == Exporter.PHYSICAL_COLLECTION) {
       // Same effect than selecting all devices
       process((List<Item>) DeviceManager.getInstance().getItems());
     } else if (type == LOGICAL_COLLECTION) {
-      // Same effect than selecting all styles
-      process((List<Item>) StyleManager.getInstance().getItems());
+      // Same effect than selecting all genres
+      process((List<Item>) GenreManager.getInstance().getItems());
     }
   }
 
-  /** PRIVATE HELPER METHODS */
+  /**
+   * PRIVATE HELPER METHODS.
+   * 
+   * @param level DOCUMENT_ME
+   * @param directory DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
 
   private void exportDirectoryHelper(int level, Directory directory) throws Exception {
     // Get the children
@@ -224,6 +258,15 @@ public class XMLExporter extends Exporter {
     writer.write(addTabs(level) + Tag.closeTag(Const.XML_DIRECTORY) + NEWLINE);
   }
 
+  /**
+   * Tag file.
+   * DOCUMENT_ME
+   * 
+   * @param file DOCUMENT_ME
+   * @param level DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
   private void tagFile(org.jajuk.base.File file, int level) throws Exception {
     String sFileID = file.getID();
     String sName = UtilString.formatXML(file.getName());
@@ -238,6 +281,14 @@ public class XMLExporter extends Exporter {
     writer.write(addTabs(level) + Tag.closeTag(Const.XML_FILE) + NEWLINE);
   }
 
+  /**
+   * Tag directory.
+   * DOCUMENT_ME
+   * 
+   * @param directory DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
   private void tagDirectory(Directory directory) throws Exception {
     // Make sure we have a directory.
     if (directory != null) {
@@ -263,6 +314,14 @@ public class XMLExporter extends Exporter {
     }
   }
 
+  /**
+   * Tag device.
+   * DOCUMENT_ME
+   * 
+   * @param device DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
   private void tagDevice(Device device) throws Exception {
     String sID = device.getID();
     writer.write(Tag.openTag(Const.XML_DEVICE) + NEWLINE);
@@ -289,21 +348,31 @@ public class XMLExporter extends Exporter {
     writer.write(Tag.closeTag(Const.XML_DEVICE) + NEWLINE);
   }
 
+  /**
+   * Tag track.
+   * DOCUMENT_ME
+   * 
+   * @param track DOCUMENT_ME
+   * @param level DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
   private void tagTrack(Track track, int level) throws Exception {
     String sTrackID = track.getID();
     String sTrackName = UtilString.formatXML(track.getName());
-    String sTrackStyle = UtilString.formatXML(track.getStyle().getName2());
-    String sTrackAuthor = UtilString.formatXML(track.getAuthor().getName2());
+    String sTrackGenre = UtilString.formatXML(track.getGenre().getName2());
+    String sTrackArtist = UtilString.formatXML(track.getArtist().getName2());
     String sTrackAlbum = UtilString.formatXML(track.getAlbum().getName2());
     long lTrackLength = track.getDuration();
     long lTrackRate = track.getRate();
     String sTrackComment = UtilString.formatXML(track.getComment());
     long lTrackOrder = track.getOrder();
+    long lTrackDiscNumber = track.getDiscNumber();
     writer.write(addTabs(level) + Tag.openTag(Const.XML_TRACK) + NEWLINE);
     writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_ID, sTrackID) + NEWLINE);
     writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_TRACK_NAME, sTrackName) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_TRACK_STYLE, sTrackStyle) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_TRACK_AUTHOR, sTrackAuthor) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_TRACK_GENRE, sTrackGenre) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_TRACK_ARTIST, sTrackArtist) + NEWLINE);
     writer.write(addTabs(level + 1)
         + Tag.tagData(Const.XML_TRACK_LENGTH, UtilString.formatTimeBySec(lTrackLength)) + NEWLINE);
     writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_TRACK_RATE, lTrackRate) + NEWLINE);
@@ -312,26 +381,38 @@ public class XMLExporter extends Exporter {
     writer.write(addTabs(level + 1)
         + Tag.tagData(Const.XML_TRACK_ORDER, UtilString.padNumber(lTrackOrder, 2)) + NEWLINE);
     writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_TRACK_ALBUM, sTrackAlbum) + NEWLINE);
+    writer.write(addTabs(level + 1)
+        + Tag.tagData(Const.XML_TRACK_DISC_NUMBER, UtilString.padNumber(lTrackDiscNumber, 2))
+        + NEWLINE);
     writer.write(addTabs(level) + Tag.closeTag(Const.XML_TRACK) + NEWLINE);
   }
 
+  /**
+   * Tag album.
+   * DOCUMENT_ME
+   * 
+   * @param album DOCUMENT_ME
+   * @param level DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
   private void tagAlbum(Album album, int level) throws Exception {
     String sAlbumID = album.getID();
     String sAlbumName = UtilString.formatXML(album.getName2());
-    String sStyleName = "";
-    String sAuthorName = "";
+    String sGenreName = "";
+    String sArtistName = "";
     String sYear = "";
     List<Track> tracks = TrackManager.getInstance().getAssociatedTracks(album, true);
     if (tracks.size() > 0) {
-      sStyleName = UtilString.formatXML(tracks.iterator().next().getStyle().getName2());
-      sAuthorName = UtilString.formatXML(tracks.iterator().next().getAuthor().getName2());
+      sGenreName = UtilString.formatXML(tracks.iterator().next().getGenre().getName2());
+      sArtistName = UtilString.formatXML(tracks.iterator().next().getArtist().getName2());
       sYear = tracks.iterator().next().getYear().getName2();
     }
     writer.write(addTabs(level) + Tag.openTag(Const.XML_ALBUM) + NEWLINE);
     writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_ID, sAlbumID) + NEWLINE);
     writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_NAME, sAlbumName) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_AUTHOR, sAuthorName) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_STYLE, sStyleName) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_ARTIST, sArtistName) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_GENRE, sGenreName) + NEWLINE);
     writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_YEAR, sYear) + NEWLINE);
     // For full collection, we don't show detailed tracks for performance
     // reasons
@@ -343,19 +424,37 @@ public class XMLExporter extends Exporter {
     writer.write(addTabs(level) + Tag.closeTag(Const.XML_ALBUM) + NEWLINE);
   }
 
-  private void tagAuthor(Author author, int level) throws Exception {
-    String sAuthorID = author.getID();
-    String sAuthorName = UtilString.formatXML(author.getName2());
-    writer.write(addTabs(level) + Tag.openTag(Const.XML_AUTHOR) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_ID, sAuthorID) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_NAME, sAuthorName) + NEWLINE);
-    List<Album> albums = AlbumManager.getInstance().getAssociatedAlbums(author);
+  /**
+   * Tag artist.
+   * DOCUMENT_ME
+   * 
+   * @param artist DOCUMENT_ME
+   * @param level DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
+  private void tagArtist(Artist artist, int level) throws Exception {
+    String sArtistID = artist.getID();
+    String sArtistName = UtilString.formatXML(artist.getName2());
+    writer.write(addTabs(level) + Tag.openTag(Const.XML_ARTIST) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_ID, sArtistID) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_NAME, sArtistName) + NEWLINE);
+    List<Album> albums = AlbumManager.getInstance().getAssociatedAlbums(artist);
     for (Album album : albums) {
       tagAlbum(album, level + 1);
     }
-    writer.write(addTabs(level) + Tag.closeTag(Const.XML_AUTHOR) + NEWLINE);
+    writer.write(addTabs(level) + Tag.closeTag(Const.XML_ARTIST) + NEWLINE);
   }
 
+  /**
+   * Tag year.
+   * DOCUMENT_ME
+   * 
+   * @param year DOCUMENT_ME
+   * @param level DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
   private void tagYear(Year year, int level) throws Exception {
     String sYearID = year.getID();
     String sYearName = year.getName();
@@ -369,23 +468,40 @@ public class XMLExporter extends Exporter {
     writer.write(addTabs(level) + Tag.closeTag(Const.XML_YEAR) + NEWLINE);
   }
 
-  private void tagStyle(Style style, int level) throws Exception {
-    String sStyleID = style.getID();
-    String sStyleName = UtilString.formatXML(style.getName2());
-    writer.write(addTabs(level) + Tag.openTag(Const.XML_STYLE) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_ID, sStyleID) + NEWLINE);
-    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_NAME, sStyleName) + NEWLINE);
-    List<Album> albums = AlbumManager.getInstance().getAssociatedAlbums(style);
+  /**
+   * Tag genre.
+   * DOCUMENT_ME
+   * 
+   * @param genre DOCUMENT_ME
+   * @param level DOCUMENT_ME
+   * 
+   * @throws Exception the exception
+   */
+  private void tagGenre(Genre genre, int level) throws Exception {
+    String sGenreID = genre.getID();
+    String sGenreName = UtilString.formatXML(genre.getName2());
+    writer.write(addTabs(level) + Tag.openTag(Const.XML_GENRE) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_ID, sGenreID) + NEWLINE);
+    writer.write(addTabs(level + 1) + Tag.tagData(Const.XML_NAME, sGenreName) + NEWLINE);
+    List<Album> albums = AlbumManager.getInstance().getAssociatedAlbums(genre);
     for (Album album : albums) {
       tagAlbum(album, level + 1);
     }
-    List<Author> authors = AuthorManager.getInstance().getAssociatedAuthors(style);
-    for (Author author : authors) {
-      tagAuthor(author, level + 1);
+    List<Artist> artists = ArtistManager.getInstance().getAssociatedArtists(genre);
+    for (Artist artist : artists) {
+      tagArtist(artist, level + 1);
     }
-    writer.write(addTabs(level) + Tag.closeTag(Const.XML_STYLE) + NEWLINE);
+    writer.write(addTabs(level) + Tag.closeTag(Const.XML_GENRE) + NEWLINE);
   }
 
+  /**
+   * Adds the tabs.
+   * DOCUMENT_ME
+   * 
+   * @param num DOCUMENT_ME
+   * 
+   * @return the string
+   */
   private String addTabs(int num) {
     StringBuilder sb = new StringBuilder();
     int i = 0;
@@ -422,21 +538,30 @@ public class XMLExporter extends Exporter {
             .getString("ReportAction." + i)));
         i++;
       }
-      writer.write('\t' + Tag.tagData("ReportAction.name", Messages.getString("Property_name")));
-      writer
-          .write('\t' + Tag.tagData("ReportAction.author", Messages.getString("Property_author")));
-      writer.write('\t' + Tag.tagData("ReportAction.style", Messages.getString("Property_style")));
-      writer.write('\t' + Tag.tagData("ReportAction.order", Messages.getString("Property_track")));
-      writer.write('\t' + Tag.tagData("ReportAction.track", Messages.getString("Item_Track")));
-      writer.write('\t' + Tag.tagData("ReportAction.album", Messages.getString("Property_album")));
-      writer
-          .write('\t' + Tag.tagData("ReportAction.length", Messages.getString("Property_length")));
-      writer.write('\t' + Tag.tagData("ReportAction.year", Messages.getString("Property_year")));
-      writer.write('\t' + Tag.tagData("ReportAction.rate", Messages.getString("Property_rate")));
-      writer.write('\t' + Tag.tagData("ReportAction.url", Messages.getString("Property_url")));
-      writer.write('\t' + Tag.tagData("ReportAction.type", Messages.getString("Property_type")));
+      writer.write('\t' + Tag.tagData("ReportAction.name", Messages
+          .getHumanPropertyName(Const.XML_NAME)));
+      writer.write('\t' + Tag.tagData("ReportAction.artist", Messages
+          .getHumanPropertyName(Const.XML_ARTIST)));
+      writer.write('\t' + Tag.tagData("ReportAction.genre", Messages
+          .getHumanPropertyName(Const.XML_GENRE)));
+      writer.write('\t' + Tag.tagData("ReportAction.order", Messages
+          .getHumanPropertyName(Const.XML_TRACK_ORDER)));
+      writer.write('\t' + Tag.tagData("ReportAction.track", Messages
+          .getHumanPropertyName(Const.XML_TRACK)));
+      writer.write('\t' + Tag.tagData("ReportAction.album", Messages
+          .getHumanPropertyName(Const.XML_ALBUM)));
+      writer.write('\t' + Tag.tagData("ReportAction.length", Messages
+          .getHumanPropertyName(Const.XML_TRACK_LENGTH)));
+      writer.write('\t' + Tag.tagData("ReportAction.year", Messages
+          .getHumanPropertyName(Const.XML_YEAR)));
+      writer.write('\t' + Tag.tagData("ReportAction.rate", Messages
+          .getHumanPropertyName(Const.XML_TRACK_RATE)));
+      writer.write('\t' + Tag.tagData("ReportAction.url", Messages
+          .getHumanPropertyName(Const.XML_URL)));
+      writer.write('\t' + Tag.tagData("ReportAction.type", Messages
+          .getHumanPropertyName(Const.XML_TYPE)));
       writer.write('\t' + Tag.tagData("ReportAction.comment", Messages
-          .getString("Property_comment")));
+          .getHumanPropertyName(Const.XML_TRACK_COMMENT)));
       writer.write(Tag.closeTag("i18n"));
       writer.write(Tag.closeTag(Const.XML_COLLECTION));
     } finally {
@@ -446,6 +571,11 @@ public class XMLExporter extends Exporter {
 
   }
 
+  /**
+   * Sets the show tracks.
+   * 
+   * @param showTracks the new show tracks
+   */
   protected void setShowTracks(boolean showTracks) {
     this.showTracks = showTracks;
   }
