@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2005 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,16 +16,15 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $$Revision: 4463 $$
+ *  $Revision: 6186 $
  */
 
 package org.jajuk.ui.views;
 
 import ext.AutoCompleteDecorator;
-import ext.SwingWorker;
-import info.clearthought.layout.TableLayout;
 
 import java.awt.Color;
+import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.ItemEvent;
@@ -50,21 +50,25 @@ import javax.swing.JTable;
 import javax.swing.JTextField;
 import javax.swing.KeyStroke;
 import javax.swing.SwingUtilities;
+import javax.swing.SwingWorker;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
 import javax.swing.event.TableModelEvent;
 import javax.swing.event.TableModelListener;
 import javax.swing.table.TableColumn;
 
-import org.jajuk.base.AuthorManager;
+import net.miginfocom.swing.MigLayout;
+
+import org.jajuk.base.AlbumArtistManager;
+import org.jajuk.base.ArtistManager;
 import org.jajuk.base.File;
+import org.jajuk.base.GenreManager;
 import org.jajuk.base.Item;
 import org.jajuk.base.ItemManager;
-import org.jajuk.base.StyleManager;
 import org.jajuk.events.JajukEvent;
 import org.jajuk.events.JajukEvents;
 import org.jajuk.events.ObservationManager;
-import org.jajuk.services.players.FIFO;
+import org.jajuk.services.players.QueueModel;
 import org.jajuk.ui.actions.ActionManager;
 import org.jajuk.ui.actions.JajukActions;
 import org.jajuk.ui.helpers.FontManager;
@@ -72,6 +76,7 @@ import org.jajuk.ui.helpers.ILaunchCommand;
 import org.jajuk.ui.helpers.JajukTableModel;
 import org.jajuk.ui.helpers.PreferencesJMenu;
 import org.jajuk.ui.helpers.TableTransferHandler;
+import org.jajuk.ui.helpers.TwoStepsDisplayable;
 import org.jajuk.ui.helpers.FontManager.JajukFont;
 import org.jajuk.ui.widgets.InformationJPanel;
 import org.jajuk.ui.widgets.JajukTable;
@@ -88,76 +93,105 @@ import org.jajuk.util.error.JajukException;
 import org.jajuk.util.error.NoneAccessibleFileException;
 import org.jajuk.util.log.Log;
 import org.jdesktop.swingx.autocomplete.ComboBoxCellEditor;
-import org.jdesktop.swingx.decorator.SortOrder;
 import org.jdesktop.swingx.table.DefaultTableColumnModelExt;
 import org.jdesktop.swingx.table.TableColumnExt;
 
 /**
  * Abstract table view : common implementation for both files and tracks table
- * views
+ * views.
  */
 public abstract class AbstractTableView extends ViewAdapter implements ActionListener,
-    ItemListener, TableModelListener {
+    ItemListener, TableModelListener, TwoStepsDisplayable, ListSelectionListener {
 
+  /** Generated serialVersionUID. */
+  private static final long serialVersionUID = -4418626517605128694L;
+
+  /** DOCUMENT_ME. */
   JajukTable jtable;
 
+  /** DOCUMENT_ME. */
   private JPanel jpControl;
 
+  /** DOCUMENT_ME. */
   JajukToggleButton jtbEditable;
 
+  /** DOCUMENT_ME. */
   private JLabel jlFilter;
 
+  /** DOCUMENT_ME. */
   private JComboBox jcbProperty;
 
+  /** DOCUMENT_ME. */
   private JLabel jlEquals;
 
+  /** DOCUMENT_ME. */
   private JTextField jtfValue;
 
-  /** Table model */
+  /** Table model. */
   JajukTableModel model;
 
-  /** Currently applied filter */
+  /** Currently applied filter. */
   private String sAppliedFilter = "";
 
-  /** Currently applied criteria */
+  /** Currently applied criteria. */
   private String sAppliedCriteria;
 
-  /** Do search panel need a search */
+  /** Do search panel need a search. */
   private boolean bNeedSearch = false;
 
-  /** Default time in ms before launching a search automatically */
+  /** Default time in ms before launching a search automatically. */
   private static final int WAIT_TIME = 300;
 
-  /** Date last key pressed */
+  /** Date last key pressed. */
   private long lDateTyped;
 
-  /** Editable table configuration name, must be overwritten by child classes */
+  /** Editable table configuration name, must be overwritten by child classes. */
   String editableConf;
 
-  /**
-   * Columns to show table configuration name, must be overwritten by child
-   * classes
-   */
+  /** Columns to show table configuration name, must be overwritten by child classes. */
   String columnsConf;
 
-  JMenuItem jmiPlay;
-  JMenuItem jmiPush;
-  JMenuItem jmiDelete;
-  JMenuItem jmiPlayRepeat;
-  JMenuItem jmiPlayShuffle;
-  JMenuItem jmiBookmark;
-  JMenuItem jmiProperties;
-  JMenuItem jmiFileCopyURL;
+  /** DOCUMENT_ME. */
+  private JMenuItem jmiPlay;
 
+  /** DOCUMENT_ME. */
+  private JMenuItem jmiPush;
+
+  /** DOCUMENT_ME. */
+  private JMenuItem jmiFrontPush;
+
+  /** DOCUMENT_ME. */
+  JMenuItem jmiDelete;
+
+  /** DOCUMENT_ME. */
+  private JMenuItem jmiPlayRepeat;
+
+  /** DOCUMENT_ME. */
+  private JMenuItem jmiPlayShuffle;
+
+  /** DOCUMENT_ME. */
+  JMenuItem jmiBookmark;
+
+  /** DOCUMENT_ME. */
+  JMenuItem jmiProperties;
+
+  /** DOCUMENT_ME. */
+  private JMenuItem jmiFileCopyURL;
+
+  /** DOCUMENT_ME. */
   PreferencesJMenu pjmTracks;
 
-  /**
-   * Launches a thread used to perform dynamic filtering when user is typing
-   */
-  Thread filteringThread = new Thread("Dynamic user input filtering thread") {
+  /** The table/tree sync toggle button. */
+  JajukToggleButton jtbSync;
+
+  /** DOCUMENT_ME. */
+  private boolean bStopThread = false;
+
+  /** Launches a thread used to perform dynamic filtering when user is typing. */
+  private Thread filteringThread = new Thread("Dynamic user input filtering thread") {
     @Override
     public void run() {
-      while (true) {
+      while (!bStopThread) {
         try {
           Thread.sleep(200);
         } catch (InterruptedException ie) {
@@ -174,6 +208,7 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
   };
 
   /**
+   * Gets the apply criteria.
    * 
    * @return Applied criteria
    */
@@ -189,19 +224,31 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
 
   /**
    * Code used in child class SwingWorker for long delay computations (used in
-   * initUI())
+   * initUI()).
    * 
-   * @return
+   * @return the object
    */
-  public Object construct() {
+  public Object longCall() {
     model = populateTable();
-    jtable = new JajukTable(model, true, columnsConf);
+    return null;
+  }
 
+  /**
+   * Code used in child class SwingWorker for display computations (used in
+   * initUI()).
+   * 
+   * @param in DOCUMENT_ME
+   */
+  public void shortCall(Object in) {
     // Add generic menus
     jmiPlay = new JMenuItem(ActionManager.getAction(JajukActions.PLAY_SELECTION));
     jmiPlay.putClientProperty(Const.DETAIL_SELECTION, jtable.getSelection());
     jtable.getMenu().add(jmiPlay);
 
+    jmiFrontPush = new JMenuItem(ActionManager.getAction(JajukActions.PUSH_FRONT_SELECTION));
+    jmiFrontPush.putClientProperty(Const.DETAIL_SELECTION, jtable.getSelection());
+    jtable.getMenu().add(jmiFrontPush);
+
     jmiPush = new JMenuItem(ActionManager.getAction(JajukActions.PUSH_SELECTION));
     jmiPush.putClientProperty(Const.DETAIL_SELECTION, jtable.getSelection());
     jtable.getMenu().add(jmiPush);
@@ -254,8 +301,8 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
           List<File> files = UtilFeatures.getPlayableFiles(item);
           if (files.size() > 0) {
             // launch it
-            FIFO.push(UtilFeatures.createStackItems(UtilFeatures.applyPlayOption(files), Conf
-                .getBoolean(Const.CONF_STATE_REPEAT), true), Conf
+            QueueModel.push(UtilFeatures.createStackItems(UtilFeatures.applyPlayOption(files), Conf
+                .getBoolean(Const.CONF_STATE_REPEAT_ALL), true), Conf
                 .getBoolean(Const.CONF_OPTIONS_PUSH_ON_CLICK));
           } else {
             Messages.showErrorMessage(10);
@@ -263,22 +310,18 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
         }
       }
     });
-    return null;
-  }
-
-  /**
-   * Code used in child class SwingWorker for display computations (used in
-   * initUI())
-   * 
-   * @return
-   */
-  public void finished() {
     // Control panel
     jpControl = new JPanel();
     jpControl.setBorder(BorderFactory.createEtchedBorder());
     jtbEditable = new JajukToggleButton(IconLoader.getIcon(JajukIcons.EDIT));
     jtbEditable.setToolTipText(Messages.getString("AbstractTableView.11"));
     jtbEditable.addActionListener(this);
+
+    // Create the sync toggle button and restore its state
+    jtbSync = new JajukToggleButton(ActionManager.getAction(JajukActions.SYNC_TREE_TABLE));
+    jtbSync.putClientProperty(Const.DETAIL_VIEW, getID());
+    jtbSync.setSelected(Conf.getBoolean(Const.CONF_SYNC_TABLE_TREE + "." + getID()));
+
     jlFilter = new JLabel(Messages.getString("AbstractTableView.0"));
     // properties combo box, fill with columns names expect ID
     jcbProperty = new JComboBox();
@@ -292,8 +335,8 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
     jcbProperty.addItemListener(this);
     jlEquals = new JLabel(Messages.getString("AbstractTableView.7"));
     jtfValue = new JTextField();
-    jtfValue.setBorder(BorderFactory.createLineBorder(Color.BLUE));
     jtfValue.setFont(FontManager.getInstance().getFont(JajukFont.SEARCHBOX));
+    jtfValue.setMargin(new Insets(0, 3, 0, 0));
     jtfValue.setForeground(new Color(172, 172, 172));
     jtfValue.addKeyListener(new KeyAdapter() {
       @Override
@@ -303,38 +346,24 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
       }
     });
     jtfValue.setToolTipText(Messages.getString("AbstractTableView.3"));
-    int iXspace = 5;
-    double sizeControl[][] = {
-        { iXspace, 20, 3 * iXspace, TableLayout.FILL, iXspace, 0.3, TableLayout.FILL,
-            TableLayout.FILL, iXspace, 0.3, 2 }, { 5, 23, 5 } };
-    TableLayout layout = new TableLayout(sizeControl);
-    jpControl.setLayout(layout);
-    jpControl.add(jtbEditable, "1,1");
-    jpControl.add(jlFilter, "3,1");
-    jpControl.add(jcbProperty, "5,1");
-    jpControl.add(jlEquals, "7,1");
-    jpControl.add(jtfValue, "9,1");
-    double size[][] = { { 0.99 }, { TableLayout.PREFERRED, 0.99 } };
-    setLayout(new TableLayout(size));
-    add(jpControl, "0,0");
+    jpControl.setLayout(new MigLayout("insets 5", "[][20][grow,gp 70][grow]"));
+    jpControl.add(jtbSync, "gapleft 5");
+    jpControl.add(jtbEditable, "gapright 15");
+    jpControl.add(jlFilter, "split 2");
+    jpControl.add(jcbProperty, "grow,gapright 15");
+    jpControl.add(jlEquals, "split 2");
+    jpControl.add(jtfValue, "grow,gapright 2");
     setCellEditors();
     JScrollPane jsp = new JScrollPane(jtable);
-    jsp.setBorder(BorderFactory.createEmptyBorder(0, 1, 0, 0));
-    add(jsp, "0,1");
-    jtable.setDragEnabled(true);
+    setLayout(new MigLayout("ins 0", "[grow]", "[][grow]"));
+    add(jpControl, "wrap,grow");
+    add(jsp, "grow");
     jtable.setTransferHandler(new TableTransferHandler(jtable));
     jtable.showColumns(jtable.getColumnsConf());
     applyFilter(null, null);
-    jtable.setSortOrder(0, SortOrder.ASCENDING);
-    jtable.setHighlighters(UtilGUI.getAlternateHighlighter());
-    jtable.packTable(5);
-    // Hide the copy url if several items selection. Do not simply disable them
-    // as the getMenu() method enable all menu items
-    jtable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
-      public void valueChanged(ListSelectionEvent e) {
-        jmiFileCopyURL.setVisible(jtable.getSelectedRowCount() < 2);
-      }
-    });
+
+    jtable.getSelectionModel().addListSelectionListener(this);
+
     // Register on the list for subject we are interested in
     ObservationManager.register(this);
     // refresh columns conf in case of some attributes been removed
@@ -350,47 +379,52 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
     setKeystrokes();
   }
 
+  /*
+   * (non-Javadoc)
+   * 
+   * @see org.jajuk.events.Observer#getRegistrationKeys()
+   */
   public Set<JajukEvents> getRegistrationKeys() {
     Set<JajukEvents> eventSubjectSet = new HashSet<JajukEvents>();
     eventSubjectSet.add(JajukEvents.DEVICE_MOUNT);
     eventSubjectSet.add(JajukEvents.DEVICE_UNMOUNT);
     eventSubjectSet.add(JajukEvents.DEVICE_REFRESH);
-    eventSubjectSet.add(JajukEvents.SYNC_TREE_TABLE);
     eventSubjectSet.add(JajukEvents.CUSTOM_PROPERTIES_ADD);
     eventSubjectSet.add(JajukEvents.CUSTOM_PROPERTIES_REMOVE);
     eventSubjectSet.add(JajukEvents.RATE_CHANGED);
     eventSubjectSet.add(JajukEvents.TABLE_CLEAR_SELECTION);
     eventSubjectSet.add(JajukEvents.PARAMETERS_CHANGE);
     eventSubjectSet.add(JajukEvents.VIEW_REFRESH_REQUEST);
-    eventSubjectSet.add(JajukEvents.TABLE_SELECTION_CHANGED);
+    eventSubjectSet.add(JajukEvents.TREE_SELECTION_CHANGED);
     return eventSubjectSet;
   }
 
   /**
    * Apply a filter, to be implemented by files and tracks tables, alter the
-   * model
+   * model.
+   * 
+   * @param sPropertyName DOCUMENT_ME
+   * @param sPropertyValue DOCUMENT_ME
    */
   public void applyFilter(final String sPropertyName, final String sPropertyValue) {
-    SwingWorker sw = new SwingWorker() {
+    SwingWorker<Void, Void> sw = new SwingWorker<Void, Void>() {
       @Override
-      public Object construct() {
+      public Void doInBackground() {
         model.removeTableModelListener(AbstractTableView.this);
         model.populateModel(sPropertyName, sPropertyValue, jtable.getColumnsConf());
         model.addTableModelListener(AbstractTableView.this);
-        model.fireTableDataChanged();
         return null;
       }
 
       @Override
-      public void finished() {
+      public void done() {
         // Force table repaint (for instance for rating stars update)
-        jtable.revalidate();
-        jtable.repaint();
+        model.fireTableDataChanged();
         UtilGUI.stopWaiting();
       }
     };
     UtilGUI.waiting();
-    sw.start();
+    sw.execute();
   }
 
   /*
@@ -415,20 +449,27 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
             jtable.clearSelection();
             // force filter to refresh
             applyFilter(sAppliedCriteria, sAppliedFilter);
-          } else if (JajukEvents.SYNC_TREE_TABLE.equals(subject)) {
-            // Consume only events from the same perspective for
-            // table/tree synchronization
-            if (event.getDetails() != null
-                && !(event.getDetails().getProperty(Const.DETAIL_ORIGIN).equals(getPerspective()
-                    .getID()))) {
-              return;
+          } else if (JajukEvents.TREE_SELECTION_CHANGED.equals(subject)) {
+            // Check if the sync tree table option is set for this tree
+            if (jtbSync.isSelected()) {
+              // Consume only events from the same perspective and different view
+              // (for table/tree synchronization)
+              Properties details = event.getDetails();
+              if (details != null) {
+                String sourcePerspective = details.getProperty(Const.DETAIL_PERSPECTIVE);
+                String sourceView = details.getProperty(Const.DETAIL_VIEW);
+                if (!(sourcePerspective.equals(getPerspective().getID()))
+                    || sourceView.equals(getID())) {
+                  return;
+                }
+              }
+              // Update model tree selection
+              model.setTreeSelection((Set<Item>) details.get(Const.DETAIL_SELECTION));
+              // force redisplay to apply the filter
+              jtable.clearSelection();
+              // force filter to refresh
+              applyFilter(sAppliedCriteria, sAppliedFilter);
             }
-            // Update model tree selection
-            model.setTreeSelection((Set<Item>) event.getDetails().get(Const.DETAIL_SELECTION));
-            // force redisplay to apply the filter
-            jtable.clearSelection();
-            // force filter to refresh
-            applyFilter(sAppliedCriteria, sAppliedFilter);
           } else if (JajukEvents.PARAMETERS_CHANGE.equals(subject)) {
             // force redisplay to apply the filter
             jtable.clearSelection();
@@ -486,11 +527,7 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
             jtable.showColumns(jtable.getColumnsConf());
             applyFilter(sAppliedCriteria, sAppliedFilter);
             jcbProperty.removeItem(properties.get(Const.DETAIL_CONTENT));
-          } else if (JajukEvents.TABLE_SELECTION_CHANGED.equals(subject)) {
-            // Refresh the preference menu according to the selection
-            pjmTracks.resetUI(jtable.getSelection());
           }
-
         } catch (Exception e) {
           Log.error(e);
         } finally {
@@ -501,7 +538,7 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
   }
 
   /**
-   * Add keystroke support on the tree
+   * Add keystroke support.
    */
   private void setKeystrokes() {
     jtable.putClientProperty(Const.DETAIL_SELECTION, jtable.getSelection());
@@ -519,26 +556,40 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
     actionMap.put("properties", action);
   }
 
-  /** Fill the table */
+  /**
+   * Fill the table.
+   * 
+   * @return the jajuk table model
+   */
   abstract JajukTableModel populateTable();
 
+  /**
+   * Sets the cell editors. DOCUMENT_ME
+   */
   private void setCellEditors() {
     for (TableColumn tc : ((DefaultTableColumnModelExt) jtable.getColumnModel()).getColumns(true)) {
       TableColumnExt col = (TableColumnExt) tc;
       String sIdentifier = model.getIdentifier(col.getModelIndex());
-      // create a combo box for styles, note that we can't add new
-      // styles dynamically
-      if (Const.XML_STYLE.equals(sIdentifier)) {
-        JComboBox jcb = new JComboBox(StyleManager.getInstance().getStylesList());
+      // create a combo box for genres, note that we can't add new
+      // genres dynamically
+      if (Const.XML_GENRE.equals(sIdentifier)) {
+        JComboBox jcb = new JComboBox(GenreManager.getInstance().getGenresList());
         jcb.setEditable(true);
         AutoCompleteDecorator.decorate(jcb);
         col.setCellEditor(new ComboBoxCellEditor(jcb));
         col.setSortable(true);
       }
-      // create a combo box for authors, note that we can't add new
-      // authors dynamically
-      if (Const.XML_AUTHOR.equals(sIdentifier)) {
-        JComboBox jcb = new JComboBox(AuthorManager.getAuthorsList());
+      // create a combo box for artists, note that we can't add new
+      // artists dynamically
+      else if (Const.XML_ARTIST.equals(sIdentifier)) {
+        JComboBox jcb = new JComboBox(ArtistManager.getArtistsList());
+        jcb.setEditable(true);
+        AutoCompleteDecorator.decorate(jcb);
+        col.setCellEditor(new ComboBoxCellEditor(jcb));
+      }
+      // Same for for album-artists
+      else if (Const.XML_ALBUM_ARTIST.equals(sIdentifier)) {
+        JComboBox jcb = new JComboBox(AlbumArtistManager.getAlbumArtistsList());
         jcb.setEditable(true);
         AutoCompleteDecorator.decorate(jcb);
         col.setCellEditor(new ComboBoxCellEditor(jcb));
@@ -555,7 +606,9 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
   }
 
   /**
-   * Detect property change
+   * Detect property change.
+   * 
+   * @param ie DOCUMENT_ME
    */
   public void itemStateChanged(ItemEvent ie) {
     if (ie.getSource() == jcbProperty) {
@@ -568,7 +621,7 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
   /*
    * (non-Javadoc)
    * 
-   * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
+   * @seejavax.swing.event.TableModelListener#tableChanged(javax.swing.event. TableModelEvent)
    */
   public void tableChanged(TableModelEvent e) {
     // Check the table change event has not been generated by a
@@ -592,8 +645,8 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
       model.setItemAt(e.getFirstRow(), itemNew); // update model
       // user message
       InformationJPanel.getInstance().setMessage(
-          Messages.getString("PropertiesWizard.8") + ": " + ItemManager.getHumanType(sKey),
-          InformationJPanel.INFORMATIVE);
+          Messages.getString("PropertiesWizard.8") + ": " + Messages.getHumanPropertyName(sKey),
+          InformationJPanel.MessageType.INFORMATIVE);
       // Require refresh of all tables
       Properties properties = new Properties();
       properties.put(Const.DETAIL_ORIGIN, AbstractTableView.this);
@@ -613,8 +666,7 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
   }
 
   /**
-   * Table initialization after table display
-   * 
+   * Table initialization after table display.
    */
   abstract void initTable();
 
@@ -633,4 +685,86 @@ public abstract class AbstractTableView extends ViewAdapter implements ActionLis
 
   }
 
+  /* (non-Javadoc)
+   * @see org.jajuk.ui.views.ViewAdapter#cleanup()
+   */
+  @Override
+  public void cleanup() {
+    // stop the thread that is waiting for input
+    if (filteringThread != null) {
+      bStopThread = true;
+      try {
+        filteringThread.join();
+        filteringThread = null;
+      } catch (InterruptedException e) {
+        Log.error(e);
+      }
+    }
+
+    super.cleanup();
+  }
+
+  /**
+   * Called when table selection changed.
+   * 
+   * @param e the List selection event
+   */
+  public void valueChanged(ListSelectionEvent e) {
+    if (e.getValueIsAdjusting()) {
+      // leave during normal refresh
+      return;
+    }
+
+    // Ignore event if the model is refreshing
+    if (((JajukTableModel) jtable.getModel()).isRefreshing()) {
+      return;
+    }
+
+    // Call view specific behavior on selection change
+    onSelectionChange();
+
+    // Hide the copy url if several items selection. Do not simply disable them
+    // as the getMenu() method enable all menu items
+    jmiFileCopyURL.setVisible(jtable.getSelectedRowCount() < 2);
+
+    // Compute Information view message
+    if (AbstractTableView.this instanceof TracksTableView) {
+      int rows = jtable.getSelection().size();
+      StringBuilder sbOut = new StringBuilder().append(rows).append(
+          Messages.getString("TracksTreeView.31"));
+      InformationJPanel.getInstance().setSelection(sbOut.toString());
+    } else if (AbstractTableView.this instanceof FilesTableView) {
+      // Compute recursive selection size, nb of items...
+      long lSize = 0l;
+      int items = 0;
+      for (Item item : jtable.getSelection()) {
+        if (item instanceof File) {
+          lSize += ((File) item).getSize();
+        }
+      }
+      items = jtable.getSelection().size();
+      lSize /= 1048576; // set size in MB
+      StringBuilder sbOut = new StringBuilder().append(items).append(
+          Messages.getString("FilesTreeView.52"));
+      if (lSize > 1024) { // more than 1024 MB -> in GB
+        sbOut.append(lSize / 1024).append('.').append(lSize % 1024).append(
+            Messages.getString("FilesTreeView.53"));
+      } else {
+        sbOut.append(lSize).append(Messages.getString("FilesTreeView.54"));
+      }
+      InformationJPanel.getInstance().setSelection(sbOut.toString());
+    }
+
+    // Refresh the preference menu according to the selection
+    pjmTracks.resetUI(jtable.getSelection());
+  }
+
+  /**
+   * Callback method called on table selection change.
+   */
+  void onSelectionChange() {
+    // Do nothing by default
+    Log.debug("Table selection changed for : " + this);
+  }
+
 }
