@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2005 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,25 +16,25 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $$Revision: 3402 $$
+ *  $Revision$
  */
 
 package org.jajuk.ui.views;
 
-import info.clearthought.layout.TableLayout;
-
+import java.awt.BorderLayout;
 import java.awt.Color;
 import java.awt.Dimension;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.KeyAdapter;
 import java.awt.event.KeyEvent;
+import java.util.ArrayList;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Properties;
 import java.util.Set;
 
 import javax.swing.BorderFactory;
-import javax.swing.DefaultListSelectionModel;
 import javax.swing.JLabel;
 import javax.swing.JMenuItem;
 import javax.swing.JPanel;
@@ -44,42 +45,57 @@ import javax.swing.ListSelectionModel;
 import javax.swing.SwingUtilities;
 import javax.swing.event.ListSelectionEvent;
 
+import net.miginfocom.swing.MigLayout;
+
 import org.jajuk.base.File;
 import org.jajuk.base.FileManager;
 import org.jajuk.base.Playlist;
+import org.jajuk.base.SmartPlaylist;
 import org.jajuk.events.JajukEvent;
 import org.jajuk.events.JajukEvents;
 import org.jajuk.events.ObservationManager;
-import org.jajuk.services.players.FIFO;
+import org.jajuk.services.players.QueueModel;
 import org.jajuk.services.players.StackItem;
 import org.jajuk.ui.actions.ActionManager;
 import org.jajuk.ui.actions.JajukActions;
 import org.jajuk.ui.helpers.ILaunchCommand;
+import org.jajuk.ui.helpers.IndexHighlighterPredicate;
 import org.jajuk.ui.helpers.JajukTableModel;
-import org.jajuk.ui.helpers.PlayHighlighterPredicate;
 import org.jajuk.ui.helpers.PlaylistEditorTransferHandler;
 import org.jajuk.ui.helpers.PlaylistTableModel;
 import org.jajuk.ui.widgets.JajukButton;
 import org.jajuk.ui.widgets.JajukJToolbar;
 import org.jajuk.ui.widgets.JajukTable;
+import org.jajuk.ui.widgets.JajukToggleButton;
 import org.jajuk.util.Conf;
 import org.jajuk.util.Const;
 import org.jajuk.util.IconLoader;
 import org.jajuk.util.JajukIcons;
 import org.jajuk.util.Messages;
-import org.jajuk.util.UtilGUI;
-import org.jajuk.util.error.JajukException;
+import org.jajuk.util.UtilFeatures;
 import org.jajuk.util.log.Log;
 import org.jdesktop.swingx.decorator.ColorHighlighter;
-import org.jdesktop.swingx.decorator.Highlighter;
+import org.jvnet.substance.SubstanceLookAndFeel;
+import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.SubstanceSkin;
 
 /**
- * Adapter for playlists editors *
+ * Adapter for playlists editors *.
  */
 public class QueueView extends PlaylistView {
 
+  /** Generated serialVersionUID. */
   private static final long serialVersionUID = -2851288035506442507L;
 
+  /** DOCUMENT_ME. */
+  private JScrollPane jsp;
+
+  /** DOCUMENT_ME. */
+  private JajukToggleButton jtbAutoScroll;
+
+  /** Last scrolled-item *. */
+  private StackItem lastScrolledItem;
+
   /*
    * (non-Javadoc)
    * 
@@ -87,17 +103,11 @@ public class QueueView extends PlaylistView {
    */
   @Override
   public void initUI() {
-    plf = new Playlist(Playlist.Type.QUEUE, Integer.toString(Playlist.Type.QUEUE.ordinal()), null,
-        null);
+    plf = new SmartPlaylist(Playlist.Type.QUEUE, Integer.toString(Playlist.Type.QUEUE.ordinal()),
+        null, null);
     // Control panel
     jpEditorControl = new JPanel();
     jpEditorControl.setBorder(BorderFactory.createEtchedBorder());
-    // Note : we don't use toolbar because it's buggy in Metal look and feel
-    // : icon get bigger
-    double sizeControl[][] = { { 5, TableLayout.PREFERRED, 15, TableLayout.FILL, 5 }, { 5, 25, 5 } };
-    TableLayout layout = new TableLayout(sizeControl);
-    layout.setHGap(2);
-    jpEditorControl.setLayout(layout);
     jbSave = new JajukButton(IconLoader.getIcon(JajukIcons.SAVE));
     jbSave.setToolTipText(Messages.getString("AbstractPlaylistEditorView.3"));
     jbSave.addActionListener(this);
@@ -113,14 +123,25 @@ public class QueueView extends PlaylistView {
     jbAddShuffle = new JajukButton(IconLoader.getIcon(JajukIcons.ADD_SHUFFLE));
     jbAddShuffle.setToolTipText(Messages.getString("AbstractPlaylistEditorView.10"));
     jbAddShuffle.addActionListener(this);
-    jlTitle = new JLabel(" [" + FIFO.getFIFO().size() + "]");
+    jlTitle = new JLabel(" [" + QueueModel.getQueue().size() + "]");
 
     jbClear = new JajukButton(IconLoader.getIcon(JajukIcons.CLEAR));
     jbClear.setToolTipText(Messages.getString("QueueView.1"));
     jbClear.addActionListener(this);
 
-    JToolBar jtb = new JajukJToolbar();
+    jtbAutoScroll = new JajukToggleButton(IconLoader.getIcon(JajukIcons.AUTOSCROLL));
+    jtbAutoScroll.setToolTipText(Messages.getString("QueueView.2"));
+    jtbAutoScroll.setSelected(Conf.getBoolean(Const.CONF_AUTO_SCROLL));
+    jtbAutoScroll.addActionListener(new ActionListener() {
+      public void actionPerformed(ActionEvent e) {
+        Conf.setProperty(Const.CONF_AUTO_SCROLL, Boolean.toString(jtbAutoScroll.isSelected()));
+        if (jtbAutoScroll.isSelected()) {
+          autoScroll();
+        }
+      }
+    });
 
+    JToolBar jtb = new JajukJToolbar();
     jtb.add(jbSave);
     jtb.add(jbRemove);
     jtb.add(jbAddShuffle);
@@ -129,15 +150,17 @@ public class QueueView extends PlaylistView {
     jtb.addSeparator();
     jtb.add(jbClear);
 
-    jpEditorControl.add(jtb, "1,1");
-    jpEditorControl.add(jlTitle, "3,1,r,c");
+    // Add items
+    jpEditorControl.setLayout(new MigLayout("insets 5", "[][grow][]"));
+    jpEditorControl.add(jtb, "left,gapright 15::");
+    jpEditorControl.add(jlTitle, "right,gapright 5");
+    jpEditorControl.add(jtbAutoScroll, "right");
     editorModel = new PlaylistTableModel(true);
     editorTable = new JajukTable(editorModel, CONF_QUEUE_COLUMNS);
     editorModel.populateModel(editorTable.getColumnsConf());
-    editorTable.setSelectionMode(DefaultListSelectionModel.MULTIPLE_INTERVAL_SELECTION); // multi-row
+    editorTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION); // multi-row
     // selection
     editorTable.setSortable(false);
-    editorTable.setDragEnabled(true);
     editorTable.setTransferHandler(new PlaylistEditorTransferHandler(editorTable));
     setRenderers();
     // just an icon
@@ -145,14 +168,11 @@ public class QueueView extends PlaylistView {
     editorTable.getColumnModel().getColumn(0).setMaxWidth(20);
     editorTable.getTableHeader().setPreferredSize(new Dimension(0, 20));
     editorTable.showColumns(editorTable.getColumnsConf());
-    ListSelectionModel lsm = editorTable.getSelectionModel();
-    lsm.addListSelectionListener(this);
-    double size[][] = { { 0.99 }, { TableLayout.PREFERRED, 0.99 } };
-    setLayout(new TableLayout(size));
-    add(jpEditorControl, "0,0");
-    JScrollPane jsp = new JScrollPane(editorTable);
+    setLayout(new BorderLayout());
+    add(jpEditorControl, BorderLayout.NORTH);
+    jsp = new JScrollPane(editorTable);
     jsp.setBorder(BorderFactory.createEmptyBorder(0, 1, 0, 0));
-    add(jsp, "0,1");
+    add(jsp, BorderLayout.CENTER);
     // menu items
     jmiFilePlay = new JMenuItem(Messages.getString("TracksTableView.7"), IconLoader
         .getIcon(JajukIcons.PLAY_16X16));
@@ -166,14 +186,19 @@ public class QueueView extends PlaylistView {
     });
     initMenuItems();
 
-    ColorHighlighter colorHighlighter = new ColorHighlighter(Color.ORANGE, null,
-        new PlayHighlighterPredicate(editorModel));
-    Highlighter alternate = UtilGUI.getAlternateHighlighter();
-    editorTable.setHighlighters(alternate, colorHighlighter);
+    SubstanceSkin theme = SubstanceLookAndFeel.getCurrentSkin();
+    SubstanceColorScheme scheme = theme.getMainActiveColorScheme();
+    Color queueHighlighterColor = null;
+    if (scheme.isDark()) {
+      queueHighlighterColor = scheme.getUltraLightColor();
+    } else {
+      queueHighlighterColor = scheme.getLineColor();
+    }
+    ColorHighlighter colorHighlighter = new ColorHighlighter(new IndexHighlighterPredicate(),
+        queueHighlighterColor, null);
+    editorTable.addHighlighter(colorHighlighter);
     // register events
     ObservationManager.register(this);
-    // -- force a refresh --
-    refreshQueue();
     // Add key listener to enable row suppression using SUPR key
     editorTable.addKeyListener(new KeyAdapter() {
       @Override
@@ -190,52 +215,67 @@ public class QueueView extends PlaylistView {
     // Add specific behavior on left click
     editorTable.setCommand(new ILaunchCommand() {
       public void launch(int nbClicks) {
-        if (nbClicks == 2) {
-          // double click, launches selected track and all after
+        int iSelectedCol = editorTable.getSelectedColumn();
+        // Convert column selection as columns may have been moved
+        iSelectedCol = editorTable.convertColumnIndexToModel(iSelectedCol);
+        // double click, launches selected track and all after
+        if (nbClicks == 2
+        // click on play icon
+            || (nbClicks == 1 && iSelectedCol == 0)) {
           StackItem item = editorModel.getStackItem(editorTable.getSelectedRow());
           if (item.isPlanned()) {
             item.setPlanned(false);
-            item.setRepeat(Conf.getBoolean(Const.CONF_STATE_REPEAT));
+            item.setRepeat(Conf.getBoolean(Const.CONF_STATE_REPEAT_ALL));
             item.setUserLaunch(true);
-            FIFO.push(item, Conf.getBoolean(Const.CONF_OPTIONS_PUSH_ON_CLICK));
+            QueueModel.push(item, Conf.getBoolean(Const.CONF_OPTIONS_PUSH_ON_CLICK));
           } else { // non planned items
             goToSelection();
           }
         }
       }
     });
+    //  Note : don't add a ListSelectionListener here, see JajukTable code, 
+    //  all the event code is centralized over there 
+    editorTable.addListSelectionListener(this);
     // Register keystrokes over table
-    editorTable.putClientProperty(Const.DETAIL_SELECTION, editorTable.getSelection());
     super.setKeystrokes();
+    // Force a first need refresh event
+    update(new JajukEvent(JajukEvents.QUEUE_NEED_REFRESH));
   }
 
   /**
-   * Go to selected row, do it asynchronously because FIFO.goTO() can frezze the
+   * Go to selected row, do it asynchronously because FIFO.goTO() can freeze the
    * GUI
    */
   private void goToSelection() {
-    new Thread() {
+    new Thread("Queue Selection Thread") {
+      @Override
       public void run() {
         try {
-          FIFO.goTo(editorTable.getSelectedRow());
+          QueueModel.goTo(editorTable.getSelectedRow());
           // remove selection for planned tracks
           ListSelectionModel lsm = editorTable.getSelectionModel();
-          bSettingSelection = true;
+          editorModel.setRefreshing(true);
           editorTable.getSelectionModel().removeSelectionInterval(lsm.getMinSelectionIndex(),
               lsm.getMaxSelectionIndex());
-          bSettingSelection = false;
         } catch (Exception e) {
           Log.error(e);
+        } finally {
+          editorModel.setRefreshing(false);
         }
       }
     }.start();
   }
 
+  /*
+   * (non-Javadoc)
+   * 
+   * @see org.jajuk.ui.views.PlaylistView#getRegistrationKeys()
+   */
   @Override
   public Set<JajukEvents> getRegistrationKeys() {
     Set<JajukEvents> eventSubjectSet = new HashSet<JajukEvents>();
     eventSubjectSet.add(JajukEvents.QUEUE_NEED_REFRESH);
-    eventSubjectSet.add(JajukEvents.FILE_LAUNCHED);
     eventSubjectSet.add(JajukEvents.DEVICE_REFRESH);
     eventSubjectSet.add(JajukEvents.DEVICE_MOUNT);
     eventSubjectSet.add(JajukEvents.DEVICE_UNMOUNT);
@@ -243,7 +283,6 @@ public class QueueView extends PlaylistView {
     eventSubjectSet.add(JajukEvents.CUSTOM_PROPERTIES_REMOVE);
     eventSubjectSet.add(JajukEvents.VIEW_REFRESH_REQUEST);
     eventSubjectSet.add(JajukEvents.RATE_CHANGED);
-    eventSubjectSet.add(JajukEvents.TABLE_SELECTION_CHANGED);
     eventSubjectSet.add(JajukEvents.PARAMETERS_CHANGE);
     return eventSubjectSet;
   }
@@ -280,6 +319,13 @@ public class QueueView extends PlaylistView {
             editorModel.getItems().clear();
             editorModel.getPlanned().clear();
             refreshQueue();
+            // Only scroll if song actually changed, otherwise, any queue refresh
+            // would scroll and annoy users
+            if (Conf.getBoolean(CONF_AUTO_SCROLL) && QueueModel.getCurrentItem() != null
+                && !QueueModel.getCurrentItem().equals(lastScrolledItem)) {
+              autoScroll();
+              lastScrolledItem = QueueModel.getCurrentItem();
+            }
           } else if (JajukEvents.CUSTOM_PROPERTIES_ADD.equals(subject)) {
             Properties properties = event.getDetails();
             if (properties == null) {
@@ -323,44 +369,83 @@ public class QueueView extends PlaylistView {
               editorModel.getPlanned().clear();
               refreshQueue();
             }
-          } else if (JajukEvents.TABLE_SELECTION_CHANGED.equals(subject)) {
-            // Refresh the preference menu according to the selection
-            pjmFilesEditor.resetUI(editorTable.getSelection());
           }
         } catch (Exception e) {
           Log.error(e);
         } finally {
           editorTable.setAcceptColumnsEvents(true);
           // Update number of tracks remaining
-          jlTitle.setText(" [" + FIFO.getFIFO().size() + "]");
+          jlTitle.setText(" [" + QueueModel.getQueue().size() + "]");
         }
       }
     });
 
   }
 
+  /**
+   * Auto scroll to played track if option is enabled.
+   */
+  private void autoScroll() {
+
+    SwingUtilities.invokeLater(new Runnable() {
+      public void run() {
+
+        if (QueueModel.getQueueSize() > 0) {
+          double index = QueueModel.getIndex();
+          double size = QueueModel.getQueueSize() + QueueModel.getPlanned().size();
+          double factor = (index / size);
+          int value = (int) (factor * jsp.getVerticalScrollBar().getMaximum());
+
+          // 'center' played track
+          value -= (jsp.getVerticalScrollBar().getHeight() / 2) - (editorTable.getRowHeight() / 2);
+
+          if (value < 0) {
+            value = 0;
+          }
+          if (value >= jsp.getVerticalScrollBar().getMinimum()
+              && value <= jsp.getVerticalScrollBar().getMaximum()) {
+            jsp.getVerticalScrollBar().setValue(value);
+
+          }
+        }
+      }
+    });
+
+  }
+
+  /**
+   * Refresh queue. DOCUMENT_ME
+   */
   private void refreshQueue() {
     // when nothing is selected, set default button state
     if (editorTable.getSelectionModel().getMinSelectionIndex() == -1) {
       setDefaultButtonState();
     }
-    editorModel.setItems(FIFO.getFIFO());
-    editorModel.setPlanned(FIFO.getPlanned());
+    editorModel.setItems(QueueModel.getQueue());
+    editorModel.setPlanned(QueueModel.getPlanned());
     ((JajukTableModel) editorTable.getModel()).populateModel(editorTable.getColumnsConf());
+    // save selection to avoid reseting selection the user is doing
     int[] rows = editorTable.getSelectedRows();
-    // save selection
-    editorModel.fireTableDataChanged();// refresh
-    bSettingSelection = true;
-    for (int element : rows) {
-      // set saved selection after a refresh
-      editorTable.getSelectionModel().addSelectionInterval(element, element);
+
+    try {
+      editorModel.setRefreshing(true);
+      // force table refresh
+      editorModel.fireTableDataChanged();
+
+      for (int element : rows) {
+        // set saved selection after a refresh
+        editorTable.getSelectionModel().addSelectionInterval(element, element);
+      }
+    } finally {
+      editorModel.setRefreshing(false);
     }
-    bSettingSelection = false;
+    // Refresh the preference menu according to the selection 
+    // (useful on rating change for a single-row model for ie)
+    pjmFilesEditor.resetUI(editorTable.getSelection());
   }
 
   /**
-   * Set default button state
-   * 
+   * Set default button state.
    */
   private void setDefaultButtonState() {
     // set buttons
@@ -377,6 +462,11 @@ public class QueueView extends PlaylistView {
     // disable prepare party for queue playlist
   }
 
+  /*
+   * (non-Javadoc)
+   * 
+   * @see org.jajuk.ui.views.PlaylistView#actionPerformed(java.awt.event.ActionEvent)
+   */
   @Override
   public void actionPerformed(ActionEvent ae) {
     try {
@@ -405,42 +495,42 @@ public class QueueView extends PlaylistView {
             }
           }
         }
-      } else if (ae.getSource() == jbRemove) {
+      } else if (ae.getSource() == jbRemove || ae.getSource() == jmiFileRemove) {
         removeSelection();
         refreshQueue();
       } else if (ae.getSource() == jbAddShuffle) {
         int iRow = editorTable.getSelectedRow();
         if (iRow < 0
         // no row is selected, add to the end
-            || iRow > FIFO.getFIFO().size()) {
+            || iRow > QueueModel.getQueue().size()) {
           // row can be on planned track if user select a planned track and if
           // fifo is reduced after tracks have been played
-          iRow = FIFO.getFIFO().size();
+          iRow = QueueModel.getQueue().size();
         }
         File file = FileManager.getInstance().getShuffleFile();
-        try {
-          plf.addFile(iRow, file);
-          jbRemove.setEnabled(true);
-        } catch (JajukException je) {
-          Messages.showErrorMessage(je.getCode());
-          Log.error(je);
-        }
+        List<File> files = new ArrayList<File>();
+        files.add(file);
+        QueueModel.insert(UtilFeatures.createStackItems(files, Conf
+            .getBoolean(Const.CONF_STATE_REPEAT_ALL), true), iRow);
         refreshQueue();
       } else if (ae.getSource() == jbClear) {
         // Reset the FIFO
-        FIFO.reset(); // reinit all variables
+        QueueModel.reset(); // reinit all variables
         try {
           ActionManager.getAction(JajukActions.STOP_TRACK).perform(null);
+          ObservationManager.notify(new JajukEvent(JajukEvents.ZERO));
         } catch (Exception e) {
           Log.error(e);
         }
       }
-
     } catch (Exception e2) {
       Log.error(e2);
     }
   }
 
+  /**
+   * Removes the selection. DOCUMENT_ME
+   */
   private void removeSelection() {
     int[] iRows = editorTable.getSelectedRows();
     if (iRows.length > 1) {// if multiple selection, remove
@@ -459,16 +549,18 @@ public class QueueView extends PlaylistView {
   }
 
   /**
-   * Called when table selection changed
+   * Called when table selection changed.
+   * 
+   * @param e DOCUMENT_ME
    */
   @Override
   public void valueChanged(ListSelectionEvent e) {
-    if (e.getValueIsAdjusting() || bSettingSelection) {
-      // leave during normal refresh
-      return;
-    }
     ListSelectionModel selection = (ListSelectionModel) e.getSource();
     if (!selection.isSelectionEmpty()) {
+      updateSelection();
+      updateInformationView(selectedFiles);
+      // Refresh the preference menu according to the selection
+      pjmFilesEditor.resetUI(editorTable.getSelection());
       int selectedRow = selection.getMaxSelectionIndex();
       // true if selected line is a planned track
       boolean bPlanned = false;
@@ -478,47 +570,40 @@ public class QueueView extends PlaylistView {
       }
       // -- now analyze each button --
       // Remove button
-      if (bPlanned) {// not for planned track
-        jbRemove.setEnabled(true);
+      if (bPlanned) {
+        jbRemove.setEnabled(false);
+        jmiFileRemove.setEnabled(false);
       } else {
-        // check for first row remove case : we can't remove currently
+        // check for current track case : we can't remove currently
         // played track
-        if (selection.getMinSelectionIndex() == 0) {
-          // neither for bestof nor novelties playlist
-          jbRemove.setEnabled(false);
-        } else {
-          jbRemove.setEnabled(true);
-        }
+        jbRemove.setEnabled(!selectionContainsCurrentTrack(selection));
+        jmiFileRemove.setEnabled(!selectionContainsCurrentTrack(selection));
       }
+
       // Add shuffle button
-      if (// multiple selection not supported
-      selection.getMinSelectionIndex() == 0
-      // can't add track at current track position
-          || selectedRow > FIFO.getFIFO().size()
-      // no add for planned track but user can add over first planned
-      // track to expand FIFO
-      ) {
-        jbAddShuffle.setEnabled(false);
-      } else {
-        jbAddShuffle.setEnabled(true);
-      }
+      // No adding for planned track
+      jbAddShuffle.setEnabled(!bPlanned);
+
       // Up button
       if (selection.getMinSelectionIndex() != selection.getMaxSelectionIndex()) {
         // check if several rows have been selected :
         // doesn't supported yet
         jbUp.setEnabled(false);
+        jmiFileUp.setEnabled(false);
       } else {
         // still here ?
         if (bPlanned) {
           // No up/down buttons for planned tracks
           jbUp.setEnabled(false);
+          jmiFileUp.setEnabled(false);
         } else { // normal item
-          if (selection.getMinSelectionIndex() == 0 || (selection.getMinSelectionIndex() == 1)) {
-            // check if we selected second track just after current
-            // tracks
-            jbUp.setEnabled(false); // already at the top
+          if (selection.getMinSelectionIndex() == 0) {
+            // already at the top
+            jbUp.setEnabled(false);
+            jmiFileUp.setEnabled(false);
           } else {
             jbUp.setEnabled(true);
+            jmiFileUp.setEnabled(true);
           }
         }
       }
@@ -527,23 +612,40 @@ public class QueueView extends PlaylistView {
         // check if several rows have been selected :
         // doesn't supported yet
         jbDown.setEnabled(false);
+        jmiFileDown.setEnabled(false);
       } else { // yet here ?
         if (bPlanned) {
           // No up/down buttons for planned tracks
           jbDown.setEnabled(false);
+          jmiFileDown.setEnabled(false);
         } else { // normal item
-          if (selection.getMaxSelectionIndex() == 0) {
-            // current track can't go down
-            jbDown.setEnabled(false);
-          } else if (selection.getMaxSelectionIndex() < editorModel.getItems().size() - 1) {
+          if (selection.getMaxSelectionIndex() < editorModel.getItems().size() - 1) {
             // a normal item can't go in the planned items
             jbDown.setEnabled(true);
+            jmiFileDown.setEnabled(true);
           } else {
             jbDown.setEnabled(false);
+            jmiFileDown.setEnabled(false);
           }
         }
       }
     }
   }
 
+  /**
+   * Return whether a given row selection contains the current played track.
+   * 
+   * @param selection the selection
+   * 
+   * @return whether a given row selection contains the current played track
+   */
+  private boolean selectionContainsCurrentTrack(ListSelectionModel selection) {
+    for (int i = selection.getMinSelectionIndex(); i <= selection.getMaxSelectionIndex(); i++) {
+      if (QueueModel.getItem(i).equals(QueueModel.getCurrentItem())) {
+        return true;
+      }
+    }
+    return false;
+  }
+
 }
\ No newline at end of file
