@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2003 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,30 +16,30 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * $Revision: 4462 $
+ *  $Revision: 6186 $
  */
 
 package org.jajuk.ui.widgets;
 
-import java.awt.Component;
 import java.awt.event.MouseEvent;
 import java.text.DateFormat;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
+import java.util.Set;
 import java.util.StringTokenizer;
 
-import javax.swing.JMenu;
-import javax.swing.JMenuItem;
 import javax.swing.JPopupMenu;
 import javax.swing.JTable;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
 import javax.swing.event.TableColumnModelEvent;
+import javax.swing.event.TableColumnModelListener;
 import javax.swing.table.TableColumn;
 import javax.swing.table.TableModel;
 
@@ -46,14 +47,20 @@ import org.jajuk.base.Item;
 import org.jajuk.events.JajukEvent;
 import org.jajuk.events.JajukEvents;
 import org.jajuk.events.ObservationManager;
+import org.jajuk.events.Observer;
 import org.jajuk.ui.helpers.ILaunchCommand;
 import org.jajuk.ui.helpers.JajukCellRenderer;
+import org.jajuk.ui.helpers.JajukMouseAdapter;
 import org.jajuk.ui.helpers.JajukTableModel;
-import org.jajuk.ui.helpers.TableTransferHandler;
+import org.jajuk.ui.perspectives.PerspectiveManager;
+import org.jajuk.ui.views.IView;
 import org.jajuk.util.Conf;
 import org.jajuk.util.Const;
+import org.jajuk.util.UtilGUI;
 import org.jajuk.util.UtilString;
 import org.jdesktop.swingx.JXTable;
+import org.jdesktop.swingx.decorator.CompoundHighlighter;
+import org.jdesktop.swingx.decorator.Highlighter;
 import org.jdesktop.swingx.table.DefaultTableColumnModelExt;
 import org.jdesktop.swingx.table.TableColumnExt;
 
@@ -66,78 +73,144 @@ import org.jdesktop.swingx.table.TableColumnExt;
  * <p>
  * Maintain a table of selected rows
  * <p>
- * Bring a menu displayed on right click
+ * Bring a menu displayed on right click.
  */
-public class JajukTable extends JXTable implements ListSelectionListener,
-    java.awt.event.MouseListener {
+public class JajukTable extends JXTable implements Observer, TableColumnModelListener,
+    ListSelectionListener {
 
+  /** Generated serialVersionUID. */
   private static final long serialVersionUID = 1L;
 
-  private String sConf;
+  /** DOCUMENT_ME. */
+  private final String sConf;
 
-  /** User Selection* */
-  private List<Item> selection;
+  /** User Selection*. */
+  private final List<Item> selection = new ArrayList<Item>();
 
-  private JPopupMenu jmenu;
+  /** DOCUMENT_ME. */
+  private final JPopupMenu jmenu;
 
-  /** Specific action on double click */
+  /** Specific action on double click. */
   private ILaunchCommand command;
 
-  /** Model refreshing flag */
+  /** Model refreshing flag. */
   private volatile boolean acceptColumnsEvents = false;
 
+  /** The Constant FORMATTER. DOCUMENT_ME */
   private static final DateFormat FORMATTER = UtilString.getLocaleDateFormatter();
 
-  /** Stores the last index of column move to* */
+  /** Stores the last index of column move to*. */
   private int lastToIndex = 0;
 
-  /**
-   * Constructor
-   * 
-   * @param model :
-   *          model to use
-   * @param bSortable :
-   *          is this table sortable
-   * @sConf: configuration variable used to store columns conf
-   */
-  public JajukTable(TableModel model, boolean bSortable, String sConf) {
-    super(model);
-    acceptColumnsEvents = true;
-    this.sConf = sConf;
-    selection = new ArrayList<Item>();
-    jmenu = new JPopupMenu();
-    setShowGrid(false);
-    init(bSortable);
-    // Force to use Jajuk cell render for all columns, except for boolean
-    // that should use default renderer (checkbox)
-    for (TableColumn col : getColumns()) {
-      col.setCellRenderer(new JajukCellRenderer());
+  /** Mouse draging flag. */
+  private boolean isMouseDragging;
+
+  /** List of list selection listeners whose valueChanged() method is called by this class valueChanged() method to avoid concurrency between them. Otherwise, the preference menu item could be set with the previous selection value. */
+  List<ListSelectionListener> listeners = new ArrayList<ListSelectionListener>(1);
+
+  /** The Jajuk table mouse adapter used to handle click events. */
+  JajukMouseAdapter ma = new JajukMouseAdapter() {
+
+    @Override
+    public void handlePopup(MouseEvent e) {
+      int iSelectedRow = rowAtPoint(e.getPoint());
+      // right click on a selected node set if none or 1 node is
+      // selected, a right click on another node
+      // select it if more than 1, we keep selection and display a
+      // popup for them
+      if (getSelectedRowCount() < 2) {
+        getSelectionModel().setSelectionInterval(iSelectedRow, iSelectedRow);
+      }
+      // Use getMenu() here, do not use jmenu directly as we want to enable all
+      // items before though getMenu() method
+      getMenu().show(JajukTable.this, e.getX(), e.getY());
     }
-    // Listen for row selection
-    getSelectionModel().addListSelectionListener(this);
-    // Listen for clicks
-    addMouseListener(this);
+
+    @Override
+    public void handleAction(final MouseEvent e) {
+      command.launch(e.getClickCount());
+    }
+  };
+
+  /**
+   * Return drop row.
+   * 
+   * @return drop row
+   */
+  @SuppressWarnings("cast")
+  public int getDropRow() {
+    JTable.DropLocation dl = (JTable.DropLocation) getDropLocation();
+    return dl.getRow();
   }
 
   /**
-   * Constructor
+   * Constructor.
+   * 
+   * @param model : model to use
+   * @param bSortable : is this table sortable
+   * @param sConf DOCUMENT_ME
+   * 
+   * @sConf: configuration variable used to store columns conf
+   */
+  public JajukTable(TableModel model, boolean bSortable, String sConf) {
+    // Note that JTable automatically create a default ListSelectionModel
+    // executing this.valueChanged() at selection changes
+    // so don't add a new listener to avoid double events consumption
+    super(model);
+    acceptColumnsEvents = true;
+    this.sConf = sConf;
+    jmenu = new JPopupMenu();
+    setShowGrid(false);
+    init(bSortable);
+
+    // Listen for clicks
+    addMouseListener(ma);
+
+    //Let Laf handle drag gesture recognition (don't remove it or
+    // a mouse clik disable multiple selection)
+    setDragEnabled(true);
+
+    // Add the Alternate Highlighter
+    addHighlighter(UtilGUI.getAlternateHighlighter());
+    // Register itself to incoming events
+    ObservationManager.register(this);
+  }
+
+  /**
+   * Register a new list selection listener.
+   * 
+   * @param listener the listener to register
+   */
+  public void addListSelectionListener(ListSelectionListener listener) {
+    listeners.add(listener);
+  }
+
+  /**
+   * Constructor.
+   * 
+   * @param model : model to use
+   * @param sConf DOCUMENT_ME
    * 
-   * @param model :
-   *          model to use
    * @sConf: configuration variable used to store columns conf
    */
   public JajukTable(TableModel model, String sConf) {
     this(model, true, sConf);
   }
 
+  /**
+   * Inits the. DOCUMENT_ME
+   * 
+   * @param bSortable DOCUMENT_ME
+   */
   private void init(boolean bSortable) {
     super.setSortable(bSortable);
     super.setColumnControlVisible(true);
-    setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
   }
 
   /**
-   * Select columns to show colsToShow list of columns id to keep
+   * Select columns to show colsToShow list of columns id to keep.
+   * 
+   * @param colsToShow DOCUMENT_ME
    */
   @SuppressWarnings("unchecked")
   public void showColumns(List<String> colsToShow) {
@@ -152,12 +225,20 @@ public class JajukTable extends JXTable implements ListSelectionListener,
       }
     }
     reorderColumns();
+    // Force to use Jajuk cell render for all columns
+    for (TableColumn col : getColumns()) {
+      col.setCellRenderer(new JajukCellRenderer());
+    }
     acceptColumnsEvents = acceptColumnsEventsSave;
   }
 
   /*
    * Reorder columns order according to given conf
    */
+  /**
+   * Reorder columns.
+   * DOCUMENT_ME
+   */
   private void reorderColumns() {
     // Build the index array
     List<String> index = new ArrayList<String>(10);
@@ -188,13 +269,34 @@ public class JajukTable extends JXTable implements ListSelectionListener,
         getColumnModel().addColumn(column);
       }
     }
+
+    // set stored column width
+
+    // disable auto-resize temporary to set stored sizes
+    setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
+    String tableID = getTableId();
+
+    for (int currentColumnIndex = 0; currentColumnIndex < getColumnModel().getColumnCount(); currentColumnIndex++) {
+      String identifier = ((JajukTableModel) getModel())
+          .getIdentifier(convertColumnIndexToModel(currentColumnIndex));
+      String confId = tableID + "." + identifier + ".width";
+
+      if (Conf.containsProperty(confId)) {
+        getColumnModel().getColumn(currentColumnIndex).setPreferredWidth(Conf.getInt(confId));
+      }
+    }
+    setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
+
+    // must be done here and not before we add columns
+    if (Conf.containsProperty(getConfKeyForIsHorizontalScrollable())) {
+      setHorizontalScrollEnabled(Conf.getBoolean(getConfKeyForIsHorizontalScrollable()));
+    }
   }
 
   /**
+   * Gets the columns conf.
    * 
    * @return list of visible columns names as string
-   * @param Name
-   *          of the configuration key giving configuration
    */
   public List<String> getColumnsConf() {
     List<String> alOut = new ArrayList<String>(10);
@@ -207,9 +309,9 @@ public class JajukTable extends JXTable implements ListSelectionListener,
   }
 
   /**
-   * Add a new property into columns conf
+   * Add a new property into columns conf.
    * 
-   * @param property
+   * @param property DOCUMENT_ME
    */
   public void addColumnIntoConf(String property) {
     if (sConf == null) {
@@ -223,9 +325,9 @@ public class JajukTable extends JXTable implements ListSelectionListener,
   }
 
   /**
-   * Remove a property from columns conf
+   * Remove a property from columns conf.
    * 
-   * @param property
+   * @param property DOCUMENT_ME
    */
   public void removeColumnFromConf(String property) {
     if (sConf == null) {
@@ -236,6 +338,9 @@ public class JajukTable extends JXTable implements ListSelectionListener,
     Conf.setProperty(sConf, getColumnsConf(alOut));
   }
 
+  /**
+   * Column change. DOCUMENT_ME
+   */
   private void columnChange() {
     // ignore this column change when reloading
     // model
@@ -252,24 +357,39 @@ public class JajukTable extends JXTable implements ListSelectionListener,
     }
   }
 
+  /*
+   * (non-Javadoc)
+   * 
+   * @see javax.swing.JTable#columnAdded(javax.swing.event.TableColumnModelEvent)
+   */
   @Override
   public void columnAdded(TableColumnModelEvent evt) {
     super.columnAdded(evt);
     columnChange();
   }
 
+  /*
+   * (non-Javadoc)
+   * 
+   * @seeorg.jdesktop.swingx.JXTable#columnRemoved(javax.swing.event. TableColumnModelEvent)
+   */
   @Override
   public void columnRemoved(TableColumnModelEvent evt) {
     super.columnRemoved(evt);
     columnChange();
   }
 
+  /*
+   * (non-Javadoc)
+   * 
+   * @see javax.swing.JTable#columnMoved(javax.swing.event.TableColumnModelEvent)
+   */
   @Override
   public void columnMoved(TableColumnModelEvent evt) {
     super.columnMoved(evt);
     /*
-     * We ignore events if last to index is still the same for performances
-     * reasons (this event doesn't come with a isAdjusting() method)
+     * We ignore events if last to index is still the same for performances reasons (this event
+     * doesn't come with a isAdjusting() method)
      */
     if (acceptColumnsEvents && evt.getToIndex() != lastToIndex) {
       lastToIndex = evt.getToIndex();
@@ -280,9 +400,7 @@ public class JajukTable extends JXTable implements ListSelectionListener,
   }
 
   /**
-   * 
-   * Create the jtable visible columns conf
-   * 
+   * Create the jtable visible columns conf.
    */
   public void createColumnsConf() {
     StringBuilder sb = new StringBuilder();
@@ -303,9 +421,11 @@ public class JajukTable extends JXTable implements ListSelectionListener,
   }
 
   /**
+   * Gets the columns conf.
+   * 
+   * @param alCol DOCUMENT_ME
    * 
    * @return columns configuration from given list of columns identifiers
-   * 
    */
   private String getColumnsConf(List<String> alCol) {
     StringBuilder sb = new StringBuilder();
@@ -322,7 +442,11 @@ public class JajukTable extends JXTable implements ListSelectionListener,
   }
 
   /**
-   * add tooltips to each cell
+   * add tooltips to each cell.
+   * 
+   * @param e DOCUMENT_ME
+   * 
+   * @return the tool tip text
    */
   @Override
   public String getToolTipText(MouseEvent e) {
@@ -346,10 +470,9 @@ public class JajukTable extends JXTable implements ListSelectionListener,
   }
 
   /**
-   * Select a list of rows
+   * Select a list of rows.
    * 
-   * @param indexes
-   *          list of row indexes to be selected
+   * @param indexes list of row indexes to be selected
    */
   public void setSelectedRows(int[] indexes) {
     for (int element : indexes) {
@@ -357,16 +480,45 @@ public class JajukTable extends JXTable implements ListSelectionListener,
     }
   }
 
+  /*
+   * (non-Javadoc)
+   * 
+   * @see javax.swing.JTable#valueChanged(javax.swing.event.ListSelectionEvent)
+   */
   @Override
   public void valueChanged(ListSelectionEvent e) {
-    // Ignore adjusting event
     if (e.getValueIsAdjusting()) {
       return;
     }
-    // Make sure this table uses a Jajuk table model
-    if (!(getModel() instanceof JajukTableModel)) {
-      return;
+    // Ignore event if the model is refreshing
+    if (getModel() instanceof JajukTableModel) {
+      if (((JajukTableModel) getModel()).isRefreshing()) {
+        return;
+      }
     }
+    updateSelection();
+    // throw a table selection changed event providing the current perspective, view and
+    // selection (used for tree/table sync)
+    Properties properties = new Properties();
+    properties.put(Const.DETAIL_SELECTION, getSelection());
+    properties.put(Const.DETAIL_PERSPECTIVE, PerspectiveManager.getCurrentPerspective().getID());
+    // Test parent view nullity to avoid NPE
+    IView parentView = UtilGUI.getParentView(this);
+    if (parentView != null) {
+      properties.put(Const.DETAIL_VIEW, parentView);
+    }
+    ObservationManager.notify(new JajukEvent(JajukEvents.TABLE_SELECTION_CHANGED, properties));
+
+    // Call specific behaviors
+    for (ListSelectionListener listener : listeners) {
+      listener.valueChanged(e);
+    }
+  }
+
+  /**
+   * Update the selection.
+   */
+  private void updateSelection() {
     JajukTableModel model = (JajukTableModel) getModel();
     selection.clear();
     int[] rows = getSelectedRows();
@@ -374,63 +526,23 @@ public class JajukTable extends JXTable implements ListSelectionListener,
       Item o = model.getItemAt(convertRowIndexToModel(element));
       selection.add(o);
     }
-    ObservationManager.notify(new JajukEvent(JajukEvents.TABLE_SELECTION_CHANGED));
   }
 
+  /**
+   * Gets the selection.
+   * 
+   * @return the selection
+   */
   public List<Item> getSelection() {
     return this.selection;
   }
 
-  public void handlePopup(final MouseEvent e) {
-    int iSelectedRow = rowAtPoint(e.getPoint());
-    // Store real row index
-    TableTransferHandler.setSelectedRow(iSelectedRow);
-    // right click on a selected node set if none or 1 node is
-    // selected, a right click on another node
-    // select it if more than 1, we keep selection and display a
-    // popup for them
-    if (getSelectedRowCount() < 2) {
-      getSelectionModel().setSelectionInterval(iSelectedRow, iSelectedRow);
-    }
-    // Use getMenu() here, do not use jmenu directly as we want to enable all
-    // items before though getMenu() method
-    getMenu().show(this, e.getX(), e.getY());
-  }
-
   /**
    * Return generic popup menu for items in a table. <br>
-   * All items are forced to enable state
-   * 
-   * @TODO : this is probably not a good idea to force menu items to enable
    * 
    * @return generic popup menu for items in a table
    */
   public JPopupMenu getMenu() {
-    Component[] components = this.jmenu.getComponents();
-    for (int i = 0; i < components.length; i++) {
-      components[i].setEnabled(true);
-    }
-    return this.jmenu;
-  }
-
-  /**
-   * Return generic popup menu for items in a table. <br>
-   * The provided list allow to disable some items
-   * 
-   * @param indexToDisable
-   *          list of integer of indexes of items to disable
-   * @return generic popup menu for items in a table with filter
-   */
-  public JPopupMenu getMenu(List<Integer> indexToDisable) {
-    Component[] components = this.jmenu.getComponents();
-    int index = 0;
-    for (int i = 0; i < components.length; i++) {
-      if (components[i] instanceof JMenuItem || components[i] instanceof JMenu) {
-        // disable the item if its index is in the index list to disable
-        components[i].setEnabled(!indexToDisable.contains(index));
-        index++;
-      }
-    }
     return this.jmenu;
   }
 
@@ -439,6 +551,12 @@ public class JajukTable extends JXTable implements ListSelectionListener,
    * 
    * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
    */
+  /**
+   * Mouse clicked.
+   * DOCUMENT_ME
+   * 
+   * @param e DOCUMENT_ME
+   */
   public void mouseClicked(MouseEvent e) {
     // nothing to do here for now
   }
@@ -448,6 +566,12 @@ public class JajukTable extends JXTable implements ListSelectionListener,
    * 
    * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
    */
+  /**
+   * Mouse entered.
+   * DOCUMENT_ME
+   * 
+   * @param e DOCUMENT_ME
+   */
   public void mouseEntered(MouseEvent e) {
     // nothing to do here for now
   }
@@ -457,42 +581,175 @@ public class JajukTable extends JXTable implements ListSelectionListener,
    * 
    * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
    */
+  /**
+   * Mouse exited.
+   * DOCUMENT_ME
+   * 
+   * @param e DOCUMENT_ME
+   */
   public void mouseExited(MouseEvent e) {
     // nothing to do here for now
   }
 
+  /**
+   * Gets the command.
+   * 
+   * @return the command
+   */
+  public ILaunchCommand getCommand() {
+    return this.command;
+  }
+
+  /**
+   * Sets the command.
+   * 
+   * @param command the new command
+   */
+  public void setCommand(ILaunchCommand command) {
+    this.command = command;
+  }
+
+  /**
+   * Sets the accept columns events.
+   * 
+   * @param acceptColumnsEvents the new accept columns events
+   */
+  public void setAcceptColumnsEvents(boolean acceptColumnsEvents) {
+    this.acceptColumnsEvents = acceptColumnsEvents;
+  }
+
   /*
    * (non-Javadoc)
    * 
-   * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
+   * @see org.jajuk.events.Observer#getRegistrationKeys()
    */
-  public void mousePressed(MouseEvent e) {
-    if (e.isPopupTrigger()) {
-      handlePopup(e);
-    } else if (command != null && (e.getModifiersEx() & MouseEvent.CTRL_DOWN_MASK) == 0) {
-      command.launch(e.getClickCount());
-      int iSelectedRow = rowAtPoint(e.getPoint());
-      // Store real row index for drag and drop
-      TableTransferHandler.setSelectedRow(iSelectedRow);
+  public Set<JajukEvents> getRegistrationKeys() {
+    Set<JajukEvents> eventSubjectSet = new HashSet<JajukEvents>();
+    eventSubjectSet.add(JajukEvents.EXITING);
+    return eventSubjectSet;
+  }
+
+  /*
+   * (non-Javadoc)
+   * 
+   * @see org.jajuk.events.Observer#update(org.jajuk.events.JajukEvent)
+   */
+  public void update(JajukEvent event) {
+    JajukEvents subject = event.getSubject();
+    if (JajukEvents.EXITING.equals(subject)) {
+      Conf.setProperty(getConfKeyForIsHorizontalScrollable(), Boolean
+          .toString(isHorizontalScrollEnabled()));
+
+      // store column margin
+      String tableID = getTableId();
+
+      for (int currentColumnIndex = 0; currentColumnIndex < getColumnModel().getColumnCount(); currentColumnIndex++) {
+
+        String width = Integer.toString(getColumnModel().getColumn(currentColumnIndex).getWidth());
+        String identifier = ((JajukTableModel) getModel())
+            .getIdentifier(convertColumnIndexToModel(currentColumnIndex));
+
+        Conf.setProperty(tableID + "." + identifier + ".width", width);
+      }
     }
   }
 
-  public void mouseReleased(MouseEvent e) {
-    if (e.isPopupTrigger()) {
-      handlePopup(e);
+  /**
+   * Gets the conf key for is horizontal scrollable.
+   * 
+   * @return the conf key for is horizontal scrollable
+   */
+  private String getConfKeyForIsHorizontalScrollable() {
+    return getTableId() + ".is_horizontal_scrollable";
+  }
+
+  /**
+   * Gets the table id.
+   * 
+   * @return the table id
+   */
+  private String getTableId() {
+    String tableID = sConf;
+    if (tableID == null) {
+      tableID = "jajuk.table";
+    }
+    return tableID;
+  }
+
+  /**
+   * Remove previous alternate highlighter and add a new one
+   * It is required because after theme change, the alternate
+   * highlighter colors are no more valid.
+   * 
+   * @see org.jdesktop.swingx.JXTable#updateUI()
+   */
+  public void updateUI() {
+    for (Highlighter highlighter : getHighlighters()) {
+      if (highlighter instanceof CompoundHighlighter) {
+        if (UtilGUI.isAlternateColorHighlighter(highlighter)) {
+          removeHighlighter(highlighter);
+          UtilGUI.resetAlternateColorHighlighter();
+          addHighlighter(UtilGUI.getAlternateHighlighter());
+        }
+      }
+    }
+    super.updateUI();
+  }
+
+  // Fix for a JRE issue,see   :
+  // During a single adjusting ListSelectionEvent, several rows can be selected
+  // before the drag actually begins (except when using SINGLE_SELECTION selection mode). 
+  // For instance, select row 1 and release mouse
+  // then select row 2 without releasing the mouse and begin to drag from the top to the bottom :
+  // in some cases, when dragging quickly, rows 2 AND 3 (and even row 4 sometimes) are selected.
+  // Fix thanks jeffsabin  in http://forums.sun.com/thread.jspa?threadID=5436355
+
+  /* (non-Javadoc)
+   * @see javax.swing.JComponent#processMouseEvent(java.awt.event.MouseEvent)
+   */
+  @Override
+  protected void processMouseEvent(MouseEvent e) {
+    isMouseDragging = (e.getID() == MouseEvent.MOUSE_DRAGGED);
+    super.processMouseEvent(e);
+  }
+
+  /* (non-Javadoc)
+   * @see javax.swing.JComponent#processMouseMotionEvent(java.awt.event.MouseEvent)
+   */
+  @Override
+  protected void processMouseMotionEvent(MouseEvent e) {
+    isMouseDragging = (e.getID() == MouseEvent.MOUSE_DRAGGED);
+    super.processMouseMotionEvent(e);
+  }
+
+  /* (non-Javadoc)
+   * @see javax.swing.JTable#setRowSelectionInterval(int, int)
+   */
+  @Override
+  public void setRowSelectionInterval(int index0, int index1) {
+    if (!isMouseDragging) {
+      super.setRowSelectionInterval(index0, index1);
     }
   }
 
-  public ILaunchCommand getCommand() {
-    return this.command;
+  /* (non-Javadoc)
+   * @see javax.swing.JTable#setColumnSelectionInterval(int, int)
+   */
+  @Override
+  public void setColumnSelectionInterval(int index0, int index1) {
+    if (!isMouseDragging) {
+      super.setColumnSelectionInterval(index0, index1);
+    }
   }
 
-  public void setCommand(ILaunchCommand command) {
-    this.command = command;
-  }
-
-  public void setAcceptColumnsEvents(boolean acceptColumnsEvents) {
-    this.acceptColumnsEvents = acceptColumnsEvents;
+  /* (non-Javadoc)
+   * @see javax.swing.JTable#changeSelection(int, int, boolean, boolean)
+   */
+  @Override
+  public void changeSelection(int rowIndex, int columnIndex, boolean toggle, boolean extend) {
+    if (!isMouseDragging) {
+      super.changeSelection(rowIndex, columnIndex, toggle, extend);
+    }
   }
 
 }
