@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2007 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -33,21 +34,20 @@ import javax.swing.ImageIcon;
  */
 final public class IconLoader {
 
-  /** No covers image cache : size:default icon */
-  private static Map<String, ImageIcon> nocoverCache = new HashMap<String, ImageIcon>(10);
-
-  /** Icons cache * */
+  /** Icons cache *. */
   private static Map<JajukIcons, ImageIcon> cache = new HashMap<JajukIcons, ImageIcon>(200);
 
   /**
-   * No instantiation
+   * No instantiation.
    */
   private IconLoader() {
   }
 
   /**
-   * @param nocover
-   *          size
+   * Gets the no cover icon.
+   * 
+   * @param size The size of the icon to return.
+   * 
    * @return the nocover icon for specified size
    */
   public static ImageIcon getNoCoverIcon(int size) {
@@ -57,10 +57,12 @@ final public class IconLoader {
   }
 
   /**
-   * @param icon
-   *          the JajukIcons name
+   * Gets the icon.
+   * 
+   * @param icon the JajukIcons name
+   * 
    * @return an image icon for given icon name.Note that all images are cached
-   *         before being returned.
+   * before being returned.
    */
   public static ImageIcon getIcon(JajukIcons icon) {
 
@@ -75,6 +77,10 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("images/included/" + Const.FILE_THUMB_NO_COVER));
     }
 
+    else if (icon == JajukIcons.NONE) {
+      out = new ImageIcon(UtilSystem.getResource("images/included/" + Const.FILE_THUMB_NONE));
+    }
+
     else if (icon == JajukIcons.NO_COVER_50X50) {
       out = UtilGUI.getResizedImage(getIcon(JajukIcons.NO_COVER), 50, 50);
     }
@@ -120,6 +126,10 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/repeat_16x16.png"));
     }
 
+    else if (icon == JajukIcons.REPEAT_ALL) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/repeat_all_16x16.png"));
+    }
+
     else if (icon == JajukIcons.SHUFFLE) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/shuffle_16x16.png"));
     }
@@ -132,6 +142,10 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/intro_16x16.png"));
     }
 
+    else if (icon == JajukIcons.KARAOKE) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/karaoke_16x16.png"));
+    }
+
     else if (icon == JajukIcons.SHUFFLE_GLOBAL) {
       out = new ImageIcon(UtilSystem.getResource("icons/32x32/shuffle_global_32x32.png"));
     }
@@ -176,24 +190,32 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/novelties_16x16.png"));
     }
 
-    else if (icon == JajukIcons.NEXT) {
-      out = new ImageIcon(UtilSystem.getResource("icons/16x16/next_16x16.png"));
+    else if (icon == JajukIcons.PLAYER_NEXT_SMALL) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/player_next_16x16.png"));
     }
 
     else if (icon == JajukIcons.SAVE_AS) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/saveas_16x16.png"));
     }
 
-    else if (icon == JajukIcons.PREVIOUS) {
-      out = new ImageIcon(UtilSystem.getResource("icons/16x16/previous_16x16.png"));
+    else if (icon == JajukIcons.PLAYER_PREVIOUS_SMALL) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/player_previous_16x16.png"));
     }
 
     else if (icon == JajukIcons.PLAYER_PREVIOUS) {
-      out = new ImageIcon(UtilSystem.getResource("icons/32x32/previous_32x32.png"));
+      out = new ImageIcon(UtilSystem.getResource("icons/32x32/player_previous_32x32.png"));
+    }
+
+    else if (icon == JajukIcons.PLAYER_PREVIOUS_BIG) {
+      out = new ImageIcon(UtilSystem.getResource("icons/64x64/player_previous_64x64.png"));
     }
 
     else if (icon == JajukIcons.PLAYER_NEXT) {
-      out = new ImageIcon(UtilSystem.getResource("icons/32x32/next_32x32.png"));
+      out = new ImageIcon(UtilSystem.getResource("icons/32x32/player_next_32x32.png"));
+    }
+
+    else if (icon == JajukIcons.PLAYER_NEXT_BIG) {
+      out = new ImageIcon(UtilSystem.getResource("icons/64x64/player_next_64x64.png"));
     }
 
     else if (icon == JajukIcons.INC_RATING) {
@@ -204,30 +226,50 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/32x32/player_rew_32x32.png"));
     }
 
+    else if (icon == JajukIcons.PLAYER_REWIND_BIG) {
+      out = new ImageIcon(UtilSystem.getResource("icons/64x64/player_rew_64x64.png"));
+    }
+
+    else if (icon == JajukIcons.AUTOSCROLL) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/autoscroll.png"));
+    }
+
     else if (icon == JajukIcons.REW_16X16) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/player_rew_16x16.png"));
     }
 
-    else if (icon == JajukIcons.PLAY) {
+    else if (icon == JajukIcons.PLAYER_PLAY) {
       out = new ImageIcon(UtilSystem.getResource("icons/32x32/player_play_32x32.png"));
     }
 
+    else if (icon == JajukIcons.PLAYER_PLAY_BIG) {
+      out = new ImageIcon(UtilSystem.getResource("icons/64x64/player_play_64x64.png"));
+    }
+
     else if (icon == JajukIcons.PLAY_16X16) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/player_play_16x16.png"));
     }
 
-    else if (icon == JajukIcons.PAUSE) {
+    else if (icon == JajukIcons.PLAYER_PAUSE) {
       out = new ImageIcon(UtilSystem.getResource("icons/32x32/player_pause_32x32.png"));
     }
 
+    else if (icon == JajukIcons.PLAYER_PAUSE_BIG) {
+      out = new ImageIcon(UtilSystem.getResource("icons/64x64/player_pause_64x64.png"));
+    }
+
     else if (icon == JajukIcons.PAUSE_16X16) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/player_pause_16x16.png"));
     }
 
-    else if (icon == JajukIcons.STOP) {
+    else if (icon == JajukIcons.PLAYER_STOP) {
       out = new ImageIcon(UtilSystem.getResource("icons/32x32/player_stop_32x32.png"));
     }
 
+    else if (icon == JajukIcons.PLAYER_STOP_BIG) {
+      out = new ImageIcon(UtilSystem.getResource("icons/64x64/player_stop_64x64.png"));
+    }
+
     else if (icon == JajukIcons.STOP_16X16) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/player_stop_16x16.png"));
     }
@@ -236,6 +278,10 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/32x32/player_fwd_32x32.png"));
     }
 
+    else if (icon == JajukIcons.PLAYER_FORWARD_BIG) {
+      out = new ImageIcon(UtilSystem.getResource("icons/64x64/player_fwd_64x64.png"));
+    }
+
     else if (icon == JajukIcons.FWD_16X16) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/player_fwd_16x16.png"));
     }
@@ -248,7 +294,11 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/close_16x16.png"));
     }
 
-    else if (icon == JajukIcons.FULL_WINDOW) {
+    else if (icon == JajukIcons.SLIM_WINDOW) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/slim_window_16x16.png"));
+    }
+
+    else if (icon == JajukIcons.FULL_SCREEN) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/full_window_16x16.png"));
     }
 
@@ -480,16 +530,16 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/playlist_16x16.png"));
     }
 
-    else if (icon == JajukIcons.STYLE) {
-      out = new ImageIcon(UtilSystem.getResource("icons/16x16/style_16x16.png"));
+    else if (icon == JajukIcons.GENRE) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/genre_16x16.png"));
     }
 
     else if (icon == JajukIcons.EMPTY) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/empty_16x16.png"));
     }
 
-    else if (icon == JajukIcons.AUTHOR) {
-      out = new ImageIcon(UtilSystem.getResource("icons/16x16/author_16x16.png"));
+    else if (icon == JajukIcons.ARTIST) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/artist_16x16.png"));
     }
 
     else if (icon == JajukIcons.ALBUM) {
@@ -604,6 +654,10 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/repeat_16x16.png"));
     }
 
+    else if (icon == JajukIcons.TRACK_FIFO_PLAYING) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/ok_16x16.png"));
+    }
+
     else if (icon == JajukIcons.WIZARD) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/wizard_16x16.png"));
     }
@@ -828,6 +882,10 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/ext_drive_16x16.png"));
     }
 
+    else if (icon == JajukIcons.PREPARE_PARTY_32X32) {
+      out = new ImageIcon(UtilSystem.getResource("icons/32x32/ext_drive_32x32.png"));
+    }
+
     else if (icon == JajukIcons.COPY_TO_CLIPBOARD) {
       out = new ImageIcon(UtilSystem.getResource("icons/16x16/copy_url_clipboard_16x16.png"));
     }
@@ -836,6 +894,22 @@ final public class IconLoader {
       out = new ImageIcon(UtilSystem.getResource("icons/32x32/ambience_32x32.png"));
     }
 
+    else if (icon == JajukIcons.OPEN_EXPLORER) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/explorer_16x16.png"));
+    }
+
+    else if (icon == JajukIcons.TREE_TABLE_SYNC) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/tree-table-sync_16x16.png"));
+    }
+
+    else if (icon == JajukIcons.DONATE) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/donate_16x16.png"));
+    }
+
+    else if (icon == JajukIcons.DISC_NUMBER) {
+      out = new ImageIcon(UtilSystem.getResource("icons/16x16/disc_number_16x16.png"));
+    }
+
     // Cache the result
     if (!cache.containsKey(icon)) {
       cache.put(icon, out);
