@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2003 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,34 +16,38 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 4517 $
+ *  $Revision: 6186 $
  */
 package org.jajuk.util;
 
+import java.awt.Component;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
 import java.net.URL;
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.List;
+import java.util.Locale;
 import java.util.Properties;
 import java.util.StringTokenizer;
 
-import javax.swing.BoxLayout;
 import javax.swing.Icon;
 import javax.swing.ImageIcon;
-import javax.swing.JButton;
 import javax.swing.JDialog;
 import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.JTextArea;
 import javax.swing.SwingUtilities;
+import javax.xml.parsers.ParserConfigurationException;
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
 
-import org.jajuk.ui.widgets.JajukWindow;
+import net.miginfocom.swing.MigLayout;
+
+import org.jajuk.ui.widgets.OKCancelPanel;
+import org.jajuk.ui.windows.JajukMainWindow;
 import org.jajuk.util.log.Log;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.DefaultHandler;
@@ -51,68 +56,33 @@ import org.xml.sax.helpers.DefaultHandler;
  * Utility class to get strings from localized property files
  * <p>
  * Singleton
- * </p>
+ * </p>.
  */
 public class Messages extends DefaultHandler {
-  /** Local ( language) to be used, default is English */
-  private static String sLocal = "en";
 
-  /** Supported Locals */
-  private static List<String> alLocals = new ArrayList<String>(10);
+  /** Messages initialized flag. */
+  protected static boolean bInitialized = false;
 
-  /** Messages initialized flag */
-  private static boolean bInitialized = false;
-
-  /** All choice option, completes JDialog options */
+  /** All choice option, completes JDialog options. */
   public static final int ALL_OPTION = 10;
 
-  /** Specific Yes NO All Cancel option */
+  /** Specific Yes NO All Cancel option. */
   public static final int YES_NO_ALL_CANCEL_OPTION = 11;
 
-  /**
-   * Messages themselves extracted from an XML file to this properties class*
-   */
-  private static Properties properties;
+  /** User choice. */
+  private static int choice;
 
-  /** English messages used as default* */
+  /** Messages themselves extracted from an XML file to this properties class*. */
+  protected static Properties properties;
+
+  /** English messages used as default*. */
   private static Properties propertiesEn;
 
-  /** Conf Locales */
-  public static final String[] LOCALES = { "en", "fr", "de", "nl", "es", "ca", "ko", "el", "ru",
-      "gl" };
-
-  /**
-   * We set here environment language as default language (used by first time
-   * wizard for ie) but this value can be overwritten later in startup process
-   */
-  static {
-    // Register locals, needed by Conf to choose
-    // default language
-    for (final String locale : LOCALES) {
-      Messages.registerLocal(locale);
-    }
-    // Set default local if available
-    sLocal = getNativeLocale();
-  }
-
   /**
+   * Contains.
    * 
-   * @return current default native locale or English if the native is not
-   *         supported by Jajuk
-   */
-  public static String getNativeLocale() {
-    String sLanguage = System.getProperty("user.language");
-    if (sLanguage != null && getLocales().contains(sLanguage)) {
-      return sLanguage;
-    } else { // user language is unknown, take English as a default,
-      // user will be able to change it later anyway
-      return "en";
-    }
-  }
-
-  /**
+   * @param sKey DOCUMENT_ME
    * 
-   * @param sKey
    * @return whether given key exists
    */
   public static boolean contains(final String sKey) {
@@ -120,8 +90,11 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * @param key
-   * @return
+   * Gets the string.
+   * 
+   * @param key DOCUMENT_ME
+   * 
+   * @return the string
    */
   public static String getString(final String key) {
     String sOut = key;
@@ -144,25 +117,28 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * Fetch all messages from a given base key. <P/> Example:
+   * Fetch all messages from a given base key.
+   * <P/>
+   * Example:
    * 
    * <pre>
-   *     example.0=Message 1
-   *     example.1=Message 2
-   *     example.2=Message 3
+   * example.0=Message 1
+   * example.1=Message 2
+   * example.2=Message 3
    * </pre>
    * 
    * Using <tt>Messages.getAll("example");</tt> will return a size 3 String
-   * array containing the messages in order. <P/> The keys need to have
-   * continuous numbers. So, adding <tt>example.5=Message 5</tt> to the
-   * bundle, will not result in adding it to the array without first adding
-   * <tt>example.3</tt> and <tt>example.4</tt>.
+   * array containing the messages in order.
+   * <P/>
+   * The keys need to have continuous numbers. So, adding
+   * <tt>example.5=Message 5</tt> to the bundle, will not result in adding it to
+   * the array without first adding <tt>example.3</tt> and <tt>example.4</tt>.
+   * 
+   * @param base The base to use for generating the keys.
    * 
-   * @param base
-   *          The base to use for generating the keys.
    * @return An array of Strings containing the messages linked to the key,
-   *         never <tt>null</tt>. If <tt>base.0</tt> is not found, and
-   *         empty array is returned.
+   * never <tt>null</tt>. If <tt>base.0</tt> is not found, and empty
+   * array is returned.
    */
   public static String[] getAll(final String base) {
     final List<String> msgs = new ArrayList<String>();
@@ -195,15 +171,16 @@ public class Messages extends DefaultHandler {
   }
 
   /**
+   * Gets the shuffle tip of the day.
    * 
    * @return a shuffled tip of the day <br>
    */
   public static String getShuffleTipOfTheDay() {
     try {
       String totd = null;
-      String[] TIPS = Messages.getAll("TipOfTheDay");
+      String[] tips = Messages.getAll("TipOfTheDay");
       // index contains the index of the last provided totd
-      int index = (int) (UtilSystem.getRandom().nextFloat() * (TIPS.length - 1));
+      int index = (int) (UtilSystem.getRandom().nextFloat() * (tips.length - 1));
       // display the next one
       totd = Messages.getString("TipOfTheDay." + index);
       // Remove <img> tags
@@ -219,109 +196,51 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * Register a local
+   * Return Flag icon for given description.
    * 
-   * @param sLocale :
-   *          standard local name like "en"
-   * @param sDesc :
-   *          a language-independent descriptions like "Language_desc_en"
-   */
-  public static void registerLocal(final String sLocal) {
-    alLocals.add(sLocal);
-  }
-
-  /**
-   * Return Flag icon for given description
+   * @param sDesc DOCUMENT_ME
    * 
-   * @param dDesc
-   *          language description
-   * @return
+   * @return the icon
    */
   public static Icon getIcon(final String sDesc) {
-    Icon icon = new ImageIcon(UtilSystem.getResource("icons/16x16/flag_" + getLocalForDesc(sDesc)
-        + ".png"));
+    Icon icon = new ImageIcon(UtilSystem.getResource("icons/16x16/flag_"
+        + LocaleManager.getLocaleForDesc(sDesc) + ".png"));
     return icon;
   }
 
   /**
-   * Return list of available locals
+   * ***************************************************************************
+   * Parse a fake properties file inside an XML file as CDATA.
    * 
-   * @return
-   */
-  public static List<String> getLocales() {
-    return alLocals;
-  }
-
-  /**
-   * Return list of available local descriptions
+   * @param locale DOCUMENT_ME
    * 
-   * @return
-   */
-  public static List<String> getDescs() {
-    final List<String> alDescs = new ArrayList<String>(10);
-    for (final String local : alLocals) {
-      alDescs.add(getString("Language_desc_" + local));
-    }
-    Collections.sort(alDescs);
-    return alDescs;
-  }
-
-  /**
-   * Return Description for a given local id
-   * 
-   * @return localized description
-   */
-  public static String getDescForLocal(final String sLocal) {
-    return getString("Language_desc_" + sLocal);
-  }
-
-  /**
-   * Return local for a given description
-   * 
-   * @return local
-   */
-  public static String getLocalForDesc(final String sDesc) {
-    for (final String locale : getLocales()) {
-      if (getDescForLocal(locale).equals(sDesc)) {
-        return locale;
-      }
-    }
-    return null;
-  }
-
-  /**
-   * Change current local
-   * 
-   * @param sLocal
-   */
-  public static void setLocal(final String sLocal) throws Exception {
-    Conf.setProperty(Const.CONF_OPTIONS_LANGUAGE, sLocal);
-    properties = null; // make sure to reinitialize cached strings
-    Messages.sLocal = sLocal;
-    bInitialized = true;
-  }
-
-  /*****************************************************************************
-   * Parse a fake properties file inside an XML file as CDATA
-   * 
-   * @param sLocal
    * @return a properties with all entries
-   * @throws Exception
-   */
-  private static Properties parseLangpack(final String sLocal) throws Exception {
+   * 
+   * @throws IOException Signals that an I/O exception has occurred.
+   * @throws SAXException the SAX exception
+   * @throws ParserConfigurationException the parser configuration exception
+   * @throws Exception    */
+  private static Properties parseLangpack(final Locale locale) throws SAXException, IOException,
+      ParserConfigurationException {
     final Properties lProperties = new Properties();
+
     // Choose right jajuk_<lang>.properties file to load
     final StringBuilder sbFilename = new StringBuilder(Const.FILE_LANGPACK_PART1);
-    if (!sLocal.equals("en")) { // for english, properties file is
+    if (!Locale.ENGLISH.equals(locale)) { // for English, properties file is
       // simply jajuk.properties
-      sbFilename.append('_').append(sLocal);
+      sbFilename.append('_').append(locale);
     }
     sbFilename.append(Const.FILE_LANGPACK_PART2);
-    URL url;
+
     // property file URL, either in the jajuk.jar jar
     // (normal execution) or found as regular file if in
     // development debug mode
-    url = UtilSystem.getResource("org/jajuk/i18n/" + sbFilename.toString());
+    String resource = "org/jajuk/i18n/" + sbFilename.toString();
+    URL url = UtilSystem.getResource(resource);
+    if (url == null) {
+      throw new IOException("Could not read resource: " + resource);
+    }
+
     // parse it, actually it is a big properties file as CDATA in an XML
     // file
     final SAXParserFactory spf = SAXParserFactory.newInstance();
@@ -367,8 +286,8 @@ public class Messages extends DefaultHandler {
   /**
    * Return the message display to the user corresponding to the error code.
    * 
-   * @param code
-   *          Error code.
+   * @param code Error code.
+   * 
    * @return String Message corresponding to the error code.
    */
   public static String getErrorMessage(final int code) {
@@ -384,26 +303,49 @@ public class Messages extends DefaultHandler {
   /**
    * Show a dialog waiting for a user decision
    * <p>
-   * CAUTION! the thread which calls this method musn't have locks on resources :
-   * otherwise it can conduct to GUI freeze
-   * </p>
+   * CAUTION! the thread which calls this method musn't have locks on resources
+   * : otherwise it can conduct to GUI freeze
+   * </p>.
    * 
-   * @param sText :
-   *          dialog text
-   * @param int
-   *          optionType : kind of options like JOptionPane.OK_CANCEL
-   * @param iType
-   *          message type like JOptionPane.WARNING
+   * @param sText : dialog text
+   * @param iType message type like JOptionPane.WARNING
+   * @param optionsType DOCUMENT_ME
+   * 
+   * @return the choice
    */
   public static int getChoice(final String sText, final int optionsType, final int iType) {
-    final ConfirmDialog confirm = new ConfirmDialog(sText, getTitleForType(iType), optionsType,
-        iType);
-    return confirm.getResu();
+    try {
+      // Make sure to reset the choice and to return a non-existing choice if
+      // the GUI fails
+      choice = JOptionPane.DEFAULT_OPTION;
+      Runnable t = new Thread("Get choice thread") {
+        @Override
+        public void run() {
+          // This must be done in the EDT
+          final ConfirmDialog confirm = new ConfirmDialog(sText, getTitleForType(iType),
+              optionsType, iType, JajukMainWindow.getInstance());
+          choice = confirm.getResu();
+        }
+      };
+      // invokeAndWait method cannot be called from the EDT
+      if (SwingUtilities.isEventDispatchThread()) {
+        t.run();
+      } else {
+        SwingUtilities.invokeAndWait(t);
+      }
+    } catch (InterruptedException e) {
+      Log.error(e);
+    } catch (InvocationTargetException e) {
+      Log.error(e);
+    }
+    return choice;
   }
 
   /**
+   * Gets the title for type.
+   * 
+   * @param iType DOCUMENT_ME
    * 
-   * @param iType
    * @return String for given JOptionPane message type
    */
   private static String getTitleForType(final int iType) {
@@ -419,51 +361,52 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * Show a dialog with specified warning message
+   * Show a dialog with specified warning message.
    * 
-   * @param sMessage
+   * @param sMessage DOCUMENT_ME
    */
   public static void showWarningMessage(final String sMessage) {
-    new DetailsMessageDialog(sMessage, getTitleForType(JOptionPane.WARNING_MESSAGE),
-        JOptionPane.WARNING_MESSAGE, null, null);
+    SwingUtilities.invokeLater(new Runnable() {
+      @Override
+      public void run() {
+        new DetailsMessageDialog(sMessage, getTitleForType(JOptionPane.WARNING_MESSAGE),
+            JOptionPane.WARNING_MESSAGE, null, null);
+      }
+    });
   }
 
   /**
-   * Show a dialog with specified warning message + a "not show again" button
+   * Show a dialog with specified warning message + a "not show again" button.
    * 
-   * @param sMessage
-   * @param sProperty :
-   *          property name
+   * @param sMessage DOCUMENT_ME
+   * @param sProperty : property name
    */
   public static void showHideableWarningMessage(final String sMessage, final String sProperty) {
     // User required to hide this message
     if (Conf.getBoolean(sProperty)) {
       return;
     }
-    try {
-      SwingUtilities.invokeAndWait(new Runnable() {
-        public void run() {
-          final HideableMessageDialog message = new HideableMessageDialog(sMessage,
-              getTitleForType(JOptionPane.WARNING_MESSAGE), sProperty, JOptionPane.WARNING_MESSAGE,
-              null);
-          message.getResu();
-        }
+    SwingUtilities.invokeLater(new Runnable() {
+      @Override
+      public void run() {
+        final HideableMessageDialog message = new HideableMessageDialog(sMessage,
+            getTitleForType(JOptionPane.WARNING_MESSAGE), sProperty, JOptionPane.WARNING_MESSAGE,
+            null);
+        message.getResu();
+      }
 
-      });
-    } catch (InterruptedException e) {
-      Log.error(e);
-    } catch (InvocationTargetException e) {
-      Log.error(e);
-    }
+    });
   }
 
   /**
-   * Show a dialog with specified error message and an icon
+   * Show a dialog with specified error message and an icon.
    * 
-   * @param sMessage
+   * @param sMessage DOCUMENT_ME
+   * @param icon DOCUMENT_ME
    */
   public static void showInfoMessage(final String sMessage, final Icon icon) {
     SwingUtilities.invokeLater(new Runnable() {
+      @Override
       public void run() {
         new DetailsMessageDialog(sMessage, getTitleForType(JOptionPane.INFORMATION_MESSAGE),
             JOptionPane.INFORMATION_MESSAGE, null, icon);
@@ -472,13 +415,14 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * Show a dialog with specified error message and infosup
+   * Show a dialog with specified error message and infosup.
    * 
-   * @param code
-   * @param sInfoSup
+   * @param code DOCUMENT_ME
+   * @param sInfoSup DOCUMENT_ME
    */
   public static void showErrorMessage(final int code, final String sInfoSup) {
     SwingUtilities.invokeLater(new Runnable() {
+      @Override
       public void run() {
         new ErrorMessageDialog(code, sInfoSup);
       }
@@ -486,23 +430,25 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * Show a dialog with specified error message
+   * Show a dialog with specified error message.
    * 
-   * @param sCode
+   * @param code DOCUMENT_ME
    */
   public static void showErrorMessage(final int code) {
     showErrorMessage(code, null);
   }
 
   /**
-   * Show a dialog with specified error message and infosup and details
+   * Show a dialog with specified error message and infosup and details.
    * 
-   * @param sCode
-   * @param sInfoSup
+   * @param sInfoSup DOCUMENT_ME
+   * @param code DOCUMENT_ME
+   * @param sDetails DOCUMENT_ME
    */
   public static void showDetailedErrorMessage(final int code, final String sInfoSup,
       final String sDetails) {
     SwingUtilities.invokeLater(new Runnable() {
+      @Override
       public void run() {
         new DetailsMessageDialog(Messages.getErrorMessage(code) + " : " + sInfoSup,
             getTitleForType(JOptionPane.ERROR_MESSAGE), JOptionPane.ERROR_MESSAGE, sDetails, null);
@@ -511,13 +457,14 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * Show a dialog with specified error message with infos up
+   * Show a dialog with specified error message with infos up.
    * 
-   * @param sMessage
-   * @param sInfoSup
+   * @param sMessage DOCUMENT_ME
+   * @param sInfoSup DOCUMENT_ME
    */
   public static void showInfoMessage(final String sMessage, final String sInfoSup) {
     SwingUtilities.invokeLater(new Runnable() {
+      @Override
       public void run() {
         new DetailsMessageDialog(sMessage + " : " + sInfoSup,
             getTitleForType(JOptionPane.INFORMATION_MESSAGE), JOptionPane.INFORMATION_MESSAGE,
@@ -527,12 +474,13 @@ public class Messages extends DefaultHandler {
   }
 
   /**
-   * Show a dialog with specified error message
+   * Show a dialog with specified error message.
    * 
-   * @param sMessage
+   * @param sMessage DOCUMENT_ME
    */
   public static void showInfoMessage(final String sMessage) {
     SwingUtilities.invokeLater(new Runnable() {
+      @Override
       public void run() {
         new DetailsMessageDialog(sMessage, getTitleForType(JOptionPane.INFORMATION_MESSAGE),
             JOptionPane.INFORMATION_MESSAGE, null, null);
@@ -540,46 +488,63 @@ public class Messages extends DefaultHandler {
     });
   }
 
-  /**
-   * @return Returns the sLocal.
-   */
-  public static String getLocale() {
-    return sLocal;
-  }
-
   /**
    * Return true if the messaging system is started, can be useful mainly at
    * startup by services ( like logs) using them to avoid dead locks Messages
-   * service is initialized after current has been set
+   * service is initialized after current has been set.
    * 
-   * @return
+   * @return true, if checks if is initialized
    */
   public static boolean isInitialized() {
     return bInitialized;
   }
 
   /**
-   * @return Returns the properties.
+   * Gets localized and human property name for given key.
+   * 
+   * @param sKey DOCUMENT_ME
+   * 
+   * @return the human property name or the property itself if not translated
    */
-  public static Properties getProperties() throws Exception {
+  public static String getHumanPropertyName(String sKey) {
+    String sOut = sKey;
+    if (Messages.contains(Const.PROPERTY_SEPARATOR + sKey)) {
+      return Messages.getString(Const.PROPERTY_SEPARATOR + sKey);
+    }
+    return sOut;
+  }
+
+  /**
+   * Gets the properties.
+   * 
+   * @return Returns the properties.
+   * 
+   * @throws ParserConfigurationException the parser configuration exception
+   * @throws IOException Signals that an I/O exception has occurred.
+   * @throws SAXException the SAX exception
+   */
+  public static Properties getProperties() throws SAXException, IOException,
+      ParserConfigurationException {
     if (properties == null) {
       // reuse English if possible
-      if (sLocal.equals("en")) {
+      if (Locale.ENGLISH.equals(LocaleManager.getLocale())) {
         properties = getPropertiesEn();
       } else {
-        properties = parseLangpack(sLocal);
+        properties = parseLangpack(LocaleManager.getLocale());
       }
     }
     return properties;
   }
 
   /**
+   * Gets the properties en.
+   * 
    * @return Returns the propertiesEn.
    */
   public static Properties getPropertiesEn() {
     if (propertiesEn == null) {
       try {
-        propertiesEn = parseLangpack("en");
+        propertiesEn = parseLangpack(Locale.ENGLISH);
       } catch (final Exception e) {
         Log.error(e);
       }
@@ -598,15 +563,17 @@ class ConfirmDialog extends JajukDialog {
    * 
    * @param sText
    * @param sTitle
-   * @param int
-   *          optionType : kind of options like JOptionPane.OK_CANCEL
-   *          <p>
-   *          Specific option: Messages.ALL_OPTION
-   *          </p>
+   * @param int optionType : kind of options like JOptionPane.OK_CANCEL
+   *        <p>
+   *        Specific option: Messages.ALL_OPTION
+   *        </p>
    * @param iType
    *          message type like JOptionPane.WARNING
    */
-  ConfirmDialog(final String sText, final String sTitle, final int optionsType, final int iType) {
+  ConfirmDialog(final String sText, final String sTitle, final int optionsType, final int iType,
+      Component parent) {
+    super();
+
     final JOptionPane optionPane = UtilGUI.getNarrowOptionPane(72);
     if (optionsType == Messages.YES_NO_ALL_CANCEL_OPTION) {
       optionPane.setOptions(new Object[] { Messages.getString("Yes"), Messages.getString("No"),
@@ -620,7 +587,7 @@ class ConfirmDialog extends JajukDialog {
     dialog.setModal(true);
     dialog.setAlwaysOnTop(true);
     dialog.pack();
-    dialog.setLocationRelativeTo(JajukWindow.getInstance());
+    dialog.setLocationRelativeTo(parent);
     dialog.setVisible(true);
     final Object resu = optionPane.getValue();
     // Set Cancel as default
@@ -630,27 +597,26 @@ class ConfirmDialog extends JajukDialog {
       iResu = JOptionPane.CANCEL_OPTION;
     } else if (resu instanceof String) {
       // Options are string when using custom options
-      if (resu.equals(Messages.getString("All"))) {
+      if (resu.equals(Messages.getString("YestoAll"))) {
         iResu = Messages.ALL_OPTION;
       } else if (resu.equals(Messages.getString("Yes"))) {
         iResu = JOptionPane.YES_OPTION;
-      }
-      if (resu.equals(Messages.getString("No"))) {
+      } else if (resu.equals(Messages.getString("No"))) {
         iResu = JOptionPane.NO_OPTION;
-      }
-      if (resu.equals(Messages.getString("Cancel"))) {
+      } else if (resu.equals(Messages.getString("Cancel"))) {
         iResu = JOptionPane.CANCEL_OPTION;
-      }
-      if (resu.equals(Messages.getString("Ok"))) {
+      } else if (resu.equals(Messages.getString("Ok"))) {
         iResu = JOptionPane.OK_OPTION;
-      }
-      if (resu.equals(Messages.getString("Default"))) {
+      } else if (resu.equals(Messages.getString("Default"))) {
         iResu = JOptionPane.DEFAULT_OPTION;
       }
     } else if (resu instanceof Integer) {
       // result is an integer when using JOptionPane standard types
       iResu = (Integer) resu;
     }
+
+    // manually dispose to free up memory, somehow this is not done automatically!
+    dialog.dispose();
   }
 
 }
@@ -669,6 +635,8 @@ class DetailsMessageDialog extends JajukDialog {
    */
   DetailsMessageDialog(final String sText, final String sTitle, final int iType,
       final String sDetails, final Icon icon) {
+    super();
+
     final JOptionPane optionPane = UtilGUI.getNarrowOptionPane(72);
     optionPane.setMessage(sText);
     if (sDetails != null) {
@@ -687,24 +655,30 @@ class DetailsMessageDialog extends JajukDialog {
       // details
       final JDialog dialogDetail = new JDialog(dialog, Messages.getString("Details"));
       final JPanel jp = new JPanel();
-      jp.setLayout(new BoxLayout(jp, BoxLayout.Y_AXIS));
+      jp.setLayout(new MigLayout("ins 5", "[800]", "[600][]"));
       final JTextArea jta = new JTextArea(sDetails);
       jta.setEditable(false);
-      jp.add(new JScrollPane(jta));
-      final JButton jbOK = new JButton(Messages.getString("Ok"));
-      jbOK.addActionListener(new ActionListener() {
+      jp.add(new JScrollPane(jta), "wrap");
+      final OKCancelPanel jbOKC = new OKCancelPanel(new ActionListener() {
+        @Override
         public void actionPerformed(final ActionEvent arg0) {
           dialogDetail.dispose();
         }
       });
-      jp.add(UtilGUI.getCentredPanel(jbOK));
+      jp.add(jbOKC, "span,alignx center");
       dialogDetail.setModal(true);
       dialogDetail.setAlwaysOnTop(true);
       dialogDetail.setContentPane(jp);
       dialogDetail.pack();
-      dialogDetail.setLocationRelativeTo(JajukWindow.getInstance());
+      dialogDetail.setLocationRelativeTo(JajukMainWindow.getInstance());
       dialogDetail.setVisible(true);
+
+      // manually dispose to free up memory, somehow this is not done automatically!
+      dialog.dispose();
     }
+
+    // manually dispose to free up memory, somehow this is not done automatically!
+    dialog.dispose();
   }
 
 }
@@ -725,6 +699,8 @@ class HideableMessageDialog extends JajukDialog {
    */
   HideableMessageDialog(final String sText, final String sTitle, final String sProperty,
       final int iType, final Icon icon) {
+    super();
+
     final JOptionPane optionPane = UtilGUI.getNarrowOptionPane(72);
     optionPane.setMessage(UtilGUI.getLimitedMessage(sText, 20));
     final Object[] options = { Messages.getString("Ok"), Messages.getString("Hide") };
@@ -738,12 +714,15 @@ class HideableMessageDialog extends JajukDialog {
     // keep it modal (useful at startup)
     dialog.setModal(true);
     dialog.pack();
-    dialog.setLocationRelativeTo(JajukWindow.getInstance());
+    dialog.setLocationRelativeTo(JajukMainWindow.getInstance());
     dialog.setVisible(true);
     if (Messages.getString("Hide").equals(optionPane.getValue())) {
       // Not show again
       Conf.setProperty(sProperty, Const.TRUE);
     }
+
+    // manually dispose to free up memory, somehow this is not done automatically!
+    dialog.dispose();
   }
 
 }
@@ -763,6 +742,8 @@ class ErrorMessageDialog extends JajukDialog {
    * @param icon
    */
   ErrorMessageDialog(final int code, final String sInfoSup) {
+    super();
+
     final JOptionPane optionPane = UtilGUI.getNarrowOptionPane(72);
     optionPane.setMessage(UtilGUI.getLimitedMessage(Messages.getErrorMessage(code)
         + (sInfoSup != null ? (" : " + sInfoSup) : ""), 20));
@@ -774,8 +755,11 @@ class ErrorMessageDialog extends JajukDialog {
     // keep it modal (useful at startup)
     dialog.setModal(true);
     dialog.pack();
-    dialog.setLocationRelativeTo(JajukWindow.getInstance());
+    dialog.setLocationRelativeTo(JajukMainWindow.getInstance());
     dialog.setVisible(true);
+
+    // manually dispose to free up memory, somehow this is not done automatically!
+    dialog.dispose();
   }
 }
 
