@@ -1,6 +1,7 @@
 /*
  *  Jajuk
- *  Copyright (C) 2003-2008 The Jajuk Team
+ *  Copyright (C) 2003-2010 The Jajuk Team
+ *  http://jajuk.info
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -15,36 +16,48 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *  $Revision: 3132 $
+ *  $Revision$
  */
 package org.jajuk.util;
 
 import java.text.DateFormat;
+import java.text.ParseException;
 import java.text.SimpleDateFormat;
+import java.util.ArrayList;
 import java.util.Date;
+import java.util.List;
 import java.util.Locale;
 import java.util.Properties;
 import java.util.StringTokenizer;
 
+import org.apache.commons.lang.StringUtils;
+import org.jajuk.base.AlbumArtistManager;
 import org.jajuk.base.AlbumManager;
-import org.jajuk.base.AuthorManager;
+import org.jajuk.base.ArtistManager;
+import org.jajuk.base.File;
+import org.jajuk.base.GenreManager;
 import org.jajuk.base.PropertyMetaInformation;
-import org.jajuk.base.StyleManager;
 import org.jajuk.base.Track;
 import org.jajuk.util.error.JajukException;
 
 /**
- * Set of convenient classes for string manipulation
+ * Set of convenient classes for string manipulation.
  */
 public final class UtilString {
 
-  /**
-   * The list of characters that we need to escape in strings
-   */
+  /** The list of characters that we need to escape in strings. */
   private final static String ESCAPE_CHARACTERS = "\\[](){}.*+?$^|-";
 
+  /** Constant date FORMATTER, one by thread for perfs, we need an instance by thread because this class is not thread safe. */
+  private static final ThreadLocal<SimpleDateFormat> FORMATTER = new ThreadLocal<SimpleDateFormat>() {
+    @Override
+    protected SimpleDateFormat initialValue() {
+      return new SimpleDateFormat(Const.ADDITION_DATE_FORMAT, Locale.getDefault());
+    }
+  };
+
   /**
-   * private constructor to avoid instantiating utility class
+   * private constructor to avoid instantiating utility class.
    */
   private UtilString() {
   }
@@ -52,18 +65,18 @@ public final class UtilString {
   /**
    * Apply the Album pattern.
    * 
-   * @param file
-   *          file to apply pattern to
-   * @param sPattern
-   * @param bMandatory
-   *          are all needed tags mandatory ?
-   * @param normalize
-   * @param out
-   * @param track
-   * @return
-   * @throws JajukException
+   * @param file file to apply pattern to
+   * @param sPattern DOCUMENT_ME
+   * @param bMandatory are all needed tags mandatory ?
+   * @param normalize DOCUMENT_ME
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the string
+   * 
+   * @throws JajukException the jajuk exception
    */
-  static String applyAlbumPattern(final org.jajuk.base.File file, final String sPattern,
+  private static String applyAlbumPattern(final org.jajuk.base.File file, final String sPattern,
       final boolean bMandatory, final boolean normalize, final String out, final Track track)
       throws JajukException {
     String ret = out;
@@ -89,17 +102,17 @@ public final class UtilString {
   /**
    * Apply the Year pattern.
    * 
-   * @param file
-   *          file to apply pattern to
-   * @param sPattern
-   * @param bMandatory
-   *          are all needed tags mandatory ?
-   * @param out
-   * @param track
-   * @return
-   * @throws JajukException
+   * @param file file to apply pattern to
+   * @param sPattern DOCUMENT_ME
+   * @param bMandatory are all needed tags mandatory ?
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the string
+   * 
+   * @throws JajukException the jajuk exception
    */
-  static String applyYearPattern(final org.jajuk.base.File file, final String sPattern,
+  private static String applyYearPattern(final org.jajuk.base.File file, final String sPattern,
       final boolean bMandatory, final String out, final Track track) throws JajukException {
     String ret = out;
     if (sPattern.contains(Const.PATTERN_YEAR)) {
@@ -119,14 +132,15 @@ public final class UtilString {
   /**
    * Apply the Track pattern.
    * 
-   * @param sPattern
-   * @param normalize
-   * @param out
-   * @param track
-   * @return
+   * @param sPattern DOCUMENT_ME
+   * @param normalize DOCUMENT_ME
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the string
    */
-  static String applyTrackPattern(final String sPattern, final boolean normalize, final String out,
-      final Track track) {
+  private static String applyTrackPattern(final String sPattern, final boolean normalize,
+      final String out, final Track track) {
     String ret = out;
     String sValue;
     if (sPattern.contains(Const.PATTERN_TRACKNAME)) {
@@ -142,39 +156,22 @@ public final class UtilString {
   /**
    * Apply the Track Order pattern.
    * 
-   * @param file
-   *          file to apply pattern to
-   * @param sPattern
-   * @param bMandatory
-   *          are all needed tags mandatory ?
-   * @param out
-   * @param track
-   * @return
-   * @throws JajukException
+   * @param file file to apply pattern to
+   * @param sPattern DOCUMENT_ME
+   * @param bMandatory are all needed tags mandatory ?
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the string
+   * 
+   * @throws JajukException the jajuk exception
    */
-  static String applyTrackOrderPattern(final org.jajuk.base.File file, final String sPattern,
-      final boolean bMandatory, final String out, final Track track) throws JajukException {
+  private static String applyTrackOrderPattern(final org.jajuk.base.File file,
+      final String sPattern, final boolean bMandatory, final String out, final Track track)
+      throws JajukException {
     if (sPattern.contains(Const.PATTERN_TRACKORDER)) {
       // override Order from filename if not set explicitly
-      long lOrder = track.getOrder();
-      if (lOrder == 0) {
-        final String sFilename = file.getName();
-        if (Character.isDigit(sFilename.charAt(0))) {
-          final String sTo = sFilename.substring(0, 3).trim().replaceAll("[^0-9]", "");
-          for (final char c : sTo.toCharArray()) {
-            if (!Character.isDigit(c)) {
-              throw new JajukException(152, file.getAbsolutePath());
-            }
-          }
-          lOrder = Long.parseLong(sTo);
-        } else {
-          if (bMandatory) {
-            throw new JajukException(152, file.getAbsolutePath());
-          } else {
-            lOrder = 0;
-          }
-        }
-      }
+      long lOrder = handleOrder(file, bMandatory, track);
 
       // prepend one digit numbers with "0"
       if (lOrder < 10) {
@@ -188,36 +185,72 @@ public final class UtilString {
   }
 
   /**
-   * Apply the Style pattern.
+   * Handle order.
+   * DOCUMENT_ME
    * 
-   * @param file
-   *          file to apply pattern to
-   * @param sPattern
-   * @param bMandatory
-   *          are all needed tags mandatory ?
-   * @param normalize
-   * @param out
-   * @param track
-   * @return
-   * @throws JajukException
+   * @param file DOCUMENT_ME
+   * @param bMandatory DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the long
+   * 
+   * @throws JajukException the jajuk exception
    */
-  static String applyStylePattern(final org.jajuk.base.File file, final String sPattern,
+  private static long handleOrder(final org.jajuk.base.File file, final boolean bMandatory,
+      final Track track) throws JajukException {
+    long lOrder = track.getOrder();
+    if (lOrder == 0) {
+      final String sFilename = file.getName();
+      if (Character.isDigit(sFilename.charAt(0))) {
+        final String sTo = sFilename.substring(0, 3).trim().replaceAll("[^0-9]", "");
+        for (final char c : sTo.toCharArray()) {
+          if (!Character.isDigit(c)) {
+            throw new JajukException(152, file.getAbsolutePath());
+          }
+        }
+        lOrder = Long.parseLong(sTo);
+      } else {
+        if (bMandatory) {
+          throw new JajukException(152, file.getAbsolutePath());
+        } else {
+          lOrder = 0;
+        }
+      }
+    }
+    return lOrder;
+  }
+
+  /**
+   * Apply the Genre pattern.
+   * 
+   * @param file file to apply pattern to
+   * @param sPattern DOCUMENT_ME
+   * @param bMandatory are all needed tags mandatory ?
+   * @param normalize DOCUMENT_ME
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the string
+   * 
+   * @throws JajukException the jajuk exception
+   */
+  private static String applyGenrePattern(final org.jajuk.base.File file, final String sPattern,
       final boolean bMandatory, final boolean normalize, final String out, final Track track)
       throws JajukException {
     String ret = out;
     String sValue;
-    if (sPattern.contains(Const.PATTERN_STYLE)) {
-      sValue = track.getStyle().getName();
+    if (sPattern.contains(Const.PATTERN_GENRE)) {
+      sValue = track.getGenre().getName();
       if (normalize) {
         sValue = UtilSystem.getNormalizedFilename(sValue);
       }
-      if (!sValue.equals(Const.UNKNOWN_STYLE)) {
-        ret = ret.replace(Const.PATTERN_STYLE, StyleManager.format(sValue));
+      if (!sValue.equals(Const.UNKNOWN_GENRE)) {
+        ret = ret.replace(Const.PATTERN_GENRE, GenreManager.format(sValue));
       } else {
         if (bMandatory) {
           throw new JajukException(153, file.getAbsolutePath());
         } else {
-          ret = ret.replace(Const.PATTERN_STYLE, Messages.getString(Const.UNKNOWN_STYLE));
+          ret = ret.replace(Const.PATTERN_GENRE, Messages.getString(Const.UNKNOWN_GENRE));
         }
       }
     }
@@ -225,36 +258,38 @@ public final class UtilString {
   }
 
   /**
-   * Apply the Author pattern.
+   * Apply the Artist pattern.
    * 
-   * @param file
-   *          file to apply pattern to
-   * @param sPattern
-   * @param bMandatory
-   *          are all needed tags mandatory ?
-   * @param normalize
-   * @param out
-   * @param track
-   * @return
-   * @throws JajukException
+   * @param file file to apply pattern to
+   * @param sPattern DOCUMENT_ME
+   * @param bMandatory are all needed tags mandatory ?
+   * @param normalize DOCUMENT_ME
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the string
+   * 
+   * @throws JajukException the jajuk exception
    */
-  private static String applyAuthorPattern(final org.jajuk.base.File file, final String sPattern,
+  private static String applyArtistPattern(final org.jajuk.base.File file, final String sPattern,
       final boolean bMandatory, final boolean normalize, final String out, final Track track)
       throws JajukException {
     String ret = out;
     String sValue;
-    if (sPattern.contains(Const.PATTERN_AUTHOR)) {
-      sValue = track.getAuthor().getName();
+    if (sPattern.contains(Const.PATTERN_ARTIST)) {
+
+      sValue = track.getArtist().getName();
+
       if (normalize) {
         sValue = UtilSystem.getNormalizedFilename(sValue);
       }
-      if (!sValue.equals(Const.UNKNOWN_AUTHOR)) {
-        ret = ret.replaceAll(Const.PATTERN_AUTHOR, AuthorManager.format(sValue));
+      if (!sValue.equals(Const.UNKNOWN_ARTIST)) {
+        ret = ret.replaceAll(Const.PATTERN_ARTIST, ArtistManager.format(sValue));
       } else {
         if (bMandatory) {
           throw new JajukException(150, file.getAbsolutePath());
         } else {
-          ret = ret.replaceAll(Const.PATTERN_AUTHOR, Messages.getString(Const.UNKNOWN_AUTHOR));
+          ret = ret.replaceAll(Const.PATTERN_ARTIST, Messages.getString(Const.UNKNOWN_ARTIST));
         }
       }
     }
@@ -265,27 +300,30 @@ public final class UtilString {
    * Apply a pattern. This replaces certain patterns in the provided Pattern
    * with information from the file and returns the result.
    * 
-   * @param file
-   *          file to apply pattern to
-   * @param sPattern
-   * @param bMandatory
-   *          are all needed tags mandatory ?
+   * @param file file to apply pattern to
+   * @param sPattern DOCUMENT_ME
+   * @param bMandatory are all needed tags mandatory ?
+   * @param normalize DOCUMENT_ME
+   * 
    * @return computed string
-   * @return make sure the created string can be used as file name on target
-   *         file system
-   * @throws JajukException
-   *           if some tags are missing
+   * make sure the created string can be used as file name on target
+   * file system
+   * 
+   * @throws JajukException if some tags are missing
    */
   public static String applyPattern(final org.jajuk.base.File file, final String sPattern,
       final boolean bMandatory, final boolean normalize) throws JajukException {
     String out = sPattern;
     final Track track = file.getTrack();
 
-    // Check Author name
-    out = applyAuthorPattern(file, sPattern, bMandatory, normalize, out, track);
+    // Check Artist name
+    out = UtilString.applyArtistPattern(file, sPattern, bMandatory, normalize, out, track);
 
-    // Check Style name
-    out = UtilString.applyStylePattern(file, sPattern, bMandatory, normalize, out, track);
+    // Check Album artist, use artist name if no album artist
+    out = UtilString.applyAlbumArtistPattern(sPattern, normalize, out, track);
+
+    // Check Genre name
+    out = UtilString.applyGenrePattern(file, sPattern, bMandatory, normalize, out, track);
 
     // Check Album Name
     out = UtilString.applyAlbumPattern(file, sPattern, bMandatory, normalize, out, track);
@@ -299,12 +337,107 @@ public final class UtilString {
     // Check Year Value
     out = UtilString.applyYearPattern(file, sPattern, bMandatory, out, track);
 
+    // Check Disc Value
+    out = UtilString.applyDiscPattern(file, sPattern, bMandatory, out, track);
+
     return out;
   }
 
   /**
-   * @param s
-   *          String to analyse
+   * Apply album artist pattern.
+   * 
+   * @param normalize DOCUMENT_ME
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * @param sPattern DOCUMENT_ME
+   * 
+   * @return the string
+   */
+  private static String applyAlbumArtistPattern(String sPattern, boolean normalize, String out,
+      Track track) {
+    String ret = out;
+    String sValue;
+    if (sPattern.contains(Const.PATTERN_ALBUM_ARTIST)) {
+      sValue = track.getAlbumArtistOrArtist();
+      if (normalize) {
+        sValue = UtilSystem.getNormalizedFilename(sValue);
+      }
+      ret = ret.replaceAll(Const.PATTERN_ALBUM_ARTIST, AlbumArtistManager.format(sValue));
+    }
+    return ret;
+  }
+
+  /**
+   * Apply disc pattern.
+   * 
+   * @param file DOCUMENT_ME
+   * @param out DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * @param sPattern DOCUMENT_ME
+   * @param bMandatory DOCUMENT_ME
+   * 
+   * @return the string
+   * 
+   * @throws JajukException the jajuk exception
+   */
+  private static String applyDiscPattern(File file, String sPattern, boolean bMandatory,
+      String out, Track track) throws JajukException {
+    if (sPattern.contains(Const.PATTERN_DISC)) {
+      // override Order from filename if not set explicitly
+      long lDiscNumber = handleDiscNumber(file, bMandatory, track);
+
+      // prepend one digit numbers with "0"
+      if (lDiscNumber < 10) {
+        return out.replace(Const.PATTERN_DISC, "0" + lDiscNumber);
+      } else {
+        return out.replace(Const.PATTERN_DISC, lDiscNumber + "");
+      }
+    }
+
+    return out;
+  }
+
+  /**
+   * Handle disc number.
+   * DOCUMENT_ME
+   * 
+   * @param file DOCUMENT_ME
+   * @param bMandatory DOCUMENT_ME
+   * @param track DOCUMENT_ME
+   * 
+   * @return the long
+   * 
+   * @throws JajukException the jajuk exception
+   */
+  private static long handleDiscNumber(File file, boolean bMandatory, Track track)
+      throws JajukException {
+    long lDiscNumber = track.getDiscNumber();
+    if (lDiscNumber == 0) {
+      final String sFilename = file.getName();
+      if (Character.isDigit(sFilename.charAt(0))) {
+        final String sTo = sFilename.substring(0, 3).trim().replaceAll("[^0-9]", "");
+        for (final char c : sTo.toCharArray()) {
+          if (!Character.isDigit(c)) {
+            throw new JajukException(152, file.getAbsolutePath());
+          }
+        }
+        lDiscNumber = Long.parseLong(sTo);
+      } else {
+        if (bMandatory) {
+          throw new JajukException(152, file.getAbsolutePath());
+        } else {
+          lDiscNumber = 0;
+        }
+      }
+    }
+    return lDiscNumber;
+  }
+
+  /**
+   * Contains non digit or letters.
+   * 
+   * @param s String to analyse
+   * 
    * @return whether the given string contains non digit or letter characters
    */
   public static boolean containsNonDigitOrLetters(final String s) {
@@ -319,39 +452,28 @@ public final class UtilString {
   }
 
   /**
-   * Encode URLS
+   * Encode URLS.
    * 
-   * @param s
-   * @return
+   * @param s DOCUMENT_ME
+   * 
+   * @return the string
    */
   public static String encodeURL(final String s) {
     return s.replaceAll(" +", "+");
   }
 
-  /**
-   * Build the frame title from user option
-   * 
-   * @param file
-   *          played file
-   * @return built frame title
-   */
-  public static String buildTitle(final org.jajuk.base.File file) {
-    // We use trailing pattern to allow scripting like MSN plugins to
-    // detect jajuk frames and extract current track
-    String title = Conf.getString(Const.CONF_FRAME_TITLE_PATTERN);
-    title = title.replaceAll(Const.PATTERN_TRACKNAME, file.getTrack().getName());
-    title = title.replaceAll(Const.PATTERN_ALBUM, file.getTrack().getAlbum().getName2());
-    title = title.replaceAll(Const.PATTERN_AUTHOR, file.getTrack().getAuthor().getName2());
-    title = title.replaceAll(Const.PATTERN_STYLE, file.getTrack().getStyle().getName2());
-    title = title.replaceAll(Const.PATTERN_TRACKORDER, Long.toString(file.getTrack().getOrder()));
-    title = title.replaceAll(Const.PATTERN_YEAR, file.getTrack().getYear().getName2());
-    return title;
-  }
-
   /*
    * Escape (in the regexp sense) a string Source Search reserved: $ ( ) * + - . ? [ \ ] ^ { | }
    * http://mindprod.com/jgloss/regex.html
    */
+  /**
+   * Escape string.
+   * DOCUMENT_ME
+   * 
+   * @param s DOCUMENT_ME
+   * 
+   * @return the string
+   */
   public static String escapeString(String s) {
     int length = s.length();
     StringBuffer buffer = new StringBuffer(2 * length);
@@ -373,15 +495,14 @@ public final class UtilString {
   /**
    * Format an object to a string.
    * 
-   * @param sValue
-   * @param cType
-   * @param bHuman
-   *          is this string intended to be human-readable ?
-   * @return
-   * @throws Exception
+   * @param bHuman is this string intended to be human-readable ?
+   * @param oValue DOCUMENT_ME
+   * @param meta DOCUMENT_ME
+   * 
+   * @return the string
    */
   public static String format(final Object oValue, final PropertyMetaInformation meta,
-      final boolean bHuman) throws Exception {
+      final boolean bHuman) {
     final Class<?> cType = meta.getType();
     // default (works for strings, long and double)
     String sValue = oValue.toString();
@@ -398,45 +519,51 @@ public final class UtilString {
   }
 
   /**
-   * @return locale date formatter instance
+   * Gets the locale date formatter.
+   * 
+   * @return locale date FORMATTER instance
    */
   public static DateFormat getLocaleDateFormatter() {
     return DateFormat.getDateInstance(DateFormat.DEFAULT, Locale.getDefault());
   }
 
   /**
-   * Formatter for properties dialog window
+   * Formatter for properties dialog window.
    * 
-   * @param sDesc
-   * @return
+   * @param sDesc DOCUMENT_ME
+   * 
+   * @return the string
    */
   public static String formatPropertyDesc(final String sDesc) {
     return "<HTML><center><b><font size=+0 color=#000000>" + sDesc + "</font></b><HTML>";
   }
 
   /**
-   * format style: first letter uppercase and others lowercase
+   * format genre: first letter uppercase and others lowercase.
    * 
-   * @param style
-   * @return
+   * @param genre DOCUMENT_ME
+   * 
+   * @return the string
    */
-  public static String formatStyle(final String style) {
-    if (style.length() == 0) {
+  public static String formatGenre(final String genre) {
+    if (genre.length() == 0) {
       return "";
     }
-    if (style.length() == 1) {
-      return style.substring(0, 1).toUpperCase();
+    if (genre.length() == 1) {
+      return genre.substring(0, 1).toUpperCase(Locale.getDefault());
     }
-    String sOut = style.toLowerCase().substring(1);
-    sOut = style.substring(0, 1).toUpperCase() + sOut;
-    return sOut;
+
+    // construct string with first letter uppercase and rest lowercase
+    return genre.substring(0, 1).toUpperCase(Locale.getDefault())
+        + genre.toLowerCase(Locale.getDefault()).substring(1);
   }
 
   /**
-   * Performs some cleanups for strings comming from tag libs
+   * Performs some cleanups for strings comming from tag libs.
    * 
-   * @param s
-   * @return
+   * @param s DOCUMENT_ME
+   * 
+   * @return the string
    */
   public static String formatTag(final String s) {
     // we delete all non char characters to avoid parsing errors
@@ -451,7 +578,13 @@ public final class UtilString {
     return sb.toString().trim();
   }
 
-  /** Format a time from secs to a human readable format */
+  /**
+   * Format a time from secs to a human readable format.
+   * 
+   * @param lTime DOCUMENT_ME
+   * 
+   * @return the string
+   */
   public static String formatTimeBySec(final long lTime) {
     // Convert time to int for performance reasons
     int l = (int) lTime;
@@ -478,14 +611,15 @@ public final class UtilString {
    * see http://www.w3.org/TR/2000/REC-xml-20001006
    * <p>
    * substrings
-   * <p>' to &apos;
-   * <p>" to &quot;
-   * <p>< to &lt;
+   * <p> ' to &apos;
+   * <p> " to &quot;
+   * <p> < to &lt;
    * <p>> to &gt;
-   * <p>& to &amp;
+   * <p> & to &amp;
    * 
-   * @param s
-   * @return
+   * @param s DOCUMENT_ME
+   * 
+   * @return the string
    */
   public static String formatXML(final String s) {
     String sOut = replaceReservedXMLChars(s);
@@ -506,8 +640,11 @@ public final class UtilString {
   }
 
   /**
-   * @param s
-   * @return
+   * Replace reserved xml chars.
+   * 
+   * @param s DOCUMENT_ME
+   * 
+   * @return the string
    */
   private static String replaceReservedXMLChars(final String s) {
     String sOut = s;
@@ -530,13 +667,17 @@ public final class UtilString {
   }
 
   /**
-   * @return Addition date simple format instance
+   * Gets the addition date formatter.
+   * 
+   * @return Thread-safe addition date simple format instance
    */
   public static DateFormat getAdditionDateFormatter() {
-    return new SimpleDateFormat(Const.ADDITION_DATE_FORMAT);
+    return FORMATTER.get();
   }
 
   /**
+   * Gets the anonymized jajuk properties.
+   * 
    * @return Anonymized Jajuk properties (for log or quality agent)
    */
   public static Properties getAnonymizedJajukProperties() {
@@ -550,6 +691,8 @@ public final class UtilString {
   }
 
   /**
+   * Gets the anonymized system properties.
+   * 
    * @return Anonymized System properties (for log or quality agent)
    */
   public static Properties getAnonymizedSystemProperties() {
@@ -572,12 +715,11 @@ public final class UtilString {
   }
 
   /**
-   * Make sure to reduce a string to the given size
+   * Make sure to reduce a string to the given size.
+   * 
+   * @param sIn Input string, example: blabla
+   * @param iSize max size, example: 3
    * 
-   * @param sIn
-   *          Input string, exemple: blabla
-   * @param iSize
-   *          max size, exemple: 3
    * @return bla...
    */
   public static String getLimitedString(final String sIn, final int iSize) {
@@ -589,10 +731,12 @@ public final class UtilString {
   }
 
   /**
-   * @param ucs4char
-   *          char to test
+   * Checks if is char.
+   * 
+   * @param ucs4char char to test
+   * 
    * @return whether the char is valid, code taken from Apache sax
-   *         implementation
+   * implementation
    */
   public static boolean isChar(final int ucs4char) {
     return ((ucs4char >= 32) && (ucs4char <= 55295)) || (ucs4char == 10) || (ucs4char == 9)
@@ -601,43 +745,37 @@ public final class UtilString {
   }
 
   /**
+   * Checks if is xml valid.
+   * 
+   * @param s DOCUMENT_ME
    * 
-   * @param s
-   *          String to test
-   * @return whether the string is void or not
-   */
-  public static boolean isVoid(final String s) {
-    return (s == null) || s.trim().equals("");
-  }
-
-  /**
-   * @param s
    * @return whether given string is XML-valid
    */
   public static boolean isXMLValid(final String s) {
-    // check reserved chars
-    if (s.contains("&") || s.contains("\'") || s.contains("\"") || s.contains("<")
-        || s.contains(">")) {
-      return false;
-    }
     // check invalid chars
     for (int i = 0; i < s.length(); i++) {
       final char c = s.charAt(i);
+
+      // check reserved chars
+      if (-1 != "&\'\"<>".indexOf(c)) {
+        return false;
+      }
+
       if (!UtilString.isChar(c)) {
         return false;
       }
     }
+
     return true;
   }
 
   /**
-   * Pad an int with zeros
+   * Pad an int with zeros.
    * 
-   * @param l
-   *          the number to be padded
-   * @param size
-   *          the targeted size
-   * @return
+   * @param l the number to be padded
+   * @param size the targeted size
+   * 
+   * @return the string
    */
   public static String padNumber(final long l, final int size) {
     final StringBuilder sb = new StringBuilder(Long.toString(l));
@@ -648,25 +786,21 @@ public final class UtilString {
   }
 
   /**
-   * Parse a string to an object
+   * Parse a string to an object.
+   * 
+   * @param sValue DOCUMENT_ME
+   * @param cType DOCUMENT_ME
    * 
-   * @param sValue
-   * @param cType
    * @return parsed item
-   * @throws Exception
+   * 
+   * @throws ParseException the parse exception
+   * @throws ClassNotFoundException the class not found exception
    */
-  public static Object parse(final String sValue, final Class<?> cType) throws Exception {
+  public static Object parse(final String sValue, final Class<?> cType) throws ParseException,
+      ClassNotFoundException {
     Object oDefaultValue = sValue; // String by default
     if (cType.equals(Boolean.class)) {
-      // "y" and "n" is an old boolean
-      // attribute notation prior to 1.0
-      if (sValue.equals("y")) {
-        oDefaultValue = true;
-      } else if (sValue.equals("n")) {
-        oDefaultValue = false;
-      } else {
-        oDefaultValue = fastBooleanParser(sValue);
-      }
+      oDefaultValue = handleBoolean(sValue);
     } else if (cType.equals(Date.class)) {
       oDefaultValue = getAdditionDateFormatter().parseObject(sValue);
     } else if (cType.equals(Long.class)) {
@@ -679,15 +813,37 @@ public final class UtilString {
     return oDefaultValue;
   }
 
+  /**
+   * Handle boolean.
+   * DOCUMENT_ME
+   * 
+   * @param sValue DOCUMENT_ME
+   * 
+   * @return the boolean
+   */
+  private static Boolean handleBoolean(final String sValue) {
+    Boolean oValue;
+    // "y" and "n" is an old boolean
+    // attribute notation prior to 1.0
+    if ("y".equals(sValue)) {
+      oValue = true;
+    } else if ("n".equals(sValue)) {
+      oValue = false;
+    } else {
+      oValue = fastBooleanParser(sValue);
+    }
+    return oValue;
+  }
+
   /**
    * Fast long parser, low level check, replacement of Long.parseLong()
    * 
    * CAUTION : do not use if the value can be negative or you will get
    * unexpected results
    * 
-   * @param in
-   *          must be a set of digits with a size > 0 and be positive
-   * @return
+   * @param in must be a set of digits with a size > 0 and be positive
+   * 
+   * @return the long
    */
   public static long fastLongParser(String in) {
     int length = in.length();
@@ -706,9 +862,9 @@ public final class UtilString {
   /**
    * Fast Boolean parser, low level check, replacement of Boolean.parseBoolean()
    * 
-   * @param in
-   *          must be a string beginning by true or false (lower case)
-   * @return
+   * @param in must be a string beginning by true or false (lower case)
+   * 
+   * @return true, if fast boolean parser
    */
   public static boolean fastBooleanParser(String in) {
     return (in.charAt(0) == 't');
@@ -717,16 +873,16 @@ public final class UtilString {
   /**
    * Rot13 encode/decode,
    * <p>
-   * Thx
-   * http://www.idevelopment.info/data/Programming/java/security/java_cryptography_extension/rot13.java
+   * Thx http://www.idevelopment.info/data/Programming/java/security/
+   * java_cryptography_extension/rot13.java
    * </p>
    * 
-   * @param in
-   *          text to encode / decode in rote 13
+   * @param in text to encode / decode in rote 13
+   * 
    * @return encoded /decoded text
    */
   public static String rot13(final String in) {
-    if (UtilString.isVoid(in)) {
+    if (StringUtils.isBlank(in)) {
       return "";
     }
     int abyte = 0;
@@ -742,32 +898,16 @@ public final class UtilString {
   }
 
   /**
+   * Matches ignore case and order.
    * 
-   * @param s
-   *          string to be checked
-   * @return whether provided string is a number or not
-   */
-  public static boolean isNumber(String s) {
-    for (int i = 0; i < s.length(); i++) {
-      char c = s.charAt(i);
-      if ((c < '0') || (c > '9')) {
-        return false;
-      }
-    }
-    return true;
-  }
-
-  /**
+   * @param tested the string to be tested
+   * @param key the search criteria, can be several words separated by a space
    * 
-   * @param tested
-   *          the string to be tested
-   * @param key
-   *          the search criteria, can be several words separated by a space
    * @return whether the given tested string matches the key
    */
   public static boolean matchesIgnoreCaseAndOrder(final String tested, final String key) {
-    String testedLower = tested.toLowerCase();
-    String keyLower = key.toLowerCase();
+    String testedLower = tested.toLowerCase(Locale.getDefault());
+    String keyLower = key.toLowerCase(Locale.getDefault());
     StringTokenizer st = new StringTokenizer(testedLower, " ");
     while (st.hasMoreTokens()) {
       String token = st.nextToken();
@@ -781,11 +921,11 @@ public final class UtilString {
   /**
    * Encode a string to unicode representation (ie \\uxxxx\\uyyyyy...)
    * 
-   * @param in
-   *          string to encode
+   * @param in string to encode
+   * 
    * @return encoded string
    */
-  static public String encodeToUnicode(String in) {
+  public static String encodeToUnicode(String in) {
     StringBuilder sb = new StringBuilder(in.length() * 5);
     for (int i = 0; i < in.length(); i++) {
       char c = in.charAt(i);
@@ -798,16 +938,68 @@ public final class UtilString {
   }
 
   /**
-   * Convert byte to hexadecimal representation
+   * Convert byte to hexadecimal representation.
    * 
-   * @param b
-   * @return
+   * @param b DOCUMENT_ME
+   * 
+   * @return the string
    */
-  static public String byteToHex(byte b) {
+  public static String byteToHex(byte b) {
     char hexDigit[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e',
         'f' };
     char[] array = { hexDigit[(b >> 4) & 0x0f], hexDigit[b & 0x0f] };
     return new String(array);
   }
 
+  /**
+   * Returns a concatenation of argument array.
+   * 
+   * @param strings strings to be concatened
+   * 
+   * @return concatenation of given strings
+   */
+  public static String concat(Object... strings) {
+    StringBuilder sb = new StringBuilder();
+    for (Object element : strings) {
+      sb.append(element);
+    }
+    return sb.toString();
+  }
+
+  /**
+   * Code token from aTunes 1.14.0 *Copyright (C) 2006-2009 Alex Aranda, Sylvain
+   * Gaudard, Thomas Beckers and contributors Returns list of text between
+   * specified chars. Both chars are included in result elements. Returns empty
+   * list if chars are not found in string in given order For example given
+   * string "ab cd (ef) gh (ij)" and chars '(' and ')' will return a list with
+   * two strings: "(ef)" and "(ij)"
+   * 
+   * @param string DOCUMENT_ME
+   * @param beginChar DOCUMENT_ME
+   * @param endChar DOCUMENT_ME
+   * 
+   * @return the text between chars
+   */
+  public static final List<String> getTextBetweenChars(String string, char beginChar, char endChar) {
+    List<String> result = new ArrayList<String>();
+
+    if (string == null || string.indexOf(beginChar) == -1 || string.indexOf(endChar) == -1) {
+      return result;
+    }
+
+    String auxStr = string;
+    int beginIndex = auxStr.indexOf(beginChar);
+    int endIndex = auxStr.indexOf(endChar);
+    while (beginIndex != -1 && endIndex != -1) {
+      if (beginIndex < endIndex) {
+        result.add(auxStr.substring(beginIndex, endIndex + 1));
+      }
+      auxStr = auxStr.substring(endIndex + 1);
+      beginIndex = auxStr.indexOf(beginChar);
+      endIndex = auxStr.indexOf(endChar);
+    }
+
+    return result;
+  }
+
 }
